//Test Cloth Implementation

#if defined WIN32
#define WIN32_LEAN_AND_MEAN
#define NOMINMAX		//suppress windows' global min,max macros.
//#include <windows.h>	//needed for gl.h
#endif

#include <stdio.h>
#include "MyCloth.h"
#include "Stream.h"
#include <NxCooking.h>

#define TEAR_MEMORY_FACTOR 2


// -----------------------------------------------------------------------
//MyCloth::MyCloth(NxScene *scene, NxClothDesc &desc, struct _BBVERTEX* ThisVertex, int VertexCount, struct _BBTRIANGE* ThisTriangle, unsigned short int IndexCount)
MyCloth::MyCloth(NxScene *scene, NxClothDesc &desc, unsigned long *surf, float *entity, float *vbank,int *fbank,int MESH_NBVERTICES,int MESH_NBFACES)
{
	mInitDone = false;
	NxClothMeshDesc meshDesc;
	generateRegularMeshDesc(meshDesc, surf, entity,vbank, fbank, MESH_NBVERTICES, MESH_NBFACES);
	init(scene, desc, meshDesc);
}
/*MyCloth::MyCloth(NxScene *scene, NxClothDesc &desc, float *vbank,float *fbank,int MESH_NBVERTICES,int MESH_NBFACES)
{
	mInitDone = false;
	NxClothMeshDesc meshDesc;
	generateRegularMeshDescSpec(meshDesc, vbank, fbank, MESH_NBVERTICES, MESH_NBFACES);
	init(scene, desc, meshDesc);
}*/

// -----------------------------------------------------------------------
void MyCloth::init(NxScene *scene, NxClothDesc &desc, NxClothMeshDesc &meshDesc)
{
	mScene = scene;

	if (desc.flags & NX_CLF_TEARABLE)
	meshDesc.flags |= NX_CLOTH_MESH_TEARABLE;

	cookMesh(meshDesc);
	releaseMeshDescBuffers(meshDesc);

	allocateReceiveBuffers(meshDesc.numVertices, meshDesc.numTriangles);

	desc.clothMesh = mClothMesh;
	desc.meshData = mReceiveBuffers;
	mCloth = scene->createCloth(desc);
	mInitDone = true;
}

// -----------------------------------------------------------------------
MyCloth::~MyCloth()
{
	if (mInitDone) {
		mScene->releaseCloth(*mCloth);
		mScene->getPhysicsSDK().releaseClothMesh(*mClothMesh);
		releaseReceiveBuffers();
		delete[] FullBufferVertex;
		delete[] FullBufferTriangle;
	}
}


// -----------------------------------------------------------------------
void MyCloth::generateRegularMeshDesc(NxClothMeshDesc &desc, unsigned long *surf, float *entity, float *vbank, int *fbank,int MESH_NBVERTICES,int MESH_NBFACES)
{
if (surf)
{
	LPBBVERTEX ThisVertex;
	ThisVertex		= (struct _BBVERTEX *)*(surf+7);
	float *VertBufStart;
	float *VertBufEnd;
	VertBufStart	= (float*)*(surf+7);
	VertBufEnd		= (float*)*(surf+8);
	int VertexCount = (VertBufEnd-VertBufStart)/16;

	adresOldBuferVertexStart = (float*)*(surf+7);
	adresOldBuferVertexEnd	 = (float*)*(surf+8);

	LPBBTRIANGE ThisTriangle;
	ThisTriangle =	(struct _BBTRIANGE *)*(surf+11);
	unsigned short int *IndexBuf, *IndexBufEnd, IndexCount;
	IndexBuf	=	(unsigned short int *)*(surf+11);
	IndexBufEnd =	(unsigned short int *)*(surf+12);
	IndexCount	=	(IndexBufEnd-IndexBuf)/3;

	adresOldBuferTriangleStart	= (float*)*(surf+11);
	adresOldBuferTriangleEnd	= (float*)*(surf+12);

	FullBufferVertex	= new _BBVERTEX[4*VertexCount];
	FullBufferTriangle	= new _BBTRIANGE[IndexCount];

//============      ======
	desc.numVertices				= VertexCount;// - numFalseVert;
	desc.numTriangles				= IndexCount;
	desc.triangleStrideBytes		= 3*sizeof(NxU32);
	desc.pointStrideBytes			= 3*sizeof(float);
	desc.vertexMassStrideBytes		= sizeof(NxReal);
	desc.vertexFlagStrideBytes		= sizeof(NxU32);
	desc.triangles					= (NxU32*)malloc(sizeof(NxU32)*desc.numTriangles*3);
	desc.points						= (NxVec3*)malloc(sizeof(float)*desc.numVertices*3);
	desc.vertexMasses				= 0;
	desc.vertexFlags				= 0;
	desc.flags						= 0;


	int dV;

	NxVec3 *p = (NxVec3*)desc.points;
	for (int i = 0; i < VertexCount; i++) 
	{
		dV = PointerIndexBuffer[i].destV;
		p->set(ThisVertex[i].x, ThisVertex[i].y, ThisVertex[i].z); 
		p++;
	}

	NxU32 *id = (NxU32*)desc.triangles;
	for (int i = 0; i < IndexCount; i++) 
	{
		*id++ = ThisTriangle[i].a;
		*id++ = ThisTriangle[i].b;
		*id++ = ThisTriangle[i].c; 
	}
}
else
{
	desc.numVertices				= MESH_NBVERTICES;
	desc.numTriangles				= MESH_NBFACES;
	desc.triangleStrideBytes		= 3*sizeof(NxU32);
	desc.pointStrideBytes			= 3*sizeof(float);
	desc.vertexMassStrideBytes		= sizeof(NxReal);
	desc.vertexFlagStrideBytes		= sizeof(NxU32);
	desc.triangles					= (NxU32*)malloc(sizeof(NxU32)*desc.numTriangles*3);
	desc.points						= (NxVec3*)malloc(sizeof(float)*desc.numVertices*3);
	desc.vertexMasses				= 0;
	desc.vertexFlags				= 0;
	desc.flags						= 0;

	NxVec3 *p = (NxVec3*)desc.points;
	for (int i = 0; i < MESH_NBVERTICES; i++) 
	{
		p->set(vbank[i*3+0], vbank[i*3+1], vbank[i*3+2]); 
		p++;
	}

	NxU32 *id = (NxU32*)desc.triangles;
	for (int i = 0; i < MESH_NBFACES; i++) 
	{
		*id++ = fbank[i*3+0];
		*id++ = fbank[i*3+1];
		*id++ = fbank[i*3+2]; 
	}
}
}
/*void MyCloth::generateRegularMeshDescSpec(NxClothMeshDesc &desc, float *vbank,float *fbank,int MESH_NBVERTICES,int MESH_NBFACES)
{

//============      ======
	MESH_NBVERTICES = 441;
	MESH_NBFACES = 800;
	desc.numVertices				= MESH_NBVERTICES;
	desc.numTriangles				= MESH_NBFACES;
	desc.triangleStrideBytes		= 3*sizeof(NxU32);
	desc.pointStrideBytes			= 3*sizeof(float);
	desc.vertexMassStrideBytes		= sizeof(NxReal);
	desc.vertexFlagStrideBytes		= sizeof(NxU32);
	desc.triangles					= (NxU32*)malloc(sizeof(NxU32)*desc.numTriangles*3);
	desc.points						= (NxVec3*)malloc(sizeof(float)*desc.numVertices*3);
	desc.vertexMasses				= 0;
	desc.vertexFlags				= 0;
	desc.flags						= 0;

	NxVec3 *p = (NxVec3*)desc.points;
	for (int i = 0; i < MESH_NBVERTICES; i++) 
	{
		p->set(vbank[i*3+0], vbank[i*3+1], vbank[i*3+2]); 
		p++;
	}

	NxU32 *id = (NxU32*)desc.triangles;
	for (int i = 0; i < MESH_NBFACES; i++) 
	{
		*id++ = fbank[i*3+0];
		*id++ = fbank[i*3+1];
		*id++ = fbank[i*3+2]; 
	}
}*/

// -----------------------------------------------------------------------
void MyCloth::releaseMeshDescBuffers(const NxClothMeshDesc& desc)
{
	NxVec3* p = (NxVec3*)desc.points;
	NxU32* t = (NxU32*)desc.triangles;
	NxReal* m = (NxReal*)desc.vertexMasses;
	NxU32* f = (NxU32*)desc.vertexFlags;
	free(p);
	free(t);
	free(m);
	free(f);
}

// -----------------------------------------------------------------------
bool MyCloth::cookMesh(NxClothMeshDesc& desc)
{
	// we cook the mesh on the fly through a memory stream
	// we could also use a file stream and pre-cook the mesh
	MemoryWriteBuffer wb;
	if (!NxCookClothMesh(desc, wb)) 
		return false;

	MemoryReadBuffer rb(wb.data);
	mClothMesh = mScene->getPhysicsSDK().createClothMesh(rb);
	return true;
}

// -----------------------------------------------------------------------
void MyCloth::allocateReceiveBuffers(int numVertices, int numTriangles)
{
	// here we setup the buffers through which the SDK returns the dynamic cloth data
	// we reserve more memory for vertices than the initial mesh takes
	// because tearing creates new vertices
	// the SDK only tears cloth as long as there is room in these buffers
	NxU32 maxVertices = TEAR_MEMORY_FACTOR * numVertices;
	mReceiveBuffers.verticesPosBegin = (NxVec3*)malloc(sizeof(NxVec3)*maxVertices);
	mReceiveBuffers.verticesNormalBegin = (NxVec3*)malloc(sizeof(NxVec3)*maxVertices);
	mReceiveBuffers.verticesPosByteStride = sizeof(NxVec3);
	mReceiveBuffers.verticesNormalByteStride = sizeof(NxVec3);
	mReceiveBuffers.maxVertices = maxVertices;
	mReceiveBuffers.numVerticesPtr = (NxU32*)malloc(sizeof(NxU32));

	// the number of triangles is constant, even if the cloth is torn
	NxU32 maxIndices = 3*numTriangles;
	mReceiveBuffers.indicesBegin = (NxU32*)malloc(sizeof(NxU32)*maxIndices);
	mReceiveBuffers.indicesByteStride = sizeof(NxU32);
	mReceiveBuffers.maxIndices = maxIndices;
	mReceiveBuffers.numIndicesPtr = (NxU32*)malloc(sizeof(NxU32));

	// the parent index information would be needed if we used textured cloth
	NxU32 maxParentIndices = maxVertices;
	mReceiveBuffers.parentIndicesBegin = (NxU32*)malloc(sizeof(NxU32)*maxParentIndices);
	mReceiveBuffers.parentIndicesByteStride = sizeof(NxU32);
	mReceiveBuffers.maxParentIndices = maxParentIndices;
	mReceiveBuffers.numParentIndicesPtr = (NxU32*)malloc(sizeof(NxU32));

	// init the buffers in case we want to draw the mesh 
	// before the SDK as filled in the correct values
	*mReceiveBuffers.numVerticesPtr = 0;
	*mReceiveBuffers.numIndicesPtr = 0;
}

// -----------------------------------------------------------------------
void MyCloth::releaseReceiveBuffers()
{
	NxVec3* vp;
	NxU32* up; 
	vp = (NxVec3*)mReceiveBuffers.verticesPosBegin; free(vp);
	vp = (NxVec3*)mReceiveBuffers.verticesNormalBegin; free(vp);
	up = (NxU32*)mReceiveBuffers.numVerticesPtr; free(up);

	up = (NxU32*)mReceiveBuffers.indicesBegin; free(up);
	up = (NxU32*)mReceiveBuffers.numIndicesPtr; free(up);

	up = (NxU32*)mReceiveBuffers.parentIndicesBegin; free(up);
	up = (NxU32*)mReceiveBuffers.numParentIndicesPtr; free(up);
}
//------------------------------------------------------------------------
void MyCloth::updateTVertex(float* vbank)
{
	NxU32 numVertices = *mReceiveBuffers.numVerticesPtr;
	NxVec3* normals = (NxVec3*)mReceiveBuffers.verticesNormalBegin;
	NxVec3* vertices = (NxVec3*)mReceiveBuffers.verticesPosBegin;
	NxU32 numTriangles = *mReceiveBuffers.numIndicesPtr/3;
	NxU32* indices = (NxU32*)mReceiveBuffers.indicesBegin;
	NxU32* vertID = (NxU32*)mReceiveBuffers.parentIndicesBegin;
	
	for (NxU32 i=0; i<numVertices; i++)
	{
		vbank[i*3+0] = vertices[i].x;
		vbank[i*3+1] = vertices[i].y;
		vbank[i*3+2] = vertices[i].z;
	}
}
void MyCloth::updateTIndices(int* ibank)
{
	NxU32 numVertices = *mReceiveBuffers.numVerticesPtr;
	NxVec3* normals = (NxVec3*)mReceiveBuffers.verticesNormalBegin;
	NxVec3* vertices = (NxVec3*)mReceiveBuffers.verticesPosBegin;
	NxU32 numTriangles = *mReceiveBuffers.numIndicesPtr/3;
	NxU32* indices = (NxU32*)mReceiveBuffers.indicesBegin;
	NxU32* vertID = (NxU32*)mReceiveBuffers.parentIndicesBegin;
	
	for (NxU32 i=0; i<numTriangles; i++)
	{
		ibank[i*3+0] = indices[3*i+0];
		ibank[i*3+1] = indices[3*i+1];
		ibank[i*3+2] = indices[3*i+2];
	}
}
void MyCloth::updateTPIndices(int* ibank)
{
	NxU32 numVertices = *mReceiveBuffers.numVerticesPtr;
	NxVec3* normals = (NxVec3*)mReceiveBuffers.verticesNormalBegin;
	NxVec3* vertices = (NxVec3*)mReceiveBuffers.verticesPosBegin;
	NxU32 numTriangles = *mReceiveBuffers.numIndicesPtr/3;
	NxU32* indices = (NxU32*)mReceiveBuffers.indicesBegin;
	NxU32* vertID = (NxU32*)mReceiveBuffers.parentIndicesBegin;
	
	for (NxU32 i=0; i<numVertices; i++)
	{
		ibank[i] = vertID[i];
	}
}
void MyCloth::updateNormals(float* nbank)
{
	NxU32 numVertices = *mReceiveBuffers.numVerticesPtr;
	NxVec3* normals = (NxVec3*)mReceiveBuffers.verticesNormalBegin;
	NxVec3* vertices = (NxVec3*)mReceiveBuffers.verticesPosBegin;
	NxU32 numTriangles = *mReceiveBuffers.numIndicesPtr/3;
	NxU32* indices = (NxU32*)mReceiveBuffers.indicesBegin;
	NxU32* vertID = (NxU32*)mReceiveBuffers.parentIndicesBegin;
	
	for (NxU32 i=0; i<numVertices; i++)
	{

		nbank[i*3+0] = normals[i].x;
		nbank[i*3+1] = normals[i].y;
		nbank[i*3+2] = normals[i].z;

	}
}
int MyCloth::getNumVertices()
{
	NxU32 numVertices = *mReceiveBuffers.numVerticesPtr;
	return numVertices;
}
int MyCloth::getNumIndices()
{
	NxU32 numTriangles = *mReceiveBuffers.numIndicesPtr/3;
	return numTriangles;
}
//------------------------------------------------------------------------
int MyCloth::updateAllCloth(NxCloth* cloth, float* entity, unsigned long *surf, LPBBVERTEX OldVertex)
{
	NxU32 numVertices = *mReceiveBuffers.numVerticesPtr;
	NxVec3* normals = (NxVec3*)mReceiveBuffers.verticesNormalBegin;
	NxVec3* vertices = (NxVec3*)mReceiveBuffers.verticesPosBegin;
	NxU32 numTriangles = *mReceiveBuffers.numIndicesPtr/3;
	NxU32* indices = (NxU32*)mReceiveBuffers.indicesBegin;
	NxU32* vertID = (NxU32*)mReceiveBuffers.parentIndicesBegin;
	
	LPBBVERTEX ThisVertex;
	ThisVertex = (struct _BBVERTEX *)*(surf+7);
	float *VertBufStart;
	float *VertBufEnd;
	VertBufStart = (float*)*(surf+7);
	VertBufEnd = (float*)*(surf+8);
	int VertexCount = (VertBufEnd-VertBufStart)/16;

	//NxVec3 CoreSpeed = NxVec3(0,0,0);
	//if (coreActor) CoreSpeed = coreActor->getLinearVelocity();

	for (NxU32 i=0; i<numVertices; i++)
	{
		FullBufferVertex[i] = ThisVertex[vertID[i]];

		FullBufferVertex[i].x = vertices[i].x;// + CoreSpeed.x/30;
		FullBufferVertex[i].y = vertices[i].y;// + CoreSpeed.y/30;
		FullBufferVertex[i].z = vertices[i].z;// + CoreSpeed.z/30;

		FullBufferVertex[i].nx = normals[i].x;
		FullBufferVertex[i].ny = normals[i].y;
		FullBufferVertex[i].nz = normals[i].z;

		FullBufferVertex[i].u0 = OldVertex[vertID[i]].u0;
		FullBufferVertex[i].v0 = OldVertex[vertID[i]].v0;
	}
	for (NxU32 i=0; i<numTriangles; i++)
	{
		FullBufferTriangle[i].a = indices[3*i+0];
		FullBufferTriangle[i].b = indices[3*i+1];
		FullBufferTriangle[i].c = indices[3*i+2];
	}

	*(surf+7) = (unsigned long)FullBufferVertex;
	*(surf+8) = (unsigned long)FullBufferVertex + (unsigned long)(numVertices)*64;

	*(surf+11) = (unsigned long)FullBufferTriangle;
	*(surf+12) = (unsigned long)FullBufferTriangle + (unsigned long)(numTriangles)*6;

//	int moved = 1;
//	surf[16] = *((float*)(&moved));
	//surf[17] = *((int*)(&moved));
	*(surf+16)=0;
	*(surf+17)=0;
	
	return 1;
}

int MyCloth::updateAllUnitedCloth(NxCloth* cloth, float* entity, unsigned long *surf, LPBBVERTEX OldVertex)
{
	NxU32 numVertices = *mReceiveBuffers.numVerticesPtr;
	NxVec3* normals = (NxVec3*)mReceiveBuffers.verticesNormalBegin;
	NxVec3* vertices = (NxVec3*)mReceiveBuffers.verticesPosBegin;
	NxU32 numTriangles = *mReceiveBuffers.numIndicesPtr/3;
	NxU32* indices = (NxU32*)mReceiveBuffers.indicesBegin;
	NxU32* vertID = (NxU32*)mReceiveBuffers.parentIndicesBegin;
	
	LPBBVERTEX ThisVertex;
	ThisVertex = (struct _BBVERTEX *)*(surf+7);
	float *VertBufStart;
	float *VertBufEnd;
	VertBufStart = (float*)*(surf+7);
	VertBufEnd = (float*)*(surf+8);
	int VertexCount = (VertBufEnd-VertBufStart)/16;

	NxVec3 CoreSpeed = NxVec3(0,0,0);
	//if (coreActor) CoreSpeed = coreActor->getLinearVelocity();
	int sV, dV; //     

	for (NxU32 i=0; i<numVertices; i++)
	{
		sV = PointerIndexBuffer[i].sourceV;
		dV = PointerIndexBuffer[i].destV;

		FullBufferVertex[sV] = ThisVertex[vertID[dV]];

		FullBufferVertex[sV].x = vertices[dV].x;// + CoreSpeed.x/30;
		FullBufferVertex[sV].y = vertices[dV].y;// + CoreSpeed.y/30;
		FullBufferVertex[sV].z = vertices[dV].z;// + CoreSpeed.z/30

		FullBufferVertex[sV].nx = normals[dV].x;
		FullBufferVertex[sV].ny = normals[dV].y;
		FullBufferVertex[sV].nz = normals[dV].z;

		FullBufferVertex[sV].u0 = OldVertex[vertID[dV]].u0;
		FullBufferVertex[sV].v0 = OldVertex[vertID[dV]].v0;
	}
	for (NxU32 i=0; i<numTriangles; i++)
	{
		FullBufferTriangle[i].a = indices[3*i+0];
		FullBufferTriangle[i].b = indices[3*i+1];
		FullBufferTriangle[i].c = indices[3*i+2];
	}

	*(surf+7) = (unsigned long)FullBufferVertex;
	*(surf+8) = (unsigned long)FullBufferVertex + (unsigned long)(numVertices)*64;

	*(surf+11) = (unsigned long)FullBufferTriangle;
	*(surf+12) = (unsigned long)FullBufferTriangle + (unsigned long)(numTriangles)*6;

//	int moved = 1;
//	surf[16] = *((float*)(&moved));
	//surf[17] = *((int*)(&moved));
	*(surf+16)=0;
	*(surf+17)=0;
	
	return 1;
}

