// ===============================================================================
//						  AGEIA PHYSX SDK TRAINING PROGRAMS
//					            PARTICLE SYSTEM CLASS
//
//						  Written by Sam Thompson, 3-17-05
//						    Edited by Bob Schade, 5-1-06
// ===============================================================================
#ifndef PARTICLE_H
#define PARTICLE_H

#include <NxActor.h>
#include <NxSphereShape.h>
#include <NxSphereShapeDesc.h>
#include <NxScene.h>
#include <stdio.h>

const int ciMaxParticles = 10256;
const NxReal cfTimeBetweenParticles = 0.0005f;

const NxCollisionGroup cParticleCollisionGroup = 31;

// Make a simple particle class
class Particle 
{
	public:
		Particle();
		Particle( NxActor* pToSet, NxVec3& vThermoDynamicAcceleration );
		void update( NxReal fDeltaTime, NxReal fMaxRadius, NxReal fGrowthRate );
		void setActor( NxActor* pToSet );
		NxActor& getActorReference( );
//	private:
		NxActor* _pActor;
		float* pEntity;
		NxVec3 _vThermoDynamicForce;
		int _pIndex;
};

class ParticleEmitter 
{
	public:
		ParticleEmitter(NxScene* pSceneToSet, NxVec3 vStartingPos, NxReal fStartingVelScale, NxVec3 vThermoDynamicAcceleration);
		~ParticleEmitter();
		void removeAllParticles();
		void update(NxReal fDeltaTime);
		Particle* addParticle();
		float* removeParticle();
		void removeOneParticle(Particle* particle);
	//private:
		NxScene* _pScene;
		Particle* _aParticleArray[ciMaxParticles];
		NxQuat _peOrientation;
		NxVec3 _pRotation;
		int _iHeadIndex;
		int _iTailIndex;
		int _iParticleCount;
		NxVec3 _vStartingPos;
		NxReal _fNewParticleTimer;
		NxVec3 _vThermoDynamicAcceleration;
		NxReal _pScaleRadius;
		NxReal _pScaleStartVelmin;
		NxReal _pScaleStartVelmax;
		NxReal _pMass;
		NxReal _pADamping;
		NxReal _pLDamping;
		NxReal _pRadius;
		NxReal _fGrowthRate;
		NxReal _fMaxRadius;
};

#endif
