// ******************************************************
// *                                                    *
// *                 XorsTeam comments                  *
// *                                                    *
// * This is a source code of PhysX Wrapper for Blitz3d *
// *      and Xors3d. This library was developed by     *
// *     Andry Gutyria aka Render (render@bigmir.net)   *
// *                                                    *
// *   You can use this source code in commercial and   *
// *             non-commercial projects.               *
// *                                                    *
// *        XorsTeam was authorised to publish          *
// *                 this source code.                  *
// *                                                    *
// ******************************************************


#define _MySuperDLL
#define NOMINMAX
#include <windows.h>

#include <stdio.h>
#include <vector>
#include <algorithm>
// Physics code
#undef random

#include <NxPhysics.h>
#include "sources/ErrorStream.h"
#include "sources/UserAllocator.h"

//#include "DebugRenderer.h"
#include "sources/cooking.h"
#include "sources/Stream.h"
#include <NxCooking.h>
#include "sources/MyCloth.h"
#include <NxMath.h>
#include "sources/NxWheel.h"
#include "sources/UserData.h"
#include "sources/Particle.h" 

//--- Soft Bodies---
#include "sources/MySoftBody.h"
#include "sources/ObjMesh.h"

//---Vehicle---
#include "sources/NxVehicle.h"
#include "sources/NxAllVehicles.h"
//#include "reggg.h"
//#include "Afx.h"
#define FATAL_ERRORS
#include <time.h>
#include <string>

//#ifdef NX_DISABLE_HARDWARE
bool              gHardwareSimulation = false;
//#else
//bool              gHardwareSimulation = true;
//#endif
	
static UserAllocator*	gMyAllocator = NULL;
static ErrorStream*		gErrorStream = NULL;

static int				gPause = false;
static NxPhysicsSDK*	gPhysicsSDK = NULL;
static NxScene*			gScene = NULL;
static NxVec3			gDefaultGravity(0.0f, -9.81f, 0.0f);
//static ErrorStream		gErrorStream;

NxArray<MyCloth *> gCloths = 0;
NxArray<NxWheelShape *> gWheels = 0;
static NxArray<NxI32>	gNbTouchedBodies;
static NxI32			gNbTriggers = 0;
static int				gTrigersActive;

 int number = 0;
 int bah = 0;
 static int cubici = 0;
 static int scubici = 0;
 NxActor* test_max = 0;


//NxActor* actors[1000];
//float * mesh_actors[1000];
//int number_add_actor=0;

float angle, x,y,z;
NxQuat q;
NxShape*const* shapes;

float SB_Link_Radius = 0.01f;


//================ContactReport================
static NxActor*			global_plane = NULL;
static NxU32			gNbReports = 0;
 
//static NxActor* gTerrain = NULL;
static int MAXCONTACTS = 50;
int MAXTRIGGERACTORS = 100;

NxReal gTime;
NxReal gDeltaTime;
NxReal gLastTime;
NxReal renderDeltaTime;


#include <math.h>
#include <string.h>
#include <stdlib.h>

#define Wheel_Shape 3 

#define BBDECL extern "C" __declspec(dllexport)
#define BBCALL _stdcall

typedef struct tagCONTACTDATA
{
	NxVec3 pos;
	NxVec3 normal;
	NxVec3 nforce;
	NxVec3 tforce;
	NxActor* body;
	NxShape* shapes;
} contactdata;
typedef struct pxKepDATA
{
	NxVec3 pos;
	NxVec3 locpos;
	float buoyancy;
	float lindamp, angdamp;
	float maxdistance;
	float radius;
	NxActor* entity;

}KepDATA;
typedef struct tagBODYDATA
{
	int BlitzData;
	int StaticBody;
	float *Entity;

	int nContacts;
	contactdata Contacts[50];
	int contactEvent;

	KepDATA* Keps[50];
	int nkeps;

	int contacttable, magnitable, triggertable;
	int magnetMask;

	std::vector<NxActor*> dTriggerActors;
	//NxActor* triggerActors[200];
	NxActor* Trigger;
	int numTriggerActors;
	int trigerControl;
	NxTriggerFlag triggerFlag;

}BodyData;
typedef struct tagSHAPEDATA
{
	NxVec3 bpos;

}ShapeData;
static void BodyAddUserData(NxActor* body)
{
	BodyData* data = new BodyData;
	data->nContacts = 0;
	data->StaticBody = 0;
	data->contacttable = 1;
	data->magnitable = 1;
	data->magnetMask = 1;
	data->triggertable = 1;
	data->nkeps = 0;
	data->numTriggerActors = 0;
	data->trigerControl = 0;
	data->Trigger = NULL;
	body->userData = data;

//	#ifdef TEST_SPECIFIC_SHAPES
//	if(global_plane && body)
//	{
//		gScene->setActorPairFlags(*body, *global_plane, NX_NOTIFY_ON_START_TOUCH|NX_NOTIFY_ON_TOUCH|NX_NOTIFY_ON_END_TOUCH|NX_NOTIFY_FORCES);
//	}
//	#endif
}
static void StaticBodyAddUserData(NxActor* body)
{
	BodyData* data = new BodyData;
	data->StaticBody = 1;
	data->contacttable = 1;
	data->magnitable = 0;
	data->magnetMask = 0;
	data->triggertable = 1;
	data->nkeps = 0;
	data->numTriggerActors = 0;
	data->trigerControl = 0;
	data->Trigger = NULL;
	body->userData = data;

	#ifdef TEST_SPECIFIC_SHAPES
	if(global_plane && body)
	{
		//gScene->setActorPairFlags(*body, *global_plane, NX_NOTIFY_ON_START_TOUCH|NX_NOTIFY_ON_TOUCH|NX_NOTIFY_ON_END_TOUCH|NX_NOTIFY_FORCES);
	}
	#endif
}
static void ShapeAddUserData(NxActor* body)
{
	NxShape*const* AllShapes = body->getShapes();
	int nbAllShapes = body->getNbShapes();
	for (int i=0; i<nbAllShapes; i++)
	{
		ShapeData* data = new ShapeData;
		data->bpos = AllShapes[i]->getGlobalPosition() - body->getGlobalPosition();
		AllShapes[i]->userData = data;
	}
}
static void mainSetMask(NxActor* body)
{
	NxGroupsMask GrMask;
	GrMask.bits0 = 1;
	GrMask.bits1 = 1;
	GrMask.bits2 = 1;
	GrMask.bits3 = 1;
	NxShape*const* AllShapes = body->getShapes();
	int nbAllShapes = body->getNbShapes();
	for (int i=0; i<nbAllShapes; i++)
	{
		AllShapes[i]->setGroupsMask(GrMask);
	}
}
void ShowError(const char * errMsg, const char * func = "Unknow", const char * file = "Unknow", int line = -1)
{
	char tmpbuf[128];
	char tmpbuf2[128];
	_strtime(tmpbuf);
	_strdate(tmpbuf2);
	char errTime[128];
    sprintf(errTime, "%s %s", tmpbuf, tmpbuf2);
	std::string errorMsg;
	errorMsg += "Error has occured at ";
	errorMsg += errTime;
	char l[10];
	if(line == -1)
		{sprintf(l, "Unknow");}
	else
		{sprintf(l, "%i", line);}
	errorMsg  = errorMsg + "\nFile name: " + file + "\nLine:          " + l;
	errorMsg  = errorMsg + "\nFunction:   " + func;
	errorMsg  = errorMsg + "\nError message:\n\n" + errMsg;
//	errorMsg  = errorMsg + "\n\nSee log file for more information.";
	#ifdef FATAL_ERRORS
//		errorMsg += "\nApplication now will terminated!";
	#else
		errorMsg += "\nWARNING! Fatal errors is disabled, and application will not terminated!";
	#endif
	#ifndef NO_ERRORS
		MessageBox(NULL, errorMsg.c_str(), "PhysX::ERROR", MB_OK | MB_ICONERROR);
	#endif
	#ifdef FATAL_ERRORS
	#ifndef NO_ERRORS
		exit(0);
	#endif
	#endif
}
BBDECL void BBCALL pxDriverSetMode(int mode)
{
	if (mode==1)
	{
	#define PX_DRIVER_SET 1
	}
}
BBDECL NxActor* BBCALL pxBodyCreatePlane(float x, float y, float z)
{
	NxPlaneShapeDesc PlaneDesc;
	NxActorDesc ActorDesc;
	ActorDesc.shapes.pushBack(&PlaneDesc);
	NxActor* body = gScene->createActor(ActorDesc);
	body->setGlobalPosition(NxVec3(x,y,z));
	mainSetMask(body);
	BodyData* data = new BodyData;
	data->contacttable = 1;
	data->StaticBody = 1;
	body->userData = data;

	return body;
}
BBDECL NxActor* BBCALL pxBodyCreateCube(float dx, float dy, float dz, float mass)
{
	NxBoxShapeDesc BoxDesc;
	BoxDesc.dimensions = NxVec3(dx, dy, dz);
	if (cubici==1) 
	{
		ShowError("This is Demo version", "pxCreateBody", __FILE__ , __LINE__);
		return 0;
	}
	cubici = cubici - 1;

	NxActorDesc ActorDesc;
	ActorDesc.shapes.pushBack(&BoxDesc);
		
	if (mass)
	{
		NxBodyDesc BodyDesc;
		BodyDesc.mass = mass;
		ActorDesc.body			= &BodyDesc;
	}

	NxActor* body = gScene->createActor(ActorDesc);
	mainSetMask(body);
	if (mass) BodyAddUserData(body);
	if (!mass) StaticBodyAddUserData(body);
	if (!mass) ShapeAddUserData(body);
	
	return body;
}
BBDECL NxActor* BBCALL pxBodyCreateSphere(float radius, float mass)
{
	NxSphereShapeDesc SphereDesc;
	SphereDesc.radius		= radius;
	SphereDesc.groupsMask.bits0 = 0;
	
	if (scubici==10) 
	{
		ShowError("This is Demo version", "pxCreateBody", __FILE__ , __LINE__);
		return 0;
	}
	scubici = scubici - 2;

	NxActorDesc ActorDesc;
	ActorDesc.shapes.pushBack(&SphereDesc);
	if (mass)
	{
		NxBodyDesc BodyDesc;
		BodyDesc.mass = mass;
		ActorDesc.body			= &BodyDesc;
		//ActorDesc.density		= dens;
	}

	NxActor* body = gScene->createActor(ActorDesc);
	mainSetMask(body);
	if (mass) BodyAddUserData(body);
	if (!mass) StaticBodyAddUserData(body);
	if (!mass) ShapeAddUserData(body);

	return body;
}
BBDECL NxActor* BBCALL pxBodyCreateCylinder(float radius, float height, int nbEdge, float mass)
{
	//NxBodyDesc BodyDesc;
	NxActor* body;
	NxActorDesc ActorDesc;
	NxConvexShapeDesc ConvShapeDesc;
	
	int nbVerts = nbEdge;
	float alpha;
	float delta = 2*3.14f/nbVerts;
	float z=0;
	NxArray<NxVec3> points;

	if (radius == height) radius = radius - radius*0.01f;
	int parnost = nbEdge/2;
	for (int j=0; j<2; j++)
	{
		z = 2*height*j - height;
		if (nbEdge == 2*parnost) alpha = 3.14f/nbVerts+3.14f/4;
		if (nbEdge != 2*parnost)alpha = 3.14f/nbVerts-3.14f/2;
		for(int i=0; i<nbVerts; i++)
		{			
			points.pushBack().set(radius*cos(alpha), z, radius*sin(alpha));
			alpha = alpha + delta;
		}
	}
	
		NxConvexMeshDesc convexMesh;
		convexMesh.numVertices = points.size();
		convexMesh.points = &(points[0].x);
		convexMesh.pointStrideBytes = sizeof(NxVec3);
		convexMesh.flags = NX_CF_COMPUTE_CONVEX;

		MemoryWriteBuffer buf;
		bool status = NxCookConvexMesh(convexMesh, buf);
		if(status)
		{
			ConvShapeDesc.meshData = gPhysicsSDK->createConvexMesh(MemoryReadBuffer(buf.data));
			ActorDesc.shapes.pushBack(&ConvShapeDesc);
		}
			
			if (mass)
			{
				NxBodyDesc BodyDesc;
				BodyDesc.mass = mass;
				ActorDesc.body			= &BodyDesc;
			}

			body = gScene->createActor(ActorDesc);
			mainSetMask(body);
			if (mass) BodyAddUserData(body);
			if (!mass) StaticBodyAddUserData(body);
			if (!mass) ShapeAddUserData(body);

	return body;
}
BBDECL NxActor* BBCALL pxBodyCreateHull(float *vbank, int nvert, float mass)
{
	if (cubici==1) 
	{
		ShowError("This is Demo version", "pxCreateBody", __FILE__ , __LINE__);
		return 0;
	}
	cubici = cubici - 1;

	NxActor* body;
	
	int nbVerts=nvert;
	NxVec3* verts = new NxVec3[nbVerts];
	
	for(int i=0;i<nbVerts;i++)
	{
		verts[i].x = vbank[i*3+0];
		verts[i].y = vbank[i*3+1];
		verts[i].z = vbank[i*3+2];
	}

	NxU32 vertCount = nbVerts;
	
	NxConvexMeshDesc convexDesc;
	convexDesc.numVertices			= nbVerts;
	convexDesc.pointStrideBytes		= sizeof(NxVec3);
	convexDesc.points				= verts;
	convexDesc.flags				= NX_CF_COMPUTE_CONVEX;
	

	MemoryWriteBuffer buf;
	if(CookConvexMesh(convexDesc, buf))
	{

		NxConvexShapeDesc convexShapeDesc;
		convexShapeDesc.meshData = gPhysicsSDK->createConvexMesh(MemoryReadBuffer(buf.data));

		if(convexShapeDesc.meshData)
		{
			NxActorDesc ActorDesc;
			ActorDesc.shapes.pushBack(&convexShapeDesc);
			ActorDesc.globalPose.t  = NxVec3(0,0,0);
			if (mass)
			{
				NxBodyDesc BodyDesc;
				BodyDesc.mass = mass;
				ActorDesc.body			= &BodyDesc;
			}

			body = gScene->createActor(ActorDesc);
			mainSetMask(body);
			if (mass) BodyAddUserData(body);
			if (!mass) StaticBodyAddUserData(body);
			if (!mass) ShapeAddUserData(body);

			delete[] verts;
			return body;
		}
	}
	
	return 0;

}
BBDECL NxActor* BBCALL pxBodyCreateHullFromSSM(float* entity, float mass)
{

	int* SurfaceInfo  = ((int**)entity)[161];
	int* surf_array = ((int**)SurfaceInfo)[20];
	int count_sur = (SurfaceInfo[21] - SurfaceInfo[20])/4;

	LPBBVERTEX ThisVertex;
	int VertexCount=0;
	float* VertBufStart;
	float* VertBufEnd;
	int* surf;

	for (int i=0; i<count_sur; i++)
	{
		surf = ((int**)surf_array)[i];
		ThisVertex = (struct _BBVERTEX *)*(surf+7);
		VertBufStart = (float*)*(surf+7);
		VertBufEnd = (float*)*(surf+8);
		VertexCount = VertexCount + (VertBufEnd-VertBufStart)/16;
	}

	NxActor* body;
	
	NxVec3* verts = new NxVec3[VertexCount];
		for(int i=0;i<VertexCount;i++)
		{
			verts[i].x = ThisVertex[i].x;
			verts[i].y = ThisVertex[i].y;
			verts[i].z = ThisVertex[i].z;
		}
	
	NxConvexMeshDesc convexDesc;
	convexDesc.numVertices			= VertexCount;
	convexDesc.pointStrideBytes		= sizeof(NxVec3);
	convexDesc.points				= verts;
	convexDesc.flags				= NX_CF_COMPUTE_CONVEX;
	

	MemoryWriteBuffer buf;
	if(CookConvexMesh(convexDesc, buf))
	{

		NxConvexShapeDesc convexShapeDesc;
		convexShapeDesc.meshData = gPhysicsSDK->createConvexMesh(MemoryReadBuffer(buf.data));

		if(convexShapeDesc.meshData)
		{
			NxActorDesc ActorDesc;
			ActorDesc.shapes.pushBack(&convexShapeDesc);
			ActorDesc.globalPose.t  = NxVec3(0,0,0);
			if (mass)
			{
				NxBodyDesc BodyDesc;
				BodyDesc.mass = mass;
				ActorDesc.body			= &BodyDesc;
			}

			body = gScene->createActor(ActorDesc);
			mainSetMask(body);
			if (mass) BodyAddUserData(body);
			if (!mass) StaticBodyAddUserData(body);
			if (!mass) ShapeAddUserData(body);

			delete[] verts;
			return body;
		}
	}
	
	return 0;

}
BBDECL NxActor* BBCALL pxBodyCreateCapsule(float height, float radius, float mass)
{

	NxActorDesc ActorDesc;
	//NxBodyDesc BodyDesc;

	NxCapsuleShapeDesc CapsuleDesc;
	
	CapsuleDesc.height = height;
	CapsuleDesc.radius = radius;
	ActorDesc.shapes.pushBack(&CapsuleDesc);
	if (mass)
	{
		NxBodyDesc BodyDesc;
		BodyDesc.mass = mass;
		BodyDesc.mass = mass;
		ActorDesc.body			= &BodyDesc;
		//ActorDesc.density		= dens;
	}


	NxActor* body = gScene->createActor(ActorDesc);
	mainSetMask(body);
	//if (mass){body->setMass(mass);}
	if (mass) BodyAddUserData(body);
	if (!mass) StaticBodyAddUserData(body);
	if (!mass) ShapeAddUserData(body);

	return body;
}
//----------------Irr-------------
BBDECL void BBCALL pxIrrGetPosMat(NxActor* body, float* irrMat) 
{
	NxMat34 pose = body->getGlobalPose(); 

	const NxVec3 pos = pose.t;
	const NxMat33 orient = pose.M;
	orient.getColumnMajorStride4(&irrMat[0]);
	pos.get(&irrMat[12]);
	irrMat[3] = irrMat[7] = irrMat[11] = 0.0f;
	irrMat[15] = 1.0f;
}
BBDECL float BBCALL pxIrrGetPosElem(NxActor* body)
{
	NxMat34 pose = body->getGlobalPose();

	const NxVec3 pos = pose.t;
	float elem;
	pos.get(&elem);
	return elem;
}
BBDECL float BBCALL pxIrrGetRotElem(NxActor* body)
{
	NxMat34 pose = body->getGlobalPose();

	const NxMat33 orient = pose.M;
	float elem;
	orient.getColumnMajorStride4(&elem);
	return elem;
}
//----------------Triangle Mesh-------------
static NxU32 getFileSize(const char* name)
{
	#ifndef SEEK_END
	#define SEEK_END 2
	#endif
	
	FILE* File = fopen(name, "rb");
	if(!File)
		return 0;

	fseek(File, 0, SEEK_END);
	NxU32 eof_ftell = ftell(File);
	fclose(File);
	return eof_ftell;
}
BBDECL NxTriangleMesh* BBCALL pxCreateTriMeshPmap(float *vbank,float *fbank,int MESH_NBVERTICES,int MESH_NBFACES, const char* file_name, int pMap)
{
	// Initialize PMap
	NxPMap MeshPMap;
	MeshPMap.dataSize	= 0;
	MeshPMap.data		= NULL;
	//const char* name = "mainmap.pmap";

	NxVec3* gVertices = new NxVec3[MESH_NBVERTICES];
	NxVec3* gTriangles = new NxVec3[MESH_NBFACES];
	
	for(int i=0;i<MESH_NBVERTICES;i++)
	{
		gVertices[i].x = vbank[i*3+0];
		gVertices[i].y = vbank[i*3+1];
		gVertices[i].z = vbank[i*3+2];
	}
	for(int i=0;i<MESH_NBFACES;i++)
	{
		gTriangles[i].x = fbank[i*3+0];
		gTriangles[i].y = fbank[i*3+1];
		gTriangles[i].z = fbank[i*3+2];
	}

	NxTriangleMeshDesc meshDesc;
	meshDesc.numVertices                = MESH_NBVERTICES;
	meshDesc.numTriangles               = MESH_NBFACES;
	meshDesc.pointStrideBytes           = sizeof(NxVec3);
	meshDesc.triangleStrideBytes        = 3*sizeof(NxU32);
	meshDesc.points						= gVertices;
	meshDesc.triangles                  = gTriangles;                           
	meshDesc.flags                      = 0;
	

	bool status = InitCooking();
	if (!status) {
		printf("Unable to initialize the cooking library. Please make sure that you have correctly installed the latest version of the AGEIA PhysX SDK.");
		exit(1);
	}
	MemoryWriteBuffer buf;
	status = CookTriangleMesh(meshDesc, buf);
	if (!status) {
		printf("Unable to cook a triangle mesh.");
		exit(1);
	}
	MemoryReadBuffer readBuffer(buf.data);
	NxTriangleMesh* triangleMesh = gPhysicsSDK->createTriangleMesh(readBuffer);

	FILE* fp = fopen(file_name, "rb");
	if(!fp)
		{
		if(CreatePMap(MeshPMap, *triangleMesh, pMap))
			{
			// The pmap has successfully been created, save it to disk for later use
//			char PMapFilename[512];
//			GetTempFilePath(PMapFilename);
//			strcat(PMapFilename, "file_name");		
			fp = fopen(file_name, "wb+");
			if(fp)
				{
				fwrite(MeshPMap.data, MeshPMap.dataSize, 1, fp);
				fclose(fp);
				fp = NULL;
				}

			//assign pmap to mesh
			triangleMesh->loadPMap(MeshPMap);

			// sdk created data => sdk deletes it
			ReleasePMap(MeshPMap);
			}
		printf("...done\n");
		}
	else
		{
		// Found pmap file
		MeshPMap.dataSize	= getFileSize(file_name);
		MeshPMap.data		= new NxU8[MeshPMap.dataSize];
		fread(MeshPMap.data, MeshPMap.dataSize, 1, fp);
		fclose(fp);

		//assign pmap to mesh
		triangleMesh->loadPMap(MeshPMap);

		//we created data => we delete it
		delete MeshPMap.data;
		}
	delete[] gVertices;
	delete[] gTriangles;
	
	return triangleMesh;
}
BBDECL NxActor* BBCALL pxTestTriMesh(unsigned long* surf, float mass)
{
//	int* SurfaceInfo  = ((int**)entity)[161];
//	int* surf_array = ((int**)SurfaceInfo)[20];
//	int count_sur = (SurfaceInfo[21] - SurfaceInfo[20])/4;

	LPBBVERTEX ThisVertex;
	ThisVertex = (struct _BBVERTEX *)*(surf+7);
	float *VertBufStart;
	float *VertBufEnd;
	VertBufStart = (float*)*(surf+7);
	VertBufEnd = (float*)*(surf+8);
	int VertexCount = (VertBufEnd-VertBufStart)/16;

	LPBBTRIANGE ThisTriangle;
	ThisTriangle = (struct _BBTRIANGE *)*(surf+11);
	unsigned short int *IndexBuf, *IndexBufEnd, IndexCount;
	IndexBuf =          (unsigned short int *)*(surf+11);
	IndexBufEnd =     (unsigned short int *)*(surf+12);
	IndexCount = (IndexBufEnd-IndexBuf)/3;

	NxVec3* gVertices = new NxVec3[VertexCount];
	NxVec3* gTriangles = new NxVec3[IndexCount];
	
	for(int i=0;i<VertexCount;i++)
	{
		gVertices[i].x = ThisVertex[i].x;
		gVertices[i].y = ThisVertex[i].y;
		gVertices[i].z = ThisVertex[i].z;
	}
	for(int i=0;i<IndexCount;i++)
	{
		gTriangles[i].x = ThisTriangle[i].a;
		gTriangles[i].y = ThisTriangle[i].b;
		gTriangles[i].z = ThisTriangle[i].c;
	}

	NxActorDesc ActorDesc;
	NxTriangleMeshDesc meshDesc;
	meshDesc.numVertices                = VertexCount;
	meshDesc.numTriangles               = IndexCount;
	meshDesc.pointStrideBytes           = sizeof(NxVec3);
	meshDesc.triangleStrideBytes        = 3*sizeof(NxU32);
	meshDesc.points						= gVertices;
	meshDesc.triangles                  = gTriangles;                           
	meshDesc.flags                      = 0 ;
	
	NxInitCooking();
	MemoryWriteBuffer buf;
	bool status = NxCookTriangleMesh(meshDesc, buf);
	MemoryReadBuffer readBuffer(buf.data);
	NxTriangleMesh* terrainMesh = gPhysicsSDK->createTriangleMesh(readBuffer);

	NxTriangleMeshShapeDesc terrainShapeDesc;
	terrainShapeDesc.meshData				= terrainMesh;
	//terrainShapeDesc.meshFlags				= NX_SHAPE_MESH;
	
	ActorDesc.shapes.pushBack(&terrainShapeDesc);
	ActorDesc.globalPose.t  = NxVec3(0.0f, 0.0f, 0.0f);

	if (mass)
	{
		NxBodyDesc BodyDesc;
		BodyDesc.mass = mass;
		ActorDesc.body			= &BodyDesc;
	}

	NxActor* body = gScene->createActor(ActorDesc);
	mainSetMask(body);
	if (mass) BodyAddUserData(body);
	if (!mass) StaticBodyAddUserData(body);
	if (!mass) ShapeAddUserData(body);

	return body;

}
BBDECL NxActor* BBCALL pxCreateTriMeshFromPmap(NxTriangleMesh* triangleMesh, float mass)
{
	NxTriangleMeshShapeDesc bunnyShapeDesc;
	bunnyShapeDesc.meshData	= triangleMesh;


	NxActorDesc ActorDesc;
	ActorDesc.shapes.pushBack(&bunnyShapeDesc);
	ActorDesc.globalPose.t  = NxVec3(0.0f, 3.0f, 0.0f);
	if (mass)
	{
		NxBodyDesc BodyDesc;
		BodyDesc.mass = mass;
		ActorDesc.body			= &BodyDesc;
		//ActorDesc.density		= dens;
	}

	NxActor* body = gScene->createActor(ActorDesc);
	mainSetMask(body);
	if (mass) BodyAddUserData(body);
	if (!mass) StaticBodyAddUserData(body);
	ShapeAddUserData(body);
	return body;
}

BBDECL NxActor* BBCALL pxCreateStaticTriMesh(float *vbank,float *fbank,int MESH_NBVERTICES,int MESH_NBFACES)
{
	//NxBodyDesc BodyDesc;

	NxVec3* gVertices = new NxVec3[MESH_NBVERTICES];
	NxVec3* gTriangles = new NxVec3[MESH_NBFACES];
	
	for(int i=0;i<MESH_NBVERTICES;i++)
	{
		gVertices[i].x = vbank[i*3+0];
		gVertices[i].y = vbank[i*3+1];
		gVertices[i].z = vbank[i*3+2];
	}
	for(int i=0;i<MESH_NBFACES;i++)
	{
		gTriangles[i].x = fbank[i*3+0];
		gTriangles[i].y = fbank[i*3+1];
		gTriangles[i].z = fbank[i*3+2];
	}


	NxActorDesc ActorDesc;
	NxTriangleMeshDesc terrainDesc;

	terrainDesc.numVertices					= MESH_NBVERTICES;
	terrainDesc.numTriangles				= MESH_NBFACES;
	terrainDesc.pointStrideBytes			= sizeof(NxVec3);
	terrainDesc.triangleStrideBytes			= 3*sizeof(NxU32);
	terrainDesc.points						= gVertices;
	terrainDesc.triangles					= gTriangles;
	terrainDesc.flags						= 0;
	
	terrainDesc.heightFieldVerticalAxis		= NX_Y;
	terrainDesc.heightFieldVerticalExtent	= -1000.0f;

	NxInitCooking();
	MemoryWriteBuffer buf;
	bool status = NxCookTriangleMesh(terrainDesc, buf);
	MemoryReadBuffer readBuffer(buf.data);
	NxTriangleMesh* terrainMesh = gPhysicsSDK->createTriangleMesh(readBuffer);

	NxTriangleMeshShapeDesc terrainShapeDesc;
	terrainShapeDesc.groupsMask.bits0 = 1;
	terrainShapeDesc.meshData				= terrainMesh;
	//terrainShapeDesc.meshFlags				= NX_SHAPE_MESH;
	
	ActorDesc.shapes.pushBack(&terrainShapeDesc);
	ActorDesc.globalPose.t  = NxVec3(0.0f, 0.0f, 0.0f);

	NxActor* body = gScene->createActor(ActorDesc);
	mainSetMask(body);
	StaticBodyAddUserData(body);
	ShapeAddUserData(body);

	return body;

}
BBDECL NxActor* BBCALL pxCreateTriMesh(float *vbank, int *fbank,int MESH_NBVERTICES,int MESH_NBFACES, float mass)
{
/*	NxVec3* gVertices = new NxVec3[MESH_NBVERTICES];
	NxVec3* gTriangles = new NxVec3[MESH_NBFACES];
	
	FILE *stream;
	fopen_s( &stream, "C:\Log.txt", "w" );
	fprintf( stream, "Vert = %i\n", MESH_NBVERTICES );
	fprintf( stream, "Ind = %i\n", MESH_NBFACES );

	for(int i=0;i<MESH_NBVERTICES;i++)
	{
		gVertices[i].x = vbank[i*3+0];
		gVertices[i].y = vbank[i*3+1];
		gVertices[i].z = vbank[i*3+2];
		fprintf( stream, "v = %f\n", gVertices[i].x );
	}
	for(int i=0;i<MESH_NBFACES;i++)
	{
		fprintf( stream, "Ind = %i\n", i );
		fprintf( stream, " %i  %i  %i \n\n", fbank[i*3+0], fbank[i*3+1], fbank[i*3+2]);
	}
	fclose( stream );*/


	NxActorDesc ActorDesc;
	NxTriangleMeshDesc terrainDesc;

	terrainDesc.numVertices					= MESH_NBVERTICES;
	terrainDesc.numTriangles				= MESH_NBFACES;
	terrainDesc.pointStrideBytes			= sizeof(NxVec3);
	terrainDesc.triangleStrideBytes			= 3*sizeof(NxU32);
	terrainDesc.points						= vbank;
	terrainDesc.triangles					= fbank;
	terrainDesc.flags						= 0;
	
	NxInitCooking();
	MemoryWriteBuffer buf;
	bool status = NxCookTriangleMesh(terrainDesc, buf);
	MemoryReadBuffer readBuffer(buf.data);
	NxTriangleMesh* terrainMesh = gPhysicsSDK->createTriangleMesh(readBuffer);

	NxTriangleMeshShapeDesc terrainShapeDesc;
	terrainShapeDesc.meshData				= terrainMesh;
	//terrainShapeDesc.meshFlags				= NX_SHAPE_MESH;
	
	ActorDesc.shapes.pushBack(&terrainShapeDesc);
	ActorDesc.globalPose.t  = NxVec3(0.0f, 0.0f, 0.0f);

	if (mass)
	{
		NxBodyDesc BodyDesc;
		BodyDesc.mass = mass;
		ActorDesc.body			= &BodyDesc;
	}

	NxActor* body = gScene->createActor(ActorDesc);
	mainSetMask(body);
	if (mass) BodyAddUserData(body);
	if (!mass) StaticBodyAddUserData(body);
	if (!mass) ShapeAddUserData(body);

	return body;
}
BBDECL NxTriangleMesh* BBCALL pxCreateTerrainPmap(float *vbank,float *fbank,int MESH_NBVERTICES,int MESH_NBFACES, float axis, const char* file_name, int pMap)
{
	// Initialize PMap
	NxPMap MeshPMap;
	MeshPMap.dataSize	= 0;
	MeshPMap.data		= NULL;
	//const char* name = "mainmap.pmap";

	NxVec3* gVertices = new NxVec3[MESH_NBVERTICES];
	NxVec3* gTriangles = new NxVec3[MESH_NBFACES];
	
	for(int i=0;i<MESH_NBVERTICES;i++)
	{
		gVertices[i].x = vbank[i*3+0];
		gVertices[i].y = vbank[i*3+1];
		gVertices[i].z = vbank[i*3+2];
	}
	for(int i=0;i<MESH_NBFACES;i++)
	{
		gTriangles[i].x = fbank[i*3+0];
		gTriangles[i].y = fbank[i*3+1];
		gTriangles[i].z = fbank[i*3+2];
	}

	NxTriangleMeshDesc meshDesc;
	meshDesc.numVertices                = MESH_NBVERTICES;
	meshDesc.numTriangles               = MESH_NBFACES;
	meshDesc.pointStrideBytes           = sizeof(NxVec3);
	meshDesc.triangleStrideBytes        = 3*sizeof(NxU32);
	meshDesc.points						= gVertices;
	meshDesc.triangles                  = gTriangles;                           
	meshDesc.flags                      = 0;

	meshDesc.heightFieldVerticalAxis		= NX_Y;
	meshDesc.heightFieldVerticalExtent	= axis;
	

	bool status = InitCooking();
	if (!status) {
		printf("Unable to initialize the cooking library. Please make sure that you have correctly installed the latest version of the AGEIA PhysX SDK.");
		exit(1);
	}
	MemoryWriteBuffer buf;
	status = CookTriangleMesh(meshDesc, buf);
	if (!status) {
		printf("Unable to cook a triangle mesh.");
		exit(1);
	}
	MemoryReadBuffer readBuffer(buf.data);
	NxTriangleMesh* triangleMesh = gPhysicsSDK->createTriangleMesh(readBuffer);

	FILE* fp = fopen(file_name, "rb");
	if(!fp)
		{
		if(CreatePMap(MeshPMap, *triangleMesh, pMap))
			{
	
			fp = fopen(file_name, "wb+");
			if(fp)
				{
				fwrite(MeshPMap.data, MeshPMap.dataSize, 1, fp);
				fclose(fp);
				fp = NULL;
				}

			//assign pmap to mesh
			triangleMesh->loadPMap(MeshPMap);

			// sdk created data => sdk deletes it
			ReleasePMap(MeshPMap);
			}
		printf("...done\n");
		}
	else
		{
		// Found pmap file
		MeshPMap.dataSize	= getFileSize(file_name);
		MeshPMap.data		= new NxU8[MeshPMap.dataSize];
		fread(MeshPMap.data, MeshPMap.dataSize, 1, fp);
		fclose(fp);

		//assign pmap to mesh
		triangleMesh->loadPMap(MeshPMap);

		//we created data => we delete it
		delete MeshPMap.data;
		}
	delete[] gVertices;
	delete[] gTriangles;
	
	return triangleMesh;

}
BBDECL NxActor* BBCALL pxCreateTerrainFromMesh(float *vbank,float *fbank,int MESH_NBVERTICES,int MESH_NBFACES, float axis)
{
	//NxBodyDesc BodyDesc;

	NxVec3* gVertices = new NxVec3[MESH_NBVERTICES];
	NxVec3* gTriangles = new NxVec3[MESH_NBFACES];
	
	for(int i=0;i<MESH_NBVERTICES;i++)
	{
		gVertices[i].x = vbank[i*3+0];
		gVertices[i].y = vbank[i*3+1];
		gVertices[i].z = vbank[i*3+2];
	}
	for(int i=0;i<MESH_NBFACES;i++)
	{
		gTriangles[i].x = fbank[i*3+0];
		gTriangles[i].y = fbank[i*3+1];
		gTriangles[i].z = fbank[i*3+2];
	}


	NxActorDesc ActorDesc;
	NxTriangleMeshDesc terrainDesc;

	terrainDesc.numVertices					= MESH_NBVERTICES;
	terrainDesc.numTriangles				= MESH_NBFACES;
	terrainDesc.pointStrideBytes			= sizeof(NxVec3);
	terrainDesc.triangleStrideBytes			= 3*sizeof(NxU32);
	terrainDesc.points						= gVertices;
	terrainDesc.triangles					= gTriangles;
	terrainDesc.flags						= 0;
	
	terrainDesc.heightFieldVerticalAxis		= NX_Y;
	terrainDesc.heightFieldVerticalExtent	= axis;

	NxInitCooking();
	MemoryWriteBuffer buf;
	bool status = NxCookTriangleMesh(terrainDesc, buf);
	MemoryReadBuffer readBuffer(buf.data);
	NxTriangleMesh* terrainMesh = gPhysicsSDK->createTriangleMesh(readBuffer);

	NxTriangleMeshShapeDesc terrainShapeDesc;
	terrainShapeDesc.meshData				= terrainMesh;
	//terrainShapeDesc.meshFlags				= NX_SHAPE_MESH;
	
	ActorDesc.shapes.pushBack(&terrainShapeDesc);
	ActorDesc.globalPose.t  = NxVec3(0.0f, 0.0f, 0.0f);

	NxActor* body = gScene->createActor(ActorDesc);
	mainSetMask(body);
	StaticBodyAddUserData(body);
	ShapeAddUserData(body);

	return body;

}
BBDECL NxActor* BBCALL pxCreateTerrain(int nSize, float *bank, float scale_x, float scale_y, float scale_z)
{
    NxHeightFieldDesc heightFieldDesc;
	static NxReal sixtyFourKb = 65536.0f;
	static NxReal thirtyTwoKb = 32767.5f;

	heightFieldDesc.nbColumns           = nSize;
	heightFieldDesc.nbRows              = nSize;
	heightFieldDesc.verticalExtent      = -1000;
	heightFieldDesc.convexEdgeThreshold = 0;

	
	heightFieldDesc.samples			= new NxU32[nSize*nSize];
	heightFieldDesc.sampleStride	= sizeof(int);
	char* currentByte = (char*)heightFieldDesc.samples;

	float test = 0.0f;
	for (int column = 0; column < nSize; column++)
	{  
		for (int row = 0; row < nSize; row++)
		{
			NxHeightFieldSample* currentSample = (NxHeightFieldSample*)currentByte;
			currentSample->height = (NxI32)(0.5 * thirtyTwoKb*bank[column+row*nSize]);
			//Desired height value. Signed 16 bit integer.
		//	currentSample->materialIndex0 = gMaterial0;
		//	currentSample->materialIndex1 = gMaterial1;
		//	currentSample->tessFlag = 0;
			currentByte += heightFieldDesc.sampleStride;
			if (bank[column*nSize+row]>-0.1)	test = test+1;
		}
	}

	NxHeightField* heightField = gScene->getPhysicsSDK().createHeightField(heightFieldDesc);

	// data has been copied, we can free our buffer
	delete [] heightFieldDesc.samples;

	NxHeightFieldShapeDesc heightFieldShapeDesc;
	heightFieldShapeDesc.heightField	= heightField;
	heightFieldShapeDesc.heightScale	= scale_y*4/sixtyFourKb;
	heightFieldShapeDesc.rowScale		= scale_x;
	heightFieldShapeDesc.columnScale	= scale_z;
	heightFieldShapeDesc.materialIndexHighBits = 0;
	heightFieldShapeDesc.holeMaterial = 1;

//	NxBodyDesc bodyDesc;
//	bodyDesc.angularDamping	= 0.5f;
//	bodyDesc.flags |= NX_BF_KINEMATIC;

	NxActorDesc actorDesc;
	actorDesc.shapes.pushBack(&heightFieldShapeDesc);
	actorDesc.globalPose.t.set(0,0,0);

	NxActor* body = gScene->createActor(actorDesc);
	mainSetMask(body);
	StaticBodyAddUserData(body);
	ShapeAddUserData(body);

	return body;
}
BBDECL int BBCALL pxCreateTriMeshToFile(float *vbank,float *fbank,int MESH_NBVERTICES,int MESH_NBFACES, const char* fname)
{
	NxVec3* gVertices = new NxVec3[MESH_NBVERTICES];
	NxVec3* gTriangles = new NxVec3[MESH_NBFACES];
	
	for(int i=0;i<MESH_NBVERTICES;i++)
	{
		gVertices[i].x = vbank[i*3+0];
		gVertices[i].y = vbank[i*3+1];
		gVertices[i].z = vbank[i*3+2];
	}
	for(int i=0;i<MESH_NBFACES;i++)
	{
		gTriangles[i].x = fbank[i*3+0];
		gTriangles[i].y = fbank[i*3+1];
		gTriangles[i].z = fbank[i*3+2];
	}


	NxActorDesc ActorDesc;
	NxTriangleMeshDesc terrainDesc;

	terrainDesc.numVertices					= MESH_NBVERTICES;
	terrainDesc.numTriangles				= MESH_NBFACES;
	terrainDesc.pointStrideBytes			= sizeof(NxVec3);
	terrainDesc.triangleStrideBytes			= 3*sizeof(NxU32);
	terrainDesc.points						= gVertices;
	terrainDesc.triangles					= gTriangles;
	terrainDesc.flags						= 0;
	
	NxInitCooking();
	bool status = NxCookTriangleMesh(terrainDesc, UserStream(fname, false));
	return 1;

}
BBDECL NxActor* BBCALL pxCreateTriMeshFromFile(const char* fname, float mass)
{

	NxActorDesc ActorDesc;

	NxTriangleMesh* terrainMesh = gPhysicsSDK->createTriangleMesh(UserStream(fname, true));

	NxTriangleMeshShapeDesc terrainShapeDesc;
	terrainShapeDesc.meshData		= terrainMesh;
	
	ActorDesc.shapes.pushBack(&terrainShapeDesc);
	ActorDesc.globalPose.t  = NxVec3(0.0f, 0.0f, 0.0f);

	if (mass)
	{
		NxBodyDesc BodyDesc;
		BodyDesc.mass = mass;
		ActorDesc.body			= &BodyDesc;
	}

	NxActor* body = gScene->createActor(ActorDesc);
	mainSetMask(body);
	if (mass) BodyAddUserData(body);
	if (!mass) StaticBodyAddUserData(body);
	if (!mass) ShapeAddUserData(body);

	return body;

}
//----------------Create Soft Body-------------
typedef struct tagSBDATA
{
	MySoftBody * SB;
	int BlitzData;
}SBData;
BBDECL TSBVert* BBCALL pxSoftBodyLinkMesh(MySoftBody* softBody, float* vbank, int numVert)
{
	ObjMesh* objMesh = softBody->mSurfaceMesh;
	int objNumVert = objMesh->getNumVertices();
	float dx=0,dy=0,dz=0;

	TSBVert* SBBufferVert = new TSBVert[numVert];
	NxVec3* SBBufferNormals = new NxVec3[numVert];
	
	for (int i=0; i<numVert; i++)
	{
		for(int j=0; j<objNumVert; j++)
		{
			NxVec3 objVert = objMesh->getVertex(j);
			dx = vbank[i*3+0] - objVert.x;
			dy = vbank[i*3+1] - objVert.y;
			dz = vbank[i*3+2] + objVert.z;
//			fprintf( stream, "I= %i\n", i);
//			fprintf( stream, "J= %i\n", j);
//			fprintf( stream, "VB= %f  %f   %f\n", vbank[i*3+0], vbank[i*3+1], vbank[i*3+2]);
//			fprintf( stream, "VO= %f  %f   %f\n", objVert.x, objVert.y, objVert.z);
//			fprintf( stream, "D= %f  %f   %f\n\n", dx, dy, dz);
			if (abs(dx)<SB_Link_Radius && abs(dy)<SB_Link_Radius && abs(dz)<SB_Link_Radius)
			{
				SBBufferVert[i].x = vbank[i*3+0];
				SBBufferVert[i].y = vbank[i*3+1];
				SBBufferVert[i].z = vbank[i*3+2];
				SBBufferVert[i].vi = j;
				j = objNumVert;
			} else
			{
				SBBufferVert[i].x = vbank[i*3+0];
				SBBufferVert[i].y = vbank[i*3+1];
				SBBufferVert[i].z = vbank[i*3+2];
				SBBufferVert[i].vi = -1;
			}
		}
	}
	softBody->SBBufferVert = SBBufferVert;
	softBody->SBBufferNormals = SBBufferNormals;
	softBody->numVert = numVert;

	return SBBufferVert;
}
BBDECL MySoftBody* BBCALL pxCreateSoftBody(char* OBJfile, char* TETfile, float* vbank, int numVert, float pos_x, float pos_y, float pos_z, float rot_x, float rot_y, float rot_z)
{
	if (bah==1) 
	{
		ShowError("This is Demo version. Time over!!!", "pxCreateSoftBody", __FILE__ , __LINE__);
		return 0;
	}
	NxQuat SB_quat = NxQuat( NxVec3( 0,NxMath::sin(NxMath::degToRad(rot_y*0.5f)),0 ), -NxMath::cos(NxMath::degToRad(rot_y*0.5f))) * 
	NxQuat(NxVec3( NxMath::sin(NxMath::degToRad(-rot_x*0.5f)),0,0 ), -NxMath::cos(NxMath::degToRad(-rot_x*0.5f)) ) * 
	NxQuat( NxVec3( 0,0,NxMath::sin(NxMath::degToRad(-rot_z*0.5f))), -NxMath::cos(NxMath::degToRad(-rot_z*0.5f)) );


	NxSoftBodyDesc softBodyDesc;
	softBodyDesc.globalPose.t = NxVec3(pos_x,pos_y,pos_z);
	softBodyDesc.globalPose.M.fromQuat(SB_quat);
	softBodyDesc.particleRadius = 0.2f;
	softBodyDesc.volumeStiffness = 1.0f;
	softBodyDesc.stretchingStiffness = 0.03f;
	softBodyDesc.friction = 1.0f;
	softBodyDesc.solverIterations = 2;

	if (gHardwareSimulation)
		softBodyDesc.flags |= NX_SBF_HARDWARE;

	ObjMesh *objMesh = new ObjMesh();
	objMesh->loadFromObjFile(OBJfile);//FindMediaFile(objFileName, s));

	MySoftBody *softBody = new MySoftBody(gScene, softBodyDesc, TETfile, objMesh);
	NxSoftBody* SB = softBody->getNxSoftBody();
	if (!SB)
	{
		printf("Error: Unable to create the SoftBody for the current scene.\n");
		delete softBody;
	}

	SBData* data = new SBData;
	data->SB = softBody;
	SB->userData = data;

	NxGroupsMask GrMask;
	GrMask.bits0 = 1;
	GrMask.bits1 = 1;
	GrMask.bits2 = 1;
	GrMask.bits3 = 1;
	SB->setGroupsMask (GrMask);

	pxSoftBodyLinkMesh(softBody, vbank, numVert);

	return softBody;
}
BBDECL void BBCALL pxDeleteSoftBody(MySoftBody* softBody)
{
	gScene->releaseSoftBody(*(softBody->getNxSoftBody()));
	delete softBody;
}
BBDECL int BBCALL pxSoftBodyGetNumVert(MySoftBody* softBody)
{
	ObjMesh* mobjMesh = softBody->mSurfaceMesh;
	return mobjMesh->getNumVertices();
}
BBDECL void BBCALL pxUpdateSoftBody(MySoftBody* softBody)
{
	softBody->simulateAndDraw(0, 0, 0);
}
BBDECL void BBCALL pxSoftBodyGetVertexPos(MySoftBody* softBody, float *vbank, int numVert)
{
	//TSBVert* sbVert = new TSBVert[4*numVert];
	//NxSoftBody* SB = softBody->getNxSoftBody();
	//SB->getPositions (sbVert);  
	TSBVert* sbVert = softBody->SBBufferVert;
	for (int i=0; i<numVert; i++)
	{
		vbank[i*3+0] = sbVert[i].x;
		vbank[i*3+1] = sbVert[i].y;
		vbank[i*3+2] = sbVert[i].z;
	}
}
BBDECL void BBCALL pxSoftBodyGetNormals(MySoftBody* softBody, float *nbank, int numVert)
{
	//TSBVert* sbVert = new TSBVert[4*numVert];
	//NxSoftBody* SB = softBody->getNxSoftBody();
	//SB->getPositions (sbVert);  
	NxVec3* sbNormals = softBody->SBBufferNormals;
	for (int i=0; i<numVert; i++)
	{
		nbank[i*3+0] = sbNormals[i].x;
		nbank[i*3+1] = sbNormals[i].y;
		nbank[i*3+2] = sbNormals[i].z;
	}
}
BBDECL int BBCALL pxSoftBodyIsSleeping(MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->isSleeping();
}
BBDECL void BBCALL pxSoftBodyPutToSleep(MySoftBody* softBody)
{
	softBody->getNxSoftBody()->putToSleep();
}
BBDECL void BBCALL pxSoftBodyWakeUp(MySoftBody* softBody, int interval)
{
	softBody->getNxSoftBody()->wakeUp(NxReal(interval));
}
//--------------------------------------------
BBDECL void BBCALL pxSoftBodySetAttachmentResponseCoefficient (MySoftBody* softBody, float coef)
{
	softBody->getNxSoftBody()->setAttachmentResponseCoefficient(coef);
}
BBDECL void BBCALL pxSoftBodySetAttachmentTearFactor(MySoftBody* softBody, float coef)
{
	softBody->getNxSoftBody()->setAttachmentTearFactor(coef);
}
BBDECL void BBCALL pxSoftBodySetCollisionResponseCoefficient(MySoftBody* softBody, float coef)
{
	softBody->getNxSoftBody()->setCollisionResponseCoefficient (coef);
}
BBDECL void BBCALL pxSoftBodySetDampingCoefficient(MySoftBody* softBody, float coef)
{
	softBody->getNxSoftBody()->setFlags(softBody->getNxSoftBody()->getFlags()|NX_SBF_DAMPING);
	softBody->getNxSoftBody()->setDampingCoefficient (coef);
}
BBDECL void BBCALL pxSoftBodySetExternalAcceleration(MySoftBody* softBody, float x, float y, float z)
{
	softBody->getNxSoftBody()->setExternalAcceleration (NxVec3(x,y,z));
}
BBDECL void BBCALL pxSoftBodySetFriction(MySoftBody* softBody, float fric)
{
	softBody->getNxSoftBody()->setFriction(fric);
}
BBDECL void BBCALL pxSoftBodySetMask(MySoftBody* softBody, int mask)
{
	NxGroupsMask GrMask;//bits0, bits1, bits2, bits3;
	if (mask==0) 
	{
		GrMask.bits0 = 1;
		GrMask.bits1 = 0;
		GrMask.bits2 = 0;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==1) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 1;
		GrMask.bits2 = 0;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==2) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 0;
		GrMask.bits2 = 1;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==3) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 0;
		GrMask.bits2 = 0;
		GrMask.bits3 = 1;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}

	softBody->getNxSoftBody()->setGroupsMask(GrMask);
}

BBDECL void BBCALL pxSoftBodySetMaskCombine(MySoftBody* softBody, int mask)
{
	NxGroupsMask GrMask;
	GrMask.bits0 = mask & 1;
	GrMask.bits1 = mask & 2;
	GrMask.bits2 = mask & 4;
	GrMask.bits3 = mask & 8;

	softBody->getNxSoftBody()->setGroupsMask(GrMask);
}

BBDECL void BBCALL pxSoftBodyMaskClear(MySoftBody* softBody)
{
	NxGroupsMask GrMask;//bits0, bits1, bits2, bits3;

		GrMask.bits0 = 0xffffffff;
		GrMask.bits1 = 0xffffffff;
		GrMask.bits2 = 0xffffffff;
		GrMask.bits3 = 0xffffffff;

	softBody->getNxSoftBody()->setGroupsMask(GrMask);
}
BBDECL void BBCALL pxSoftBodySetName(MySoftBody* softBody, char* name)
{
	softBody->getNxSoftBody()->setName(name);
}
BBDECL void BBCALL pxSoftBodySetParticleRadius(MySoftBody* softBody, float radius)
{
	softBody->getNxSoftBody()->setParticleRadius (radius);
}
BBDECL void BBCALL pxSoftBodySetLinkRadius(float radius)
{
	SB_Link_Radius = radius;
}
BBDECL void BBCALL pxSoftBodySetSleepLinearVelocity(MySoftBody* softBody, float vel)
{
	softBody->getNxSoftBody()->setSleepLinearVelocity (vel);
}
BBDECL void BBCALL pxSoftBodySetSolverIterations(MySoftBody* softBody, int iter)
{
	softBody->getNxSoftBody()->setSolverIterations(iter);
}
BBDECL void BBCALL pxSoftBodySetStretchingStiffness(MySoftBody* softBody, float coef)
{
	softBody->getNxSoftBody()->setStretchingStiffness(coef);
}
BBDECL void BBCALL pxSoftBodySetVolumeStiffness(MySoftBody* softBody, float coef)
{
	softBody->getNxSoftBody()->setVolumeStiffness(coef);
}
BBDECL void BBCALL pxSoftBodySetGravity(MySoftBody* softBody, int stat)
{
	if (stat==1) softBody->getNxSoftBody()->setFlags(softBody->getNxSoftBody()->getFlags()|NX_SBF_GRAVITY);
	if (stat==0) softBody->getNxSoftBody()->setFlags(softBody->getNxSoftBody()->getFlags()&(~NX_SBF_GRAVITY));
}
BBDECL void BBCALL pxSoftBodySetStatic(MySoftBody* softBody, int stat)
{
	if (stat==1) softBody->getNxSoftBody()->setFlags(softBody->getNxSoftBody()->getFlags()|NX_SBF_STATIC);
	if (stat==0) softBody->getNxSoftBody()->setFlags(softBody->getNxSoftBody()->getFlags()&(~NX_SBF_STATIC));
}
BBDECL void BBCALL pxSoftBodySetCollision(MySoftBody* softBody, int stat)
{
     if (stat==1) softBody->getNxSoftBody()->setFlags(softBody->getNxSoftBody()->getFlags()|NX_SBF_DISABLE_COLLISION);
     if (stat==0) softBody->getNxSoftBody()->setFlags(softBody->getNxSoftBody()->getFlags()&(~NX_SBF_DISABLE_COLLISION));
}
//--------------------------------------------
BBDECL float BBCALL pxSoftBodyGetAttachmentResponseCoefficient (MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->getAttachmentResponseCoefficient();
}
BBDECL float BBCALL pxSoftBodyGetAttachmentTearFactor(MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->getAttachmentTearFactor();
}
BBDECL float BBCALL pxSoftBodyGetCollisionResponseCoefficient(MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->getCollisionResponseCoefficient ();
}
BBDECL float BBCALL pxSoftBodyGetDampingCoefficient(MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->getDampingCoefficient ();
}
BBDECL float BBCALL pxSoftBodyGetFriction(MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->getFriction();
}
BBDECL const char* BBCALL pxSoftBodyGetName(MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->getName();
}
BBDECL float BBCALL pxSoftBodyGetParticleRadius(MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->getParticleRadius ();
}
BBDECL float BBCALL pxSoftBodyGetSleepLinearVelocity(MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->getSleepLinearVelocity ();
}
BBDECL int BBCALL pxSoftBodyGetSolverIterations(MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->getSolverIterations();
}
BBDECL float BBCALL pxSoftBodyGetStretchingStiffness(MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->getStretchingStiffness();
}
BBDECL float BBCALL pxSoftBodyGetVolumeStiffness(MySoftBody* softBody)
{
	return softBody->getNxSoftBody()->getVolumeStiffness();
}
//--------------------------------------------
BBDECL void BBCALL pxSoftBodyAttachToShape(MySoftBody* softBody, NxActor* body, int flag)
{
	if (flag == 0)softBody->getNxSoftBody()->attachToShape(body->getShapes()[0], NX_SOFTBODY_ATTACHMENT_TWOWAY);
	if (flag == 1)softBody->getNxSoftBody()->attachToShape(body->getShapes()[0], NX_SOFTBODY_ATTACHMENT_TEARABLE);
}
BBDECL void BBCALL pxSoftBodyAttachToCollidingShapes(MySoftBody* softBody, int tearable)
{
	softBody->getNxSoftBody()->attachToCollidingShapes(tearable);
}
BBDECL void BBCALL pxSoftBodyDetachFromShape(MySoftBody* softBody, NxActor* body)
{
	softBody->getNxSoftBody()->detachFromShape(body->getShapes()[0]);
}
BBDECL void BBCALL pxSoftBodyAttachVertexToGlobalPosition(MySoftBody* softBody, int vertID, float x, float y, float z)
{
	softBody->getNxSoftBody()->attachVertexToGlobalPosition(vertID, NxVec3(x,y,z));
}
BBDECL void BBCALL pxSoftBodyFreeVertex(MySoftBody* softBody, int vertID)
{
	softBody->getNxSoftBody()->freeVertex(vertID);
}
BBDECL void BBCALL pxSoftBodyAddForceAtPos(MySoftBody* softBody, float pos_x, float pos_y, float pos_z, float magnitude, float radius, int mode)
{
	softBody->getNxSoftBody()->addForceAtPos(NxVec3(pos_x,pos_y,pos_z), magnitude, radius, (NxForceMode)mode);
}
BBDECL void BBCALL pxSoftBodyAddForceAtVertex(MySoftBody* softBody, float x, float y, float z, int vertID, int mode)
{
	softBody->getNxSoftBody()->addForceAtVertex(NxVec3(x,y,z), vertID, (NxForceMode)mode);
}
//----------------Create Particle-------------
BBDECL ParticleEmitter* BBCALL pxParticleCreateEmitter()
{
	NxReal fStartingParticleSpeed = 5.0f;
	NxReal fVerticalThermoDynamicAccel = 5.0f;
	ParticleEmitter* mParticleEmitter
	=	new ParticleEmitter(gScene, NxVec3(0,0,0), 5, NxVec3(0,-4,0));
	gScene->setGroupCollisionFlag(cParticleCollisionGroup, cParticleCollisionGroup, false);

	return mParticleEmitter;
}
BBDECL void BBCALL pxParticleEmitSetAngDamping(ParticleEmitter* pEmitter, float damp)
{
	pEmitter->_pADamping = damp;
}
BBDECL void BBCALL pxParticleEmitSetLinDamping(ParticleEmitter* pEmitter, float damp)
{
	pEmitter->_pLDamping = damp;
}
BBDECL void BBCALL pxParticleEmitSetMass(ParticleEmitter* pEmitter, float mass)
{
	pEmitter->_pMass = mass;
}
BBDECL void BBCALL pxParticleEmitSetRadius(ParticleEmitter* pEmitter, float radius)
{
	pEmitter->_pRadius = radius;
}
BBDECL void BBCALL pxParticleEmitSetPosition(ParticleEmitter* pEmitter, float x, float y, float z)
{
	pEmitter->_vStartingPos=NxVec3(x, y, z);
}
BBDECL void BBCALL pxParticleEmitSetRotation(ParticleEmitter* pEmitter, float pitch, float yaw, float roll)
{
	NxQuat qYaw = NxQuat( NxVec3( 0,NxMath::sin(NxMath::degToRad(yaw*0.5f)),0 ), -NxMath::cos(NxMath::degToRad(yaw*0.5f)));  
	NxQuat qPitch =	NxQuat(NxVec3( NxMath::sin(NxMath::degToRad(-pitch*0.5f)),0,0 ), -NxMath::cos(NxMath::degToRad(-pitch*0.5f)) );
	NxQuat qRoll = NxQuat( NxVec3( 0,0,NxMath::sin(NxMath::degToRad(-roll*0.5f))), -NxMath::cos(NxMath::degToRad(-roll*0.5f)) );
	NxQuat qo = qYaw*qPitch*qRoll;

	pEmitter->_pRotation.set(pitch, yaw, roll);
	pEmitter->_peOrientation = qo;
}
BBDECL void BBCALL pxParticleEmitSetRandRadius(ParticleEmitter* pEmitter, float radius)
{
	pEmitter->_pScaleRadius = radius;
}
BBDECL void BBCALL pxParticleEmitSetStartSpeed(ParticleEmitter* pEmitter, float min, float max)
{
	pEmitter->_pScaleStartVelmin = min;
	pEmitter->_pScaleStartVelmax = max;
}
BBDECL void BBCALL pxParticleEmitSetTDAcceleration(ParticleEmitter* pEmitter, float x, float y, float z)
{
	pEmitter->_vThermoDynamicAcceleration.set(x, y, z);
}
BBDECL void BBCALL pxParticleEmitSetScaleFactor(ParticleEmitter* pEmitter, float radius, float rate)
{
	pEmitter->_fMaxRadius = radius;
	pEmitter->_fGrowthRate = rate;
}
BBDECL float* BBCALL pxParticleEmitDeleteFirstParticle(ParticleEmitter* pEmitter)
{
	return pEmitter->removeParticle();
}
BBDECL void BBCALL pxParticleEmitDeleteParticle(ParticleEmitter* pEmitter, Particle* particle)
{
	pEmitter->removeOneParticle(particle);
}
//------------------------------------------
BBDECL float BBCALL pxParticleEmitGetAngDamping(ParticleEmitter* pEmitter)
{
	return pEmitter->_pADamping;
}
BBDECL float BBCALL pxParticleEmitGetLinDamping(ParticleEmitter* pEmitter)
{
	return pEmitter->_pLDamping;
}
BBDECL float BBCALL pxParticleEmitGetMass(ParticleEmitter* pEmitter)
{
	return pEmitter->_pMass;
}
BBDECL float BBCALL pxParticleEmitGetRadius(ParticleEmitter* pEmitter)
{
	return pEmitter->_pRadius;
}
BBDECL float BBCALL pxParticleEmitGetPositionX(ParticleEmitter* pEmitter)
{
	return pEmitter->_vStartingPos.x;
}
BBDECL float BBCALL pxParticleEmitGetPositionY(ParticleEmitter* pEmitter)
{
	return pEmitter->_vStartingPos.y;
}
BBDECL float BBCALL pxParticleEmitGetPositionZ(ParticleEmitter* pEmitter)
{
	return pEmitter->_vStartingPos.z;
}
BBDECL float BBCALL pxParticleEmitGetRotationPitch(ParticleEmitter* pEmitter)
{
	return pEmitter->_pRotation.x;
}
BBDECL float BBCALL pxParticleEmitGetRotationYaw(ParticleEmitter* pEmitter)
{
	return pEmitter->_pRotation.y;
}
BBDECL float BBCALL pxParticleEmitGetRotationRoll(ParticleEmitter* pEmitter)
{
	return pEmitter->_pRotation.z;
}
BBDECL float BBCALL pxParticleEmitGetRandRadius(ParticleEmitter* pEmitter)
{
	return pEmitter->_pScaleRadius;
}
BBDECL float BBCALL pxParticleEmitGetStartSpeedMax(ParticleEmitter* pEmitter)
{
	return pEmitter->_pScaleStartVelmax;
}
BBDECL float BBCALL pxParticleEmitGetStartSpeedMin(ParticleEmitter* pEmitter)
{
	return pEmitter->_pScaleStartVelmin;
}
BBDECL float BBCALL pxParticleEmitGetTDAccelerationX(ParticleEmitter* pEmitter)
{
	return pEmitter->_vThermoDynamicAcceleration.x;
}
BBDECL float BBCALL pxParticleEmitGetTDAccelerationY(ParticleEmitter* pEmitter)
{
	return pEmitter->_vThermoDynamicAcceleration.y;
}
BBDECL float BBCALL pxParticleEmitGetTDAccelerationZ(ParticleEmitter* pEmitter)
{
	return pEmitter->_vThermoDynamicAcceleration.z;
}
BBDECL float BBCALL pxParticleEmitGetScaleFactorRadius(ParticleEmitter* pEmitter)
{
	return pEmitter->_fMaxRadius;
}
BBDECL float BBCALL pxParticleEmitGetScaleFactorRate(ParticleEmitter* pEmitter)
{
	return pEmitter->_fGrowthRate;
}
BBDECL int BBCALL pxParticleEmitGetNumberParticles(ParticleEmitter* pEmitter)
{
	return pEmitter->_iParticleCount;
}
//------------------------------------------
BBDECL Particle* BBCALL pxParticleEmitAddParticle(ParticleEmitter* pEmitter, float* entity)
{
	Particle* mParticle = pEmitter->addParticle();
	NxActor* body = mParticle->_pActor;
	mainSetMask(body);
	//BodyAddUserData(body);
	BodyData* data = new BodyData;
	data->contacttable = 0;
	data->magnitable = 0;
	data->triggertable = 0;
	data->nkeps = 0;
	data->numTriggerActors = 0;
	data->trigerControl = 0;
	body->userData = data;

	mParticle->pEntity = entity;
	return mParticle;
}
BBDECL void BBCALL pxParticleEmitDelete(ParticleEmitter* pEmitter)
{
	pEmitter->~ParticleEmitter();
	delete pEmitter;
}
BBDECL void BBCALL pxParticleUpdateEmitter(ParticleEmitter* pEmitter)
{
	pEmitter->update(gDeltaTime);
}
BBDECL float* BBCALL pxParticleGetEntity(Particle* particle)
{
	return particle->pEntity;
}
BBDECL NxActor* BBCALL pxParticleGetBody(Particle* particle)
{
	return particle->_pActor;
}
BBDECL float BBCALL pxParticleGetradius(Particle* particle)
{
	NxSphereShape* pSphere = particle->_pActor->getShapes()[0]->isSphere();
	float fCurrentRadius = pSphere->getRadius();
	return fCurrentRadius;
}
//----------------Create Compound-------------
BBDECL NxActorDesc* BBCALL pxCreateCompoundDesc()
{
	NxActorDesc *ActorDesc = new NxActorDesc;
	return ActorDesc;//IDnumber;
}
BBDECL NxBoxShapeDesc* BBCALL pxCompoundAddCubeShape(NxActorDesc* compoundDesc, float dx, float dy, float dz)
{
	// Create body
	NxBoxShapeDesc *BoxDesc = new NxBoxShapeDesc;
	BoxDesc->dimensions = NxVec3(dx, dy, dz);
	//BoxDesc->groupsMask.bits0 = 1;
	compoundDesc->shapes.pushBack(BoxDesc);

	return BoxDesc;//IDnumber;
}
BBDECL NxSphereShapeDesc* BBCALL pxCompoundAddSphereShape(NxActorDesc* compoundDesc, float radius)
{
	// Create body

	NxSphereShapeDesc *SphereDesc = new NxSphereShapeDesc;
	SphereDesc->radius = radius;
	//SphereDesc->groupsMask.bits0 = 1;

	compoundDesc->shapes.pushBack(SphereDesc);
	return SphereDesc;
}
BBDECL NxCapsuleShapeDesc* BBCALL pxCompoundAddCapsuleShape(NxActorDesc* compoundDesc, float radius, float height)
{

	NxCapsuleShapeDesc *CapsuleDesc = new NxCapsuleShapeDesc;
	CapsuleDesc->groupsMask.bits0 = 1;
	
	CapsuleDesc->height = height;
	CapsuleDesc->radius = radius;

	compoundDesc->shapes.pushBack(CapsuleDesc);

	return CapsuleDesc;
}
BBDECL NxConvexShapeDesc* BBCALL pxCompoundAddCylinderShape(NxActorDesc* compoundDesc, float radius, float height, int nbEdge)
{

	NxConvexShapeDesc *ConvShapeDesc = new NxConvexShapeDesc;
	ConvShapeDesc->groupsMask.bits0 = 1;
	
	int nbVerts = nbEdge;
	float alpha;
	float delta = 2*3.14f/nbVerts;
	float z=0;
	NxArray<NxVec3> points;

	if (radius == height) radius = radius - radius*0.01f;
	int parnost = nbEdge/2;
	for (int j=0; j<2; j++)
	{
		z = 2*height*j - height;
		if (nbEdge == 2*parnost) alpha = 3.14f/nbVerts+3.14f/4;
		if (nbEdge != 2*parnost)alpha = 3.14f/nbVerts-3.14f/2;
		for(int i=0; i<nbVerts; i++)
		{			
			points.pushBack().set(radius*cos(alpha), z, radius*sin(alpha));
			alpha = alpha + delta;
		}
	}
	
		NxConvexMeshDesc *convexMesh = new NxConvexMeshDesc;
		convexMesh->numVertices = points.size();
		convexMesh->points = &(points[0].x);
		convexMesh->pointStrideBytes = sizeof(NxVec3);
		convexMesh->flags |= NX_CF_COMPUTE_CONVEX;//|NX_CF_USE_LEGACY_COOKER;

		MemoryWriteBuffer buf;
		bool status = NxCookConvexMesh(*convexMesh, buf);
		if(status)
		{
			ConvShapeDesc->meshData = gPhysicsSDK->createConvexMesh(MemoryReadBuffer(buf.data));
			compoundDesc->shapes.pushBack(ConvShapeDesc);
		}

	//compoundDesc->shapes.pushBack(CapsuleDesc);

	return ConvShapeDesc;
}
BBDECL NxConvexShapeDesc* BBCALL pxCompoundAddHullShape(NxActorDesc* compoundDesc, float *vbank, int nvert)
{
	int nbVerts=nvert;
	NxVec3* verts = new NxVec3[nbVerts];
		for(int i=0;i<nbVerts;i++)
		{
			verts[i].x = vbank[i*3+0];
			verts[i].y = vbank[i*3+1];
			verts[i].z = vbank[i*3+2];
		}
	
	NxU32 vertCount = nbVerts;
	
	NxConvexShapeDesc *convexShapeDesc = new NxConvexShapeDesc;
	NxConvexMeshDesc *convexDesc =	new NxConvexMeshDesc;
	convexDesc->numVertices			= nbVerts;
	convexDesc->pointStrideBytes	= sizeof(NxVec3);
	convexDesc->points				= verts;
	convexDesc->flags				= NX_CF_COMPUTE_CONVEX;//|NX_CF_USE_LEGACY_COOKER;
	
	// Cooking from memory
	MemoryWriteBuffer buf;
	if(CookConvexMesh(*convexDesc, buf))
	{

		convexShapeDesc->meshData = gPhysicsSDK->createConvexMesh(MemoryReadBuffer(buf.data));
		compoundDesc->shapes.pushBack(convexShapeDesc);
	}

	//compoundDesc->shapes.pushBack(CapsuleDesc);
	delete[] verts;
	return convexShapeDesc;
}
BBDECL void BBCALL pxCompoundSetShapePos(NxShapeDesc* shapeDesc, float x, float y, float z)
{

	shapeDesc->localPose.t.set(x, y, z);

}
BBDECL void BBCALL pxCompoundSetShapeRot(NxShapeDesc* shapeDesc, float pitch, float yaw, float roll)
{

	NxQuat RotationShape = NxQuat( NxVec3( 0,sinf(yaw*3.14159f/360),0 ), -cosf(yaw*3.14159f/360) ) * NxQuat(NxVec3( sinf(pitch*3.14159f/-360),0,0 ), -cosf(pitch*3.14159f/-360) ) * NxQuat( NxVec3( 0,0,sinf(roll*3.14159f/-360)), -cosf(roll*3.14159f/-360) );
	shapeDesc->localPose.M.fromQuat((NxQuat) RotationShape);

}
BBDECL NxActor* BBCALL pxCreateCompound(NxActorDesc* compoundDesc, float mass)
{
	if (mass)
	{
		NxBodyDesc BodyDesc;
		BodyDesc.mass = mass;
		compoundDesc->body		= &BodyDesc;
		//compoundDesc->density		= dens;
	}

	NxActor* body = gScene->createActor(*compoundDesc);
	mainSetMask(body);
	if (mass)BodyAddUserData(body);
	if (!mass)StaticBodyAddUserData(body);
	if (!mass) ShapeAddUserData(body);

	return body;
}
//----------Create Cinematic----------
BBDECL void BBCALL pxKinematicSet(NxActor* body)
{
	body->raiseBodyFlag(NX_BF_KINEMATIC);
	BodyData* data = (BodyData*) body->userData;
	data->StaticBody = 1;
}
BBDECL void BBCALL pxKinematicClear(NxActor* body)
{
	body->clearBodyFlag(NX_BF_KINEMATIC);
	BodyData* data = (BodyData*) body->userData;
	data->StaticBody = 0;
}
BBDECL void BBCALL pxKinematicSetPosition(NxActor* body, float x, float y, float z)
{
	body->moveGlobalPosition(NxVec3(x, y, z));
}
BBDECL void BBCALL pxKinematicSetRotation(NxActor* body, float pitch, float yaw, float roll)
{
	NxQuat qo = NxQuat( NxVec3( 0,sinf(NxMath::degToRad(yaw*0.5f)),0 ), -cosf(NxMath::degToRad(yaw*0.5f)) ) 
		* NxQuat(NxVec3( sinf(NxMath::degToRad(-pitch*0.5f)),0,0 ), -cosf(NxMath::degToRad(-pitch*0.5f))) 
		* NxQuat( NxVec3( 0,0,sinf(NxMath::degToRad(-roll*0.5f))), -cosf(NxMath::degToRad(-roll*0.5f)) );

	body->moveGlobalOrientationQuat(qo);
}
BBDECL void BBCALL pxKinematicMove(NxActor* body, float x, float y, float z)
{
	if (x || y || z)
	{
		NxVec3 mov_pos = body->getGlobalPosition();
		mov_pos +=  NxVec3(x,y,z);
		body->moveGlobalPosition(mov_pos);
	}
}
BBDECL void BBCALL pxKinematicTurn(NxActor* body, float pitch, float yaw, float roll)
{
	
	q = body->getGlobalOrientationQuat();
	float xz = q.x * q.z; 
	float wy = -q.w * q.y;
	float yz = q.y * q.z; 
	float wx = -q.w * q.x; 
	float xx = q.x * q.x; 
	float yy = q.y * q.y; 
	 
	float kx = 2*(xz - wy); 
	float ky = 2*(yz + wx); 
	float kz = 1 - 2 * (xx +yy); 
 
	float tpitch = -atan2f ( ky , sqrtf(kx*kx+kz*kz))*180/3.14159f; 
	float tyaw = -atan2f(kx,kz)*180/3.14159f; 
	
	float zz = q.z * q.z; 
	float xy = q.x * q.y; 
	float wz = -q.w * q.z; 
	float troll = atan2f(2*(xy-wz),1-2*(xx+zz))*180/3.14159f;

	tyaw = tyaw + yaw;
	tpitch = tpitch + pitch;
	troll = troll + roll;
	
	NxQuat q0 = NxQuat( NxVec3( 0,sinf(tyaw*3.14159f/360),0 ), -cosf(tyaw*3.14159f/360) ) * NxQuat(NxVec3( sinf(tpitch*3.14159f/-360),0,0 ), -cosf(tpitch*3.14159f/-360) ) * NxQuat( NxVec3( 0,0,sinf(troll*3.14159f/-360)), -cosf(troll*3.14159f/-360) );
	body->moveGlobalOrientationQuat(q0);
}
//----------Create Triger----------
class TriggerCallback : public NxUserTriggerReport
{
	public:
	virtual void onTrigger(NxShape& triggerShape, NxShape& otherShape, NxTriggerFlag status)
	{
		NxActor& triggerActor = triggerShape.getActor();
		NxActor& otherActor = otherShape.getActor();
		BodyData* data_trigger = (BodyData*)triggerActor.userData;
		BodyData* data_actor = (BodyData*)otherActor.userData;
//		data_actor->triggerFlag = status;

		if(status & NX_TRIGGER_ON_ENTER)
		{
			data_trigger->dTriggerActors.push_back(&otherActor);

		//	data_trigger->numTriggerActors++;
		//	data_trigger->triggerActors[data_trigger->numTriggerActors - 1] = &otherActor;
		}
		if(status & NX_TRIGGER_ON_LEAVE)
		{
			data_trigger->dTriggerActors.erase(std::find(data_trigger->dTriggerActors.begin(), data_trigger->dTriggerActors.end(), &otherActor));

		//	data_trigger->numTriggerActors--;
		}
		data_actor->Trigger = &triggerActor;
		gTrigersActive = 1;
	
	}

}myTriggerCallback;
BBDECL NxActor* BBCALL pxTriggerCreateCube(float dx, float dy, float dz)
{
	NxBodyDesc triggerBody;
	NxBoxShapeDesc BoxDesc;
	BoxDesc.dimensions = NxVec3(dx, dy, dz);
	BoxDesc.shapeFlags		|= NX_TRIGGER_ENABLE;
	triggerBody.flags |= NX_BF_KINEMATIC;

	NxActorDesc ActorDesc;
	ActorDesc.shapes.pushBack(&BoxDesc);

	triggerBody.mass = 1;
	ActorDesc.body = &triggerBody;
	NxBoxShapeDesc dummyShape;
	dummyShape.dimensions.set(0.1f, 0.1f, 0.1f);
	ActorDesc.shapes.pushBack(&dummyShape);

	NxActor* body = gScene->createActor(ActorDesc);
	body->raiseActorFlag(NX_AF_DISABLE_RESPONSE);
	mainSetMask(body);
	StaticBodyAddUserData(body);

	gScene->setUserTriggerReport(&myTriggerCallback);

	return body;
}
BBDECL NxActor* BBCALL pxTriggerCreateSphere(float radius)
{
	NxBodyDesc triggerBody;
	NxSphereShapeDesc BoxDesc;
	BoxDesc.radius = radius;
	BoxDesc.shapeFlags		|= NX_TRIGGER_ENABLE;
	triggerBody.flags |= NX_BF_KINEMATIC;

	NxActorDesc ActorDesc;
	ActorDesc.shapes.pushBack(&BoxDesc);

	triggerBody.mass = 1;
	ActorDesc.body = &triggerBody;
	NxBoxShapeDesc dummyShape;
	dummyShape.dimensions.set(0.1f, 0.1f, 0.1f);
	ActorDesc.shapes.pushBack(&dummyShape);

	NxActor* body = gScene->createActor(ActorDesc);
	body->raiseActorFlag(NX_AF_DISABLE_RESPONSE);
	mainSetMask(body);
	StaticBodyAddUserData(body);

	gScene->setUserTriggerReport(&myTriggerCallback);

	return body;
}
BBDECL NxActor* BBCALL pxTriggerCreateCylinder(float radius, float height, int nbEdge)
{
	NxBodyDesc triggerBody;
	triggerBody.flags |= NX_BF_KINEMATIC;

	NxActor* body;
	NxActorDesc ActorDesc;
	NxConvexShapeDesc ConvShapeDesc;
	ConvShapeDesc.shapeFlags	|= NX_TRIGGER_ENABLE;

	ActorDesc.shapes.pushBack(&ConvShapeDesc);

	triggerBody.mass = 1;
	ActorDesc.body = &triggerBody;
	NxBoxShapeDesc dummyShape;
	dummyShape.dimensions.set(0.1f, 0.1f, 0.1f);
	ActorDesc.shapes.pushBack(&dummyShape);
	
	int nbVerts = nbEdge;
	float alpha;
	float delta = 2*3.14f/nbVerts;
	float z=0;
	NxArray<NxVec3> points;

	if (radius == height) radius = radius - radius*0.01f;
	int parnost = nbEdge/2;
	for (int j=0; j<2; j++)
	{
		z = 2*height*j - height;
		if (nbEdge == 2*parnost) alpha = 3.14f/nbVerts+3.14f/4;
		if (nbEdge != 2*parnost)alpha = 3.14f/nbVerts-3.14f/2;
		for(int i=0; i<nbVerts; i++)
		{			
			points.pushBack().set(radius*cos(alpha), z, radius*sin(alpha));
			alpha = alpha + delta;
		}
	}
	
		NxConvexMeshDesc convexMesh;
		convexMesh.numVertices = points.size();
		convexMesh.points = &(points[0].x);
		convexMesh.pointStrideBytes = sizeof(NxVec3);
		convexMesh.flags = NX_CF_COMPUTE_CONVEX;

		MemoryWriteBuffer buf;
		bool status = NxCookConvexMesh(convexMesh, buf);
		if(status)
		{
			ConvShapeDesc.meshData = gPhysicsSDK->createConvexMesh(MemoryReadBuffer(buf.data));
			ActorDesc.shapes.pushBack(&ConvShapeDesc);
		}
			
		body = gScene->createActor(ActorDesc);
		mainSetMask(body);
		StaticBodyAddUserData(body);
		gScene->setUserTriggerReport(&myTriggerCallback);

	return body;
}
BBDECL NxActor* BBCALL pxTriggerCreateHull(float *vbank, int nvert)
{
	NxBodyDesc triggerBody;
	triggerBody.flags |= NX_BF_KINEMATIC;

	NxActor* body;
	NxActorDesc ActorDesc;
	NxConvexShapeDesc ConvShapeDesc;
	ConvShapeDesc.shapeFlags	|= NX_TRIGGER_ENABLE;

	ActorDesc.shapes.pushBack(&ConvShapeDesc);

	triggerBody.mass = 1;
	ActorDesc.body = &triggerBody;
	NxBoxShapeDesc dummyShape;
	dummyShape.dimensions.set(0.1f, 0.1f, 0.1f);
	ActorDesc.shapes.pushBack(&dummyShape);
	
	int nbVerts=nvert;
	NxVec3* verts = new NxVec3[nbVerts];
		for(int i=0;i<nbVerts;i++)
		{
			verts[i].x = vbank[i*3+0];
			verts[i].y = vbank[i*3+1];
			verts[i].z = vbank[i*3+2];
		}
	
	//NxU32 vertCount = nbVerts;
	NxConvexMeshDesc convexMesh;
	convexMesh.numVertices			= nbVerts;
	convexMesh.pointStrideBytes		= sizeof(NxVec3);
	convexMesh.points				= verts;
	convexMesh.flags				= NX_CF_COMPUTE_CONVEX;
	

		MemoryWriteBuffer buf;
		bool status = NxCookConvexMesh(convexMesh, buf);
		if(status)
		{
			ConvShapeDesc.meshData = gPhysicsSDK->createConvexMesh(MemoryReadBuffer(buf.data));
			ActorDesc.shapes.pushBack(&ConvShapeDesc);
		}
			
		body = gScene->createActor(ActorDesc);
		mainSetMask(body);
		StaticBodyAddUserData(body);
		gScene->setUserTriggerReport(&myTriggerCallback);
	
	return body;

}
BBDECL NxActor* BBCALL pxTriggerCreateCapsule(float height, float radius)
{

	NxActorDesc ActorDesc;

	NxCapsuleShapeDesc CapsuleDesc;
	CapsuleDesc.height = height;
	CapsuleDesc.radius = radius;
	CapsuleDesc.shapeFlags	|= NX_TRIGGER_ENABLE;
	
	NxBodyDesc triggerBody;
	triggerBody.flags |= NX_BF_KINEMATIC;

	ActorDesc.shapes.pushBack(&CapsuleDesc);

	triggerBody.mass = 1;
	ActorDesc.body = &triggerBody;
	NxBoxShapeDesc dummyShape;
	dummyShape.dimensions.set(0.1f, 0.1f, 0.1f);
	ActorDesc.shapes.pushBack(&dummyShape);
	
	ActorDesc.shapes.pushBack(&CapsuleDesc);

	NxActor* body = gScene->createActor(ActorDesc);
	mainSetMask(body);
	StaticBodyAddUserData(body);

	return body;
}
BBDECL int BBCALL pxTriggerGetNumBody(NxActor* trigger)
{
	BodyData* data = (BodyData*)trigger->userData;
//	return data->numTriggerActors;
	return data->dTriggerActors.size();
}
BBDECL NxActor* BBCALL pxTriggerGetBody(NxActor* trigger, int num)
{
	BodyData* data = (BodyData*)trigger->userData;
//	if (data->numTriggerActors <= num) return 0;
//	return data->triggerActors[num];
	return data->dTriggerActors[num];
}
BBDECL NxActor* BBCALL pxTriggerGetBodyTrigger(NxActor* body)
{
	BodyData* data = (BodyData*)body->userData;
	return data->Trigger;
}
BBDECL void BBCALL pxTriggerSetPosition(NxActor* body, float x, float y, float z)
{
	body->moveGlobalPosition(NxVec3(x, y, z));
}
BBDECL void BBCALL pxTriggerSetRotation(NxActor* body, float pitch, float yaw, float roll)
{
	NxQuat qo = NxQuat( NxVec3( 0,sinf(NxMath::degToRad(yaw*0.5f)),0 ), -cosf(NxMath::degToRad(yaw*0.5f)) ) 
		* NxQuat(NxVec3( sinf(NxMath::degToRad(-pitch*0.5f)),0,0 ), -cosf(NxMath::degToRad(-pitch*0.5f))) 
		* NxQuat( NxVec3( 0,0,sinf(NxMath::degToRad(-roll*0.5f))), -cosf(NxMath::degToRad(-roll*0.5f)) );

	body->moveGlobalOrientationQuat(qo);
}
BBDECL void BBCALL pxUpdateTriggers()
{
	NxActor** allactors = gScene->getActors();
	NxActor* TriggerAct;
	BodyData* data_trigger;

	int nbAllActors = gScene->getNbActors();
	if (gTrigersActive==1)
	{
		for (int i=0; i<nbAllActors; i++)
		{
			BodyData*data = (BodyData*)(allactors[i]->userData);
			if (data) 
			{
				data->numTriggerActors = 0;
			}
		}
		gTrigersActive = 0;
	}
	for (int i=0; i<nbAllActors; i++)
	{
		BodyData*data = (BodyData*)(allactors[i]->userData);
		if (data)
		{
			if (data->triggertable)
			{
				TriggerAct = data->Trigger;
				if (TriggerAct)
				{
					data_trigger = (BodyData*)(TriggerAct->userData);
					data_trigger->numTriggerActors ++;
//					if (data_trigger->numTriggerActors < 200)
//					data_trigger->triggerActors[data_trigger->numTriggerActors - 1] = allactors[i];
				}
			}
		}
	}
}
//----------Create Intersect----------
typedef struct pxIntersectDATA
{
	int iType;
	NxSphere* iBody;
	NxVec3 icenter;
	float iradius;

	NxBounds3* iBBody;
	NxVec3 iBpos;
	NxVec3 iBext;

	float* iEntity;

}IntersectDATA;
BBDECL IntersectDATA* BBCALL pxCreateIntersectSphere(float x, float y, float z, float radius)
{
	IntersectDATA* idata = new IntersectDATA;
	NxSphere worldSphere(NxVec3(0,0,0), 1);
	idata->iBody = new NxSphere(NxVec3(0,0,0),1);
	idata->icenter.set(x,y,z);
	idata->iradius = radius;
	idata->iType = 1;
	return idata;
}
BBDECL void BBCALL pxIntersectSphereSetPosition(IntersectDATA* idata, float x, float y, float z)
{
	idata->icenter.set(x,y,z);
}
BBDECL float BBCALL pxIntersectSphereGetPositionX(IntersectDATA* idata)
{
	return idata->icenter.x;
}
BBDECL float BBCALL pxIntersectSphereGetPositionY(IntersectDATA* idata)
{
	return idata->icenter.y;
}
BBDECL float BBCALL pxIntersectSphereGetPositionZ(IntersectDATA* idata)
{
	return idata->icenter.z;
}
BBDECL void BBCALL pxIntersectSphereSetRadius(IntersectDATA* idata, float radius)
{
	idata->iradius = radius;
}
BBDECL void BBCALL pxIntersectBoxSetPosition(IntersectDATA* idata, float x, float y, float z)
{
	idata->iBpos.set(x, y, z);
}
BBDECL void BBCALL pxIntersectBoxSetDimension(IntersectDATA* idata, float dx, float dy, float dz)
{
	idata->iBext.set(dx, dy, dz);
}
BBDECL float BBCALL pxIntersectBoxGetPositionX(IntersectDATA* idata)
{
	return idata->iBpos.x;
}
BBDECL float BBCALL pxIntersectBoxGetPositionY(IntersectDATA* idata)
{
	return idata->iBpos.y;
}
BBDECL float BBCALL pxIntersectBoxGetPositionZ(IntersectDATA* idata)
{
	return idata->iBpos.z;
}
BBDECL IntersectDATA* BBCALL pxCreateIntersectBox(float x, float y, float z, float dx, float dy, float dz)
{
	IntersectDATA* idata = new IntersectDATA;
	idata->iBBody = new NxBounds3;
	idata->iBpos.set(x, y, z);
	idata->iBext.set(dx, y, dz);
	idata->iType = 2;
	return idata;
}
BBDECL int BBCALL pxIntersectGetNb(IntersectDATA* idata)
{
	NxShapesType shapeType = NX_DYNAMIC_SHAPES;  // NX_STATIC_SHAPES, NX_ALL_SHAPES
	int nbIntersect = 0;
	int nbShapes = gScene->getNbDynamicShapes();  // + gScene->getNbStaticShapes()
    NxShape** shapes = (NxShape**)NxAlloca(nbShapes*sizeof(NxShape*));
	for (int i = 0; i < nbShapes; i++)  shapes[i] = NULL;
	if (idata->iType == 1)
	{
		idata->iBody->radius = idata->iradius;
		idata->iBody->center = idata->icenter;
		gScene->overlapSphereShapes(*idata->iBody, shapeType, nbShapes, shapes, 0, 0xffffffff, 0);
	}
	else if (idata->iType == 2)
	{
		idata->iBBody->set(idata->iBpos - idata->iBext, idata->iBpos + idata->iBext);
		gScene->overlapAABBShapes(*idata->iBBody, shapeType, nbShapes, shapes, 0, 0xffffffff, 0);
	}
	while (nbShapes--)
	{
		if (shapes[nbShapes]) 
		{	
			nbIntersect++;
		}
	}
	return nbIntersect;
}
BBDECL NxActor* BBCALL pxIntersectGetBody(IntersectDATA* idata, int nb)
{
	NxShapesType shapeType = NX_DYNAMIC_SHAPES;  // NX_STATIC_SHAPES, NX_ALL_SHAPES
	int nbShapes = gScene->getNbDynamicShapes();  // + gScene->getNbStaticShapes()
	int nbIntersect=0;
    NxShape** shapes = (NxShape**)NxAlloca(nbShapes*sizeof(NxShape*));
	for (int i = 0; i < nbShapes; i++)  shapes[i] = NULL;
	NxU32 activeGroups = 0xffffffff;
	NxGroupsMask* groupsMask = NULL;
	if (idata->iType == 1)
	{
		idata->iBody->radius = idata->iradius;
		idata->iBody->center = idata->icenter;
		gScene->overlapSphereShapes(*idata->iBody, shapeType, nbShapes, shapes, 0, 0xffffffff, 0);
	}
	else if (idata->iType == 2)
	{
		idata->iBBody->set(idata->iBpos - idata->iBext, idata->iBpos + idata->iBext);
		gScene->overlapAABBShapes(*idata->iBBody, shapeType, nbShapes, shapes, 0, 0xffffffff, 0);
	}
	while (nbShapes--)
	{
		if (shapes[nbShapes])
		{	nbIntersect++;
			if (nbIntersect == nb) return (NxActor*)&shapes[nbShapes]->getActor();
		}
	}
	return 0;
}
BBDECL void BBCALL pxIntersectSetEntity(IntersectDATA* idata, float* entity)
{
	idata->iEntity = entity;
}
BBDECL float* BBCALL pxIntersectGetEntity(IntersectDATA* idata)
{
	return idata->iEntity;
}
//----------Create Magnite----------
typedef struct pxMagnitDATA
{
	float x, y, z;
	float maxforce, middleforce, minforce;
	float distance;
	float stopradius;
	int mask;

}MagnitDATA;
BBDECL MagnitDATA* BBCALL pxCreateMagnet(float minforce, float middleforce, float maxforce)
{
	MagnitDATA* mdata = new MagnitDATA;
	mdata->x = 0;
	mdata->y = 0;
	mdata->z = 0;
	mdata->maxforce = maxforce;
	mdata->middleforce = middleforce;
	mdata->minforce = minforce;
	mdata->distance = 2;
	mdata->stopradius = 1;
	mdata->mask = 1;
	return mdata;
}
BBDECL void BBCALL pxMagnetActivate(MagnitDATA* mdata, int mmode, int fmode)
{
if (mdata)
{
	NxActor** allactors = gScene->getActors();
	int nbAllActors = gScene->getNbActors();
	for (int i=0; i<nbAllActors; i++)
	{
		BodyData* data = (BodyData*)allactors[i]->userData;
		if (data->StaticBody ==0 && data->magnitable == 1 && (data->magnetMask & mdata->mask))
		{
			NxVec3 bpos = allactors[i]->getGlobalPosition();
			float distance, nx, ny, nz;
			nx = (mdata->x - bpos.x);
			ny = (mdata->y - bpos.y);
			nz = (mdata->z - bpos.z);
			distance = sqrtf(nx*nx + ny*ny + nz*nz);
			if (distance < mdata->distance)
			{
				float dev_force;
				NxForceMode forcmode;
												
				if (fmode==0) forcmode = NX_FORCE;
				else if (fmode==1) forcmode = NX_IMPULSE;
				else if (fmode==2) forcmode = NX_VELOCITY_CHANGE;
				else if (fmode==3) forcmode = NX_SMOOTH_IMPULSE;
				else if (fmode==4) forcmode = NX_SMOOTH_VELOCITY_CHANGE;
				else if (fmode==5) forcmode = NX_ACCELERATION;
			
				if (mmode == 0) 
				{
					dev_force = mdata->minforce + mdata->middleforce/distance
						+ mdata->maxforce/(distance*distance);
					allactors[i]->addForce(NxVec3(nx*dev_force, ny*dev_force, nz*dev_force), forcmode);
				}
				else 
				if (mmode == 1)
				{
					if (distance < mdata->stopradius) 
					{
						allactors[i]->setLinearVelocity(NxVec3(0, 0, 0));
					}
					else
					if (distance > mdata->stopradius || distance == mdata->stopradius) 
					{
						dev_force = mdata->minforce + mdata->middleforce/distance
						+ mdata->maxforce/(distance*distance);	
						allactors[i]->addForce(NxVec3(nx*dev_force, ny*dev_force, nz*dev_force), forcmode);
					}
	
				}
				if (mmode == 2)
				{
					if (distance < mdata->stopradius) 
					{
						allactors[i]->setLinearVelocity(NxVec3(0, 0, 0));
					}
					else
					if (distance > mdata->stopradius || distance == mdata->stopradius) 
					{
						dev_force = mdata->minforce + mdata->middleforce/distance
						+ mdata->maxforce/(distance*distance);	
						allactors[i]->setLinearVelocity(NxVec3(nx*dev_force, ny*dev_force, nz*dev_force));
					}
					
				}
			}
		}
	}
}
}
BBDECL void BBCALL pxMagnetSetPosition(MagnitDATA* mdata, float pos_x,float pos_y,float pos_z)
{
	mdata->x = pos_x;
	mdata->y = pos_y;
	mdata->z = pos_z;
}
BBDECL void BBCALL pxMagnetSetMaxRadius(MagnitDATA* mdata, float radius)
{
	mdata->distance = radius;
}
BBDECL void BBCALL pxMagnetSetMinRadius(MagnitDATA* mdata, float sradius)
{
	mdata->stopradius = sradius;
}
BBDECL void BBCALL pxMagnetSetMaxForce(MagnitDATA* mdata, float force)
{
	mdata->maxforce = force;
}
BBDECL void BBCALL pxMagnetSetMidForce(MagnitDATA* mdata, float force)
{
	mdata->middleforce = force;
}
BBDECL void BBCALL pxMagnetSetMinForce(MagnitDATA* mdata, float force)
{
	mdata->minforce = force;
}
BBDECL void BBCALL pxMagnetSetMask(MagnitDATA* mdata, int mask)
{
	mdata->mask = mask;
}
BBDECL float BBCALL pxMagnetGetPositionX(MagnitDATA* mdata)
{
	return mdata->x;
}
BBDECL float BBCALL pxMagnetGetPositionY(MagnitDATA* mdata)
{
	return mdata->y;
}
BBDECL float BBCALL pxMagnetGetPositionZ(MagnitDATA* mdata)
{
	return mdata->z;
}
BBDECL float BBCALL pxMagnetGetMaxRadius(MagnitDATA* mdata)
{
	return mdata->distance;
}
BBDECL float BBCALL pxMagnetGetMinRadius(MagnitDATA* mdata)
{
	return mdata->stopradius;
}
BBDECL float BBCALL pxMagnetGetMaxForce(MagnitDATA* mdata)
{
	return mdata->maxforce;
}
BBDECL float BBCALL pxMagnetGetMidForce(MagnitDATA* mdata)
{
	return mdata->middleforce;
}
BBDECL float BBCALL pxMagnetGetMinForce(MagnitDATA* mdata)
{
	return mdata->minforce;
}
BBDECL int BBCALL pxMagnetGetMask(MagnitDATA* mdata)
{
	return mdata->mask;
}
BBDECL void BBCALL pxBodySetFlagMagniteble(NxActor* body, int stat)
{
	BodyData* data = (BodyData*)body->userData;
	data->magnitable = stat;
}
BBDECL void BBCALL pxBodySetFlagTriggertable(NxActor* body, int stat)
{
	BodyData* data = (BodyData*)body->userData;
	data->triggertable = stat;
}
BBDECL void BBCALL pxMagnetDelete(MagnitDATA* mdata)
{
	delete mdata;
	mdata = 0;
}
//----------Create WaterPlane----------
typedef struct pxWaterDATA
{
	int type;
	NxVec3 pos;
	float depth;
	float radius;

	float width, height;
	float point_x, point_y;
	float angle;

	NxVec3 fluxion;
}WaterDATA;
BBDECL WaterDATA* BBCALL pxCreateWaterCirPlane(float radius, float depth)
{
	WaterDATA* water = new WaterDATA;
	water->type = 1;
	water->pos.set(0, 0, 0);
	water->radius = radius;
	water->depth = depth;

	water->width = 0;
	water->height = 0;
	water->angle = 0;
	water->point_x = 0;
	water->point_y = 0;

	water->fluxion.set(0,0,0);

	return water;
}
BBDECL WaterDATA* BBCALL pxCreateWaterRectPlane(float width, float height, float depth)
{
	WaterDATA* water = new WaterDATA;
	water->type = 2;
	water->pos.set(0, 0, 0);
	water->radius = 0;
	water->depth = depth;
	water->fluxion.set(0,0,0);

	water->width = width;
	water->height = height;
	water->angle = 0;

	water->point_x = water->pos.x - water->width/2;
	water->point_y = water->pos.z - water->height/2;
	float x2 = water->point_x + water->height*cos(NxMath::degToRad(water->angle+90));
	float y2 = water->point_y + water->height*sin(NxMath::degToRad(water->angle+90));
	float x3 = x2 + water->width*cos(NxMath::degToRad(water->angle));
	float y3 = y2 + water->width*sin(NxMath::degToRad(water->angle));
	float cx0 = (x3 - water->point_x)/2;
	float cy0 = (y3 - water->point_y)/2;
	water->point_x = water->point_x - cx0 + width/2;
	water->point_y = water->point_y - cy0 + height/2;

	return water;
}
BBDECL WaterDATA* BBCALL pxCreateWaterInfinPlane(float depth)
{
	WaterDATA* water = new WaterDATA;
	water->type = 3;
	water->pos.set(0, 0, 0);
	water->radius = 0;
	water->depth = depth;
	water->fluxion.set(0,0,0);

	water->width = 0;
	water->height = 0;
	water->angle = 0;
	water->point_x = 0;
	water->point_y = 0;

	return water;
}
BBDECL void BBCALL pxWaterSetDimension(WaterDATA* water, float width, float height)
{
	water->width = width;
	water->height = height;
}
BBDECL void BBCALL pxWaterSetRadius(WaterDATA* water, float radius)
{
	water->radius = radius;
}
BBDECL void BBCALL pxWaterSetPosition(WaterDATA* water, float pos_x, float pos_y, float pos_z)
{
	water->pos.set(pos_x, pos_y, pos_z);

	water->point_x = water->pos.x - water->width/2;
	water->point_y = water->pos.z - water->height/2;
	float x2 = water->point_x + water->height*cos(NxMath::degToRad(water->angle+90));
	float y2 = water->point_y + water->height*sin(NxMath::degToRad(water->angle+90));
	float x3 = x2 + water->width*cos(NxMath::degToRad(water->angle));
	float y3 = y2 + water->width*sin(NxMath::degToRad(water->angle));
	float cx0 = (x3 - water->point_x)/2;
	float cy0 = (y3 - water->point_y)/2;
	water->point_x = water->point_x - cx0 + water->width/2;
	water->point_y = water->point_y - cy0 + water->height/2;
}
BBDECL void BBCALL pxWaterSetRotation(WaterDATA* water, float angle)
{
	water->angle = angle;

	water->point_x = water->pos.x - water->width/2;
	water->point_y = water->pos.z - water->height/2;
	float x2 = water->point_x + water->height*cos(NxMath::degToRad(water->angle+90));
	float y2 = water->point_y + water->height*sin(NxMath::degToRad(water->angle+90));
	float x3 = x2 + water->width*cos(NxMath::degToRad(water->angle));
	float y3 = y2 + water->width*sin(NxMath::degToRad(water->angle));
	float cx0 = (x3 - water->point_x)/2;
	float cy0 = (y3 - water->point_y)/2;
	water->point_x = water->point_x - cx0 + water->width/2;
	water->point_y = water->point_y - cy0 + water->height/2;
}
BBDECL void BBCALL pxWaterSetFluxion(WaterDATA* water, float fl_x, float fl_y, float fl_z)
{
	water->fluxion.set(fl_x, fl_y, fl_z);
}
BBDECL float BBCALL pxWaterGetWidth(WaterDATA* water)
{
	return water->width;
}
BBDECL float BBCALL pxWaterGetHeight(WaterDATA* water)
{
	return water->height;
}
BBDECL float BBCALL pxWaterGetRadius(WaterDATA* water)
{
	return water->radius;
}
BBDECL float BBCALL pxWaterGetPositionX(WaterDATA* water)
{
	return water->pos.x;
}
BBDECL float BBCALL pxWaterGetPositionY(WaterDATA* water)
{
	return water->pos.y;
}
BBDECL float BBCALL pxWaterGetPositionZ(WaterDATA* water)
{
	return water->pos.z;
}
BBDECL float BBCALL pxWaterGetRotation(WaterDATA* water)
{
	return water->angle;
}
BBDECL float BBCALL pxWaterGetDepth(WaterDATA* water)
{
	return water->depth;
}
BBDECL void BBCALL pxWaterDelete(WaterDATA* water)
{
	delete water;
	water = 0;
}
//-------------------------------------
BBDECL KepDATA* BBCALL pxCreateKep(float buo, float radius, float maxdis)
{
	KepDATA* kep = new KepDATA;
	kep->pos.set(0, 0, 0);
	kep->locpos.set(0, 0, 0);
	kep->buoyancy = buo;
	kep->angdamp = 10;
	kep->lindamp = 10;
	kep->maxdistance = maxdis;
	kep->radius = radius;
	kep->entity = NULL;

	return kep;
}
BBDECL void BBCALL pxKepSetLocalPosition(KepDATA* kdata, float pos_x, float pos_y, float pos_z)
{
	kdata->locpos.set(pos_x, pos_y, pos_z);

}
BBDECL void BBCALL pxKepSetGlobalPosition(KepDATA* kdata, float pos_x, float pos_y, float pos_z)
{
	NxVec3 absChild, absParent;
	absChild.set(pos_x, pos_y, pos_z);
	absParent = kdata->entity->getGlobalPosition();

	NxVec3 locChild = absChild - absParent;
	kdata->locpos = locChild;
}
BBDECL void BBCALL pxKepSetAngularDamping(KepDATA* kdata, float angdamp)
{
	kdata->angdamp = angdamp;
}
BBDECL void BBCALL pxKepSetLinearDamping(KepDATA* kdata, float lindamp)
{
	kdata->lindamp = lindamp;
}
BBDECL void BBCALL pxKepAddToBody(KepDATA* kdata, NxActor* body)
{
	kdata->entity = body;
	BodyData* data = (BodyData*)body->userData;
	data->Keps[data->nkeps] = kdata;
	data->nkeps = data->nkeps+1;
}
static void pxKepUpdate(NxActor* actor, WaterDATA* water, NxVec3 absParent)
{
	float water_pos_y = water->pos.y;
	BodyData* data = (BodyData*)actor->userData;
	int nAllkeps = data->nkeps;
	KepDATA** allkeps = data->Keps;
	//NxVec3 gPosition = NxVec3(0,0,0);
	//NxVec3 gForce = NxVec3(0,0,0);

	for (int j=0; j<nAllkeps; j++)
	{
		NxVec3 locChild, absChild;
		NxQuat quatParent; 
		absChild = allkeps[j]->pos;
		locChild = allkeps[j]->locpos;
		quatParent = actor->getGlobalOrientationQuat();

		quatParent = -quatParent;
		quatParent.rotate(locChild);
		absChild = absParent + locChild;	
		allkeps[j]->pos = absChild;

		NxVec3 force = NxVec3(0,0,0);
		NxVec3 torque = NxVec3(0,0,0);
		NxVec3 antiforce = NxVec3(0,0,0), antiangforce = NxVec3(0,0,0);
	
		if (absChild.y <= water_pos_y + allkeps[j]->radius && absChild.y >= water_pos_y - water->depth)
		{

			antiforce = actor->getLinearVelocity() + water->fluxion;
			antiangforce = actor->getAngularVelocity();

			if (absChild.y >= water_pos_y - allkeps[j]->maxdistance + allkeps[j]->radius)
			{
				force.y = (water_pos_y - absChild.y + allkeps[j]->radius)*allkeps[j]->buoyancy/allkeps[j]->maxdistance;
			}
			if (absChild.y < water_pos_y-allkeps[j]->maxdistance + allkeps[j]->radius) 
			{
				force.y = allkeps[j]->buoyancy;
			}
			//gPosition = gPosition + absChild;
			force = force - antiforce*allkeps[j]->lindamp;
			torque = -antiangforce*allkeps[j]->angdamp;
			if (gDeltaTime >=70) 
			{
				force = 100*force/gDeltaTime;
				torque = 100*torque/gDeltaTime;
			}
			else //if (gDeltaTime <70) 
			{
				force = 1.6f*force;
				torque = 1.8f*torque;
			}

		//	actor->addForceAtPos(NxVec3( - antiforce.x*allkeps[j]->lindamp, 
		//		force - antiforce.y*allkeps[j]->lindamp,
		//		- antiforce.z*allkeps[j]->lindamp), absChild, NX_FORCE );
			actor->addForceAtPos(force, absChild, NX_FORCE );
			actor->addTorque(torque,NX_FORCE );
		}
//		gForce = gForce + NxVec3( - antiforce.x*allkeps[j]->lindamp, 
//			force - antiforce.y*allkeps[j]->lindamp,
//			- antiforce.z*allkeps[j]->lindamp);
	}
//	actor->addForceAtPos(gForce, gPosition, NX_FORCE );
}
static bool pxPointInRect(WaterDATA* water, NxVec3 absParent)
{
	float x = water->point_x;
	float y = water->point_y;
	float px = absParent.x;
	float py = absParent.z;
	float len = sqrtf((px - x)*(px - x)	+ (py - y)*(py - y));
	float ang2 = atan2f(py - y, px - x);
	float px2 = x + len*cos(ang2 - NxMath::degToRad(water->angle));
	float py2 = y + len*sin(ang2 - NxMath::degToRad(water->angle));
	if (px2 > x + water->width || px2 < x || py2 > y + water->height || py2 < y) return false;
	else return true;
}
BBDECL void BBCALL pxWaterUpdate(WaterDATA* water)
{
	float water_pos_y = water->pos.y;

	NxActor** allactors = gScene->getActors();
	int nbAllActors = gScene->getNbActors();
	for (int i=0; i<nbAllActors; i++)
	{
		BodyData* data = (BodyData*)allactors[i]->userData;
		if (data->StaticBody == 0 && allactors[i]->isDynamic())
		{
			NxVec3 absParent = allactors[i]->getGlobalPosition();
			float water_distance = (water->pos.x-absParent.x)*(water->pos.x-absParent.x)
				+ (water->pos.z-absParent.z)*(water->pos.z-absParent.z);

			if (water->type == 1)
			{
				if (water_distance <= water->radius*water->radius)
				{
					pxKepUpdate(allactors[i], water, absParent);
				}
			}
			else if (water->type == 2)
			{
				if (pxPointInRect(water, absParent))
				{
					pxKepUpdate(allactors[i], water, absParent);
				}
			}
			else if (water->type == 3)
			{
				pxKepUpdate(allactors[i], water, absParent);
			}
		}
	}
}
BBDECL float BBCALL pxKepGetPosX(KepDATA* kdata)
{
	return kdata->pos.x;
}
BBDECL float BBCALL pxKepGetPosY(KepDATA* kdata)
{
	return kdata->pos.y;
}
BBDECL float BBCALL pxKepGetPosZ(KepDATA* kdata)
{
	return kdata->pos.z;
}
BBDECL int BBCALL pxKepGetNumber(NxActor* body)
{
	BodyData* data = (BodyData*)body->userData;
	return data->nkeps;
}
BBDECL KepDATA* BBCALL pxKepGetKepFromBody(NxActor* body, int num)
{
	BodyData* data = (BodyData*)body->userData;
	return data->Keps[num];
}
//--------------------Create Cloth---------------------
typedef struct _BBClothVert
{
	float x,y,z;

} BBClothVert, *LBClothVert;
typedef struct tagClothDATA
{
	unsigned long *surf;
	MyCloth *mCloth;
	int BlitzData;
}ClothData;
static void ClothAddUserData(NxCloth* cloth)
{
	ClothData* data = new ClothData;
//	data->surf = 1;
//	cloth->userData = data;
}
BBDECL NxCloth* BBCALL pxCreateCloth(float *entity, unsigned long *surf)
{
	NxQuat q;
	q.setx(entity[13]);
	q.sety(entity[14]);
	q.setz(entity[15]);
	q.setw(-entity[12]);
	
	NxClothDesc clothDesc;
	clothDesc.globalPose.t = NxVec3(entity[16],entity[17],entity[18]);
	clothDesc.globalPose.M.fromQuat(q);
	clothDesc.thickness = 0.8f;
	clothDesc.friction = 0.5f;
	if (gHardwareSimulation)
		clothDesc.flags |= NX_CLF_HARDWARE;

	clothDesc.flags |= NX_CLF_COLLISION_TWOWAY;
		
	entity[16]=0;
	entity[17]=0;
	entity[18]=0;
	entity[12]=0;
	entity[13]=0;
	entity[14]=0;
	entity[15]=0;

	LPBBVERTEX OldVertex;
	OldVertex = (struct _BBVERTEX *)*(surf+7);

	MyCloth *regularCloth = new MyCloth(gScene, clothDesc, surf, entity,0,0,0,0);
	regularCloth->entityData = entity;
	regularCloth->surfData = surf;
	regularCloth->OldVertex = OldVertex;
	regularCloth->coreActor = 0;
//	gCloths.push_back(regularCloth);
	
	NxCloth* cloth = regularCloth->getNxCloth();
	NxGroupsMask GrMask;
	GrMask.bits0 = 1;
	GrMask.bits1 = 1;
	GrMask.bits2 = 1;
	GrMask.bits3 = 1;
	cloth->setGroupsMask (GrMask);

	ClothData* data = new ClothData;
	data->surf = surf;
	data->mCloth = regularCloth;
	cloth->userData = data;

	return cloth;
}
BBDECL NxCloth* BBCALL pxCreateTearableCloth(float *entity, unsigned long *surf)
{
	NxQuat q;
	q.setx(entity[13]);
	q.sety(entity[14]);
	q.setz(entity[15]);
	q.setw(-entity[12]);
	
	NxClothDesc clothDesc;
	clothDesc.globalPose.t = NxVec3(entity[16],entity[17],entity[18]);
	clothDesc.globalPose.M.fromQuat(q);
	clothDesc.thickness = 0.8f;
	clothDesc.friction = 0.5f;

	//clothDesc.flags |= NX_CLF_BENDING;
	//clothDesc.flags |= NX_CLF_BENDING_ORTHO;
	clothDesc.flags |= NX_CLF_COLLISION_TWOWAY;
	clothDesc.flags |= NX_CLF_TEARABLE;
	if (gHardwareSimulation)
		clothDesc.flags |= NX_CLF_HARDWARE;
		
	entity[16]=0;
	entity[17]=0;
	entity[18]=0;
	entity[12]=0;
	entity[13]=0;
	entity[14]=0;
	entity[15]=0;

	LPBBVERTEX OldVertex;
	OldVertex = (struct _BBVERTEX *)*(surf+7);

	MyCloth *regularCloth = new MyCloth(gScene, clothDesc, surf, entity,0,0,0,0);
	regularCloth->entityData = entity;
	regularCloth->surfData = surf;
	regularCloth->OldVertex = OldVertex;
	regularCloth->coreActor = 0;
//	gCloths.push_back(regularCloth);
	
	NxCloth* cloth = regularCloth->getNxCloth();//gScene->getCloths();
	cloth->userData = surf;
	NxGroupsMask GrMask;
	GrMask.bits0 = 1;
	GrMask.bits1 = 1;
	GrMask.bits2 = 1;
	GrMask.bits3 = 1;
	cloth->setGroupsMask (GrMask);

	ClothData* data = new ClothData;
	data->surf = surf;
	data->mCloth = regularCloth;
	cloth->userData = data;

	return cloth;
}
BBDECL NxCloth* BBCALL pxCreateMetalCloth(float *entity, unsigned long *surf, NxActor *coreActor, float impThr, float depth)
{
	NxQuat q;
	q.setx(entity[13]);
	q.sety(entity[14]);
	q.setz(entity[15]);
	q.setw(-entity[12]);

	NxClothDesc clothDesc;
	clothDesc.globalPose.t = NxVec3(entity[16],entity[17],entity[18]);
	clothDesc.globalPose.M.fromQuat(q);
	clothDesc.thickness = 0.1f;
	clothDesc.friction = 0.5f;
	//clothDesc.flags |= NX_CLF_BENDING;
	if (gHardwareSimulation)
		clothDesc.flags |= NX_CLF_HARDWARE;

	entity[16]=0;
	entity[17]=0;
	entity[18]=0;
	entity[12]=0;
	entity[13]=0;
	entity[14]=0;
	entity[15]=0;

	LPBBVERTEX OldVertex;
	OldVertex = (struct _BBVERTEX *)*(surf+7);

	// global metal data
	NxReal impulseThreshold  = impThr;
	NxReal penetrationDepth  = depth;

	// create the shape, no need for size info

	MyCloth *regularCloth = new MyCloth(gScene, clothDesc, surf, entity,0,0,0,0);
	if (!regularCloth->getNxCloth())
	{
		return 0;
	} 
	else
	{
		regularCloth->getNxCloth()->attachToCore(coreActor, impulseThreshold, penetrationDepth); 
	}

	regularCloth->entityData = entity;
	regularCloth->surfData = surf;
	regularCloth->coreActor = coreActor;
	regularCloth->OldVertex = OldVertex;
//	gCloths.push_back(regularCloth);
	
	NxCloth* cloth = regularCloth->getNxCloth();
	NxGroupsMask GrMask;
	GrMask.bits0 = 1;
	GrMask.bits1 = 1;
	GrMask.bits2 = 1;
	GrMask.bits3 = 1;
	cloth->setGroupsMask (GrMask);

	ClothData* data = new ClothData;
	data->surf = surf;
	data->mCloth = regularCloth;
	cloth->userData = data;

	return cloth;
}
BBDECL NxCloth* BBCALL pxCreateClothSpec(float *vbank,int *fbank,int MESH_NBVERTICES,int MESH_NBFACES)
{
	
	NxClothDesc clothDesc;
	clothDesc.globalPose.t = NxVec3(0,0,0);
	//clothDesc.globalPose.M.fromQuat(q);
	clothDesc.thickness = 0.8f;
	clothDesc.friction = 0.5f;
	if (gHardwareSimulation)
		clothDesc.flags |= NX_CLF_HARDWARE;

	clothDesc.flags |= NX_CLF_COLLISION_TWOWAY;
		
	MyCloth *regularCloth = new MyCloth(gScene, clothDesc, 0,0, vbank, fbank, MESH_NBVERTICES, MESH_NBFACES);
	regularCloth->entityData = 0;
	regularCloth->surfData = 0;
	regularCloth->OldVertex = 0;
	regularCloth->coreActor = 0;
//	gCloths.push_back(regularCloth);
	
	NxCloth* cloth = regularCloth->getNxCloth();
	NxGroupsMask GrMask;
	GrMask.bits0 = 1;
	GrMask.bits1 = 1;
	GrMask.bits2 = 1;
	GrMask.bits3 = 1;
	cloth->setGroupsMask (GrMask);

	ClothData* data = new ClothData;
	data->surf = 0;
	data->mCloth = regularCloth;
	cloth->userData = data;

	return cloth;
}
BBDECL NxCloth* BBCALL pxCreateTearebleClothSpec(float *vbank,int *fbank,int MESH_NBVERTICES,int MESH_NBFACES)
{
	
	NxClothDesc clothDesc;
	clothDesc.globalPose.t = NxVec3(0,0,0);
	//clothDesc.globalPose.M.fromQuat(q);
	clothDesc.thickness = 0.8f;
	clothDesc.friction = 0.5f;

	clothDesc.flags |= NX_CLF_COLLISION_TWOWAY;
	clothDesc.flags |= NX_CLF_TEARABLE;
	if (gHardwareSimulation)
		clothDesc.flags |= NX_CLF_HARDWARE;
		
	MyCloth *regularCloth = new MyCloth(gScene, clothDesc, 0,0, vbank, fbank, MESH_NBVERTICES, MESH_NBFACES);
	regularCloth->entityData = 0;
	regularCloth->surfData = 0;
	regularCloth->OldVertex = 0;
	regularCloth->coreActor = 0;
//	gCloths.push_back(regularCloth);
	
	NxCloth* cloth = regularCloth->getNxCloth();
	NxGroupsMask GrMask;
	GrMask.bits0 = 1;
	GrMask.bits1 = 1;
	GrMask.bits2 = 1;
	GrMask.bits3 = 1;
	cloth->setGroupsMask (GrMask);

	ClothData* data = new ClothData;
	data->surf = 0;
	data->mCloth = regularCloth;
	cloth->userData = data;

	return cloth;
}
BBDECL NxCloth* BBCALL pxCreateMetalClothSpec(float *vbank,int *fbank,int MESH_NBVERTICES,int MESH_NBFACES, NxActor *coreActor, float impThr, float depth)
{
	
	NxClothDesc clothDesc;
	clothDesc.globalPose.t = NxVec3(0,0,0);
	//clothDesc.globalPose.M.fromQuat(q);
	clothDesc.thickness = 0.8f;
	clothDesc.friction = 0.5f;
	if (gHardwareSimulation)
		clothDesc.flags |= NX_CLF_HARDWARE;

	clothDesc.flags |= NX_CLF_COLLISION_TWOWAY;

	NxReal impulseThreshold  = impThr;
	NxReal penetrationDepth  = depth;

	MyCloth *regularCloth = new MyCloth(gScene, clothDesc, 0,0, vbank, fbank, MESH_NBVERTICES, MESH_NBFACES);
	if (!regularCloth->getNxCloth())
	{
		return 0;
	} 
	else
	{
		regularCloth->getNxCloth()->attachToCore(coreActor, impulseThreshold, penetrationDepth); 
	}
	regularCloth->entityData = 0;
	regularCloth->surfData = 0;
	regularCloth->OldVertex = 0;
	regularCloth->coreActor = coreActor;
//	gCloths.push_back(regularCloth);
	
	NxCloth* cloth = regularCloth->getNxCloth();
	NxGroupsMask GrMask;
	GrMask.bits0 = 1;
	GrMask.bits1 = 1;
	GrMask.bits2 = 1;
	GrMask.bits3 = 1;
	cloth->setGroupsMask (GrMask);

	ClothData* data = new ClothData;
	data->surf = 0;
	data->mCloth = regularCloth;
	cloth->userData = data;

	return cloth;
}
BBDECL float BBCALL pxClothGetAttachmentResponseCoefficient(NxCloth* Cloth)
{
	float coef = Cloth->getAttachmentResponseCoefficient(); 
	return coef;
}
BBDECL float BBCALL pxClothGetAttachmentTearFactor(NxCloth* Cloth)
{
	float coef = Cloth->getAttachmentTearFactor(); 
	return coef;
}
BBDECL float BBCALL pxClothGetBendingStiffness(NxCloth* Cloth)
{
	float coef = Cloth->getBendingStiffness(); 
	return coef;
}
BBDECL float BBCALL pxClothGetCollisionResponseCoefficient(NxCloth* Cloth)
{
	float coef = Cloth->getCollisionResponseCoefficient (); 
	return coef;
}
BBDECL float BBCALL pxClothGetDampingCoefficient(NxCloth* Cloth)
{
	float coef = Cloth->getDampingCoefficient(); 
	return coef;
}
BBDECL float BBCALL pxClothGetDensity(NxCloth* Cloth)
{
	float coef = Cloth->getDensity(); 
	return coef;
}
BBDECL float BBCALL pxClothGetFriction(NxCloth* Cloth)
{
	float coef = Cloth->getFriction(); 
	return coef;
}
BBDECL float BBCALL pxClothGetPressure(NxCloth* Cloth)
{
	float coef = Cloth->getPressure(); 
	return coef;
}
BBDECL float BBCALL pxClothGetSleepLinearVelocity(NxCloth* Cloth)
{
	float coef = Cloth->getSleepLinearVelocity(); 
	return coef;
}
BBDECL float BBCALL pxClothGetStretchingStiffness(NxCloth* Cloth)
{
	float coef = Cloth->getStretchingStiffness(); 
	return coef;
}
BBDECL float BBCALL pxClothGetTearFactor(NxCloth* Cloth)
{
	float coef = Cloth->getTearFactor(); 
	return coef;
}
BBDECL float BBCALL pxClothGetThickness(NxCloth* Cloth)
{
	float coef = Cloth->getThickness(); 
	return coef;
}
BBDECL float* BBCALL pxClothGetEntity(NxCloth* Cloth)
{
	ClothData* data = (ClothData*)Cloth->userData;
	return data->mCloth->entityData;
}
BBDECL int BBCALL pxClothIsSleeping(NxCloth* Cloth)
{
	int sleep;
	if (Cloth->isSleeping() == 1) sleep = 1; 
	if (Cloth->isSleeping() == 0) sleep = 0; 
	return sleep;
}
BBDECL void BBCALL pxClothPutToSleep(NxCloth* Cloth)
{
	Cloth->putToSleep(); 
}
BBDECL void BBCALL pxClothWakeUp(NxCloth* Cloth)
{
	Cloth->wakeUp(1); 
}
BBDECL int BBCALL pxClothGetUserData(NxCloth* Cloth)
{
	ClothData* data = (ClothData*)Cloth->userData;
	return data->BlitzData;
}
BBDECL int BBCALL pxClothGetNumVertices(NxCloth* Cloth)
{
	return  Cloth->getNumberOfParticles();
}
//------------------------
BBDECL void BBCALL pxClothSetAttachmentResponseCoefficient(NxCloth* Cloth, float coef)
{
	Cloth->setAttachmentResponseCoefficient(coef); 
}
BBDECL void BBCALL pxClothSetAttachmentTearFactor(NxCloth* Cloth, float coef)
{
	Cloth->setAttachmentTearFactor(coef); 
}
BBDECL void BBCALL pxClothSetBending(NxCloth* Cloth)
{
	Cloth->setFlags(Cloth->getFlags()|NX_CLF_BENDING);
}
BBDECL void BBCALL pxClothSetBendingStiffness(NxCloth* Cloth, float coef)
{
	Cloth->setFlags(Cloth->getFlags()|NX_CLF_BENDING);
	Cloth->setBendingStiffness(coef); 
}
BBDECL void BBCALL pxClothSetCollisionResponseCoefficient(NxCloth* Cloth, float coef)
{
	//Cloth->setFlags(Cloth->getFlags()|NX_CLF_DISABLE_COLLISION);
	Cloth->setCollisionResponseCoefficient (coef); 
}
BBDECL void BBCALL pxClothSetDampingCoefficient(NxCloth* Cloth, float coef)
{
	Cloth->setFlags(Cloth->getFlags()|NX_CLF_DAMPING);
	Cloth->setDampingCoefficient(coef); 
}
BBDECL void BBCALL pxClothSetComDampingCoefficient(NxCloth* Cloth, float coef)
{
	Cloth->setFlags(Cloth->getFlags()|NX_CLF_DAMPING|NX_CLF_COMDAMPING);
	Cloth->setDampingCoefficient(coef); 
}
BBDECL void BBCALL pxClothSetFriction(NxCloth* Cloth, float coef)
{
	Cloth->setFriction(coef); 
}
BBDECL void BBCALL pxClothSetPressure(NxCloth* Cloth, float coef)
{
	Cloth->setFlags(Cloth->getFlags()|NX_CLF_PRESSURE);
	Cloth->setPressure(coef); 
}
BBDECL void BBCALL pxClothSetSleepLinearVelocity(NxCloth* Cloth, float coef)
{
	Cloth->setSleepLinearVelocity(coef); 
}
BBDECL void BBCALL pxClothSetStretchingStiffness(NxCloth* Cloth, float coef)
{
	Cloth->setStretchingStiffness(coef); 
}
BBDECL void BBCALL pxClothSetThickness(NxCloth* Cloth, float coef)
{
	Cloth->setThickness(coef); 
}

BBDECL void BBCALL pxClothsSetCollision(NxCloth* Cloth, int stat)
{
     if (stat==1) Cloth->setFlags(Cloth->getFlags()|NX_CLF_DISABLE_COLLISION);
     if (stat==0) Cloth->setFlags(Cloth->getFlags()&(~NX_CLF_DISABLE_COLLISION));
}
BBDECL void BBCALL pxClothsSetGravity(NxCloth* Cloth, int stat)
{
     if (stat==1) Cloth->setFlags(Cloth->getFlags()|NX_CLF_GRAVITY);
     if (stat==0) Cloth->setFlags(Cloth->getFlags()&(~NX_CLF_GRAVITY));
}
BBDECL void BBCALL pxClothMaskSet(NxCloth* Cloth, int mask)
{
	NxGroupsMask GrMask;//bits0, bits1, bits2, bits3;
	if (mask==0) 
	{
		GrMask.bits0 = 1;
		GrMask.bits1 = 0;
		GrMask.bits2 = 0;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==1) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 1;
		GrMask.bits2 = 0;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==2) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 0;
		GrMask.bits2 = 1;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==3) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 0;
		GrMask.bits2 = 0;
		GrMask.bits3 = 1;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}

	Cloth->setGroupsMask (GrMask);
}
BBDECL void BBCALL pxClothMaskCombineSet(NxCloth* Cloth, int mask)
{
	NxGroupsMask GrMask;
	GrMask.bits0 = mask & 1;
	GrMask.bits1 = mask & 2;
	GrMask.bits2 = mask & 4;
	GrMask.bits3 = mask & 8;
	Cloth->setGroupsMask (GrMask);
}

BBDECL void BBCALL pxClothSetUserData(NxCloth* Cloth, int userdata)
{
	ClothData* data = (ClothData*)Cloth->userData;
	data->BlitzData = userdata;
}
BBDECL void BBCALL pxClothSetGravity(NxCloth* Cloth, int stat)
{
	if (stat==1) Cloth->setFlags(Cloth->getFlags()|NX_CLF_GRAVITY);
	if (stat==0) Cloth->setFlags(Cloth->getFlags()&(~NX_CLF_GRAVITY));
}
//------------------------
BBDECL void BBCALL pxClothSetTeareble(NxCloth* Cloth)
{
	Cloth->setFlags(Cloth->getFlags() & (~NX_CLF_TEARABLE));
}
BBDECL void BBCALL pxClothSetTearFactor(NxCloth* Cloth, float coef)
{
	Cloth->setTearFactor(coef); 
}
//------------------------
BBDECL void BBCALL pxClothSetVertexPos(NxCloth* Cloth, float *vbank, int numVert)
{
	Cloth->putToSleep();
	NxVec3* clVert = new NxVec3[4*numVert];
	clVert = (NxVec3*)vbank;
	Cloth->setPositions(clVert); 
	Cloth->wakeUp();
	delete [] clVert;
}
BBDECL void BBCALL pxClothGetVertexPos(NxCloth* Cloth, float *vbank, int numVert)
{
	NxVec3* clVert = new NxVec3[4*numVert];
	Cloth->getPositions (clVert);  
	for (int i=0; i<numVert; i++)
	{
		vbank[i*3+0] = clVert[i].x;
		vbank[i*3+1] = clVert[i].y;
		vbank[i*3+2] = clVert[i].z;
	}
	delete [] clVert;
}
BBDECL void BBCALL pxClothGetNormals(NxCloth* Cloth, float *nbank)
{
	if (Cloth)
	{
		ClothData* data = (ClothData*)Cloth->userData;
		MyCloth* mCloth = data->mCloth;
		mCloth->updateNormals(nbank);
	}
}
BBDECL void BBCALL pxClothAttachVertexToPos(NxCloth* Cloth,int vID, float pos_x, float pos_y, float pos_z)
{
	Cloth->attachVertexToGlobalPosition (vID, NxVec3(pos_x,pos_y,pos_z)); 
}
BBDECL void BBCALL pxClothAttachToCollidingShapes(NxCloth* Cloth, int flag)
{
	if (flag == 0)Cloth->attachToCollidingShapes(NX_CLOTH_ATTACHMENT_TWOWAY); 
	if (flag == 1)Cloth->attachToCollidingShapes(NX_CLOTH_ATTACHMENT_TEARABLE);
	if (flag == 2)Cloth->attachToCollidingShapes(NX_CLOTH_ATTACHMENT_TWOWAY | NX_CLOTH_ATTACHMENT_TEARABLE);
}
BBDECL void BBCALL pxClothAttachToCore(NxCloth* Cloth, NxActor* body, float Threshold, float Depth)
{
	Cloth->attachToCore(body,Threshold, Depth); 
}
BBDECL void BBCALL pxClothAttachToShape(NxCloth* Cloth, NxActor* body, int flag)
{
	if (flag==0)Cloth->attachToShape(body->getShapes()[0], NX_CLOTH_ATTACHMENT_TWOWAY ); 
	if (flag==1)Cloth->attachToShape(body->getShapes()[0], NX_CLOTH_ATTACHMENT_TEARABLE );
	if (flag==2)Cloth->attachToShape(body->getShapes()[0],NX_CLOTH_ATTACHMENT_TWOWAY | NX_CLOTH_ATTACHMENT_TEARABLE );
}
BBDECL void BBCALL pxClothDetachFromShape(NxCloth* Cloth, NxActor* body)
{
	Cloth->detachFromShape(body->getShapes()[0]); 
}
BBDECL void BBCALL pxClothFreeVertex (NxCloth* Cloth, int vID)
{
	Cloth->freeVertex(vID); 
}
BBDECL void BBCALL pxClothAddForceAtPos(NxCloth* Cloth, float pos_x, float pos_y, float pos_z, float magnitude, float radius)
{
	Cloth->addForceAtPos(NxVec3(pos_x, pos_y, pos_z), magnitude, radius, NX_FORCE); 
}
BBDECL void BBCALL pxClothAddForceAtVertex(NxCloth* Cloth, float nx, float ny, float nz, int vID)
{
	Cloth->addForceAtVertex(NxVec3(nx, ny, nz), vID, NX_FORCE); 
}
BBDECL void BBCALL pxUpdateClothOk()
{
	float * entity;
	for (MyCloth **clothM = gCloths.begin(); clothM != gCloths.end(); clothM++)
	{
		entity = (*clothM)->entityData;
		unsigned long *surf = (*clothM)->surfData;
		LPBBVERTEX OldVertex = (*clothM)->OldVertex;
		NxCloth* cloth = (*clothM)->getNxCloth();

		(*clothM)->updateAllCloth(cloth, entity, surf, OldVertex);
	}
	//entity[16] = vert_buffer[0].x;
	//entity[17] = vert_buffer[0].y;
	//entity[18] = vert_buffer[0].z;
	//int moved = 3;
	//entity[11] = *((float*)(&moved));
	//return num;
}
BBDECL void BBCALL pxUpdateCloth()
{
	float * entity;
	int numCloth = gScene->getNbCloths();
	NxCloth** allCloth = gScene->getCloths();
	for (int i=0; i< numCloth; i++)
	{
		ClothData* data = (ClothData*)allCloth[i]->userData;
		MyCloth* mCloth = data->mCloth;
		entity = mCloth->entityData;
		unsigned long *surf = mCloth->surfData;
		LPBBVERTEX OldVertex = mCloth->OldVertex;
		NxCloth* cloth = mCloth->getNxCloth();
		
		mCloth->cDeltaTime = int(gDeltaTime);
		mCloth->updateAllCloth(cloth, entity, surf, OldVertex);
	}
	//entity[16] = vert_buffer[0].x;
	//entity[17] = vert_buffer[0].y;
	//entity[18] = vert_buffer[0].z;
	//int moved = 3;
	//entity[11] = *((float*)(&moved));
	//return num;
}
BBDECL void BBCALL pxClothGetTearVertex(NxCloth* Cloth, float *vbank)
{
	if (Cloth)
	{
		ClothData* data = (ClothData*)Cloth->userData;
		MyCloth* mCloth = data->mCloth;
		mCloth->updateTVertex(vbank);
	}
}
BBDECL void BBCALL pxClothGetTearIndices(NxCloth* Cloth, int *ibank)
{
	if (Cloth)
	{
		ClothData* data = (ClothData*)Cloth->userData;
		MyCloth* mCloth = data->mCloth;
		mCloth->updateTIndices(ibank);
	}
}
BBDECL void BBCALL pxClothGetTearPIndices(NxCloth* Cloth, int *ibank)
{
	if (Cloth)
	{
		ClothData* data = (ClothData*)Cloth->userData;
		MyCloth* mCloth = data->mCloth;
		mCloth->updateTPIndices(ibank);
	}
}
BBDECL void BBCALL pxClothGetTearNormals(NxCloth* Cloth, float *nbank)
{
	if (Cloth)
	{
		ClothData* data = (ClothData*)Cloth->userData;
		MyCloth* mCloth = data->mCloth;
		mCloth->updateNormals(nbank);
	}
}
BBDECL int BBCALL pxClothGetNumTearVertex(NxCloth* Cloth)
{
	if (Cloth)
	{
		ClothData* data = (ClothData*)Cloth->userData;
		MyCloth* mCloth = data->mCloth;
		return mCloth->getNumVertices();
	}
	return 0;
}
BBDECL int BBCALL pxClothGetNumTearTris(NxCloth* Cloth)
{
	if (Cloth)
	{
		ClothData* data = (ClothData*)Cloth->userData;
		MyCloth* mCloth = data->mCloth;
		return mCloth->getNumIndices();
	}
	return 0;
}
BBDECL void BBCALL pxClothDelete(NxCloth* Cloth)
{
	if (Cloth)
	{
		ClothData* data = (ClothData*)Cloth->userData;
		MyCloth* mCloth = data->mCloth;
		unsigned long *surf = mCloth->surfData;

		if (surf)
		{
			*(surf+7) = (unsigned long)mCloth->adresOldBuferVertexStart;
			*(surf+8) = (unsigned long)mCloth->adresOldBuferVertexEnd;
			*(surf+11) = (unsigned long)mCloth->adresOldBuferTriangleStart;
			*(surf+12) = (unsigned long)mCloth->adresOldBuferTriangleEnd;
		}

		mCloth->~MyCloth();
		Cloth = 0;
	}
}
//--------------------Create Car---------------------
BBDECL NxVehicleDesc* BBCALL pxCarCreateDescriptor(float pos_x, float pos_y, float pos_z, float mass)
{
	NxVehicleDesc *vehicleDesc = new NxVehicleDesc;
	
	vehicleDesc->position				= NxVec3(pos_x, pos_y, pos_z);
	vehicleDesc->mass					= mass;//monsterTruck ? 12000 : 
	vehicleDesc->digitalSteeringDelta	= 0.04f;
	vehicleDesc->steeringMaxAngle		= 30.f;
	vehicleDesc->motorForce				= 3500.f;//monsterTruck?180.f:
	vehicleDesc->maxVelocity			= 300.f;//(monsterTruck)?20.f:
	
	vehicleDesc->steeringSteerPoint.set(1.8f, 0, 0);
	vehicleDesc->steeringTurnPoint.set(-1.5f, 0, 0);

	return vehicleDesc;
}
BBDECL void BBCALL pxCarDescSteerPoint(NxVehicleDesc* VehDesc, float steerPoint_x, float steerPoint_y, float steerPoint_z, float turnPoint_x, float turnPoint_y, float turnPoint_z)
{
	VehDesc->steeringSteerPoint = NxVec3(steerPoint_x, steerPoint_y, steerPoint_z);
	VehDesc->steeringTurnPoint = NxVec3(turnPoint_x, turnPoint_y, turnPoint_z);
}
BBDECL void BBCALL pxCarDescSetMotorParameter(NxVehicleDesc* VehDesc, float motor_force, float max_speed)
{
	VehDesc->motorForce = motor_force;
	VehDesc->maxVelocity = max_speed;
}
BBDECL void BBCALL pxCarDescSetSteeringParameter(NxVehicleDesc* VehDesc, float steering_delta, float steering_maxAngle)
{
	VehDesc->digitalSteeringDelta = steering_delta;
	VehDesc->steeringMaxAngle = steering_maxAngle;
}
BBDECL void BBCALL pxCarDescAddBoxShape(NxVehicleDesc* VehDesc, float x, float y, float z, float pos_x, float pos_y, float pos_z)
{
	NxBoxShapeDesc *boxShap = new NxBoxShapeDesc;

	boxShap->groupsMask.bits0 = 1;
	boxShap->dimensions.set(x, y, z);
	boxShap->localPose.t.set(pos_x, pos_y, pos_z);
	VehDesc->carShapes.pushBack(boxShap);
}
BBDECL void BBCALL pxCarDescAddSphereShape(NxVehicleDesc* VehDesc, float radius, float pos_x, float pos_y, float pos_z)
{
	NxSphereShapeDesc *SphereDesc = new NxSphereShapeDesc;
	SphereDesc->groupsMask.bits0 = 1;
	SphereDesc->radius		= radius;
	SphereDesc->localPose.t.set(pos_x, pos_y, pos_z);
	VehDesc->carShapes.pushBack(SphereDesc);
}
BBDECL void BBCALL pxCarDescAddHullShape(NxVehicleDesc* VehDesc, float *vbank, int nvert, float pos_x, float pos_y, float pos_z)
{
	
	int nbVerts=nvert;
	NxVec3* verts = new NxVec3[nbVerts];
		for(int i=0;i<nbVerts;i++)
		{
			verts[i].x = vbank[i*3+0];
			verts[i].y = vbank[i*3+1];
			verts[i].z = vbank[i*3+2];
		}
	
	NxU32 vertCount = nbVerts;
	
	NxConvexMeshDesc *convexDesc = new NxConvexMeshDesc;
	NxConvexShapeDesc *carShape = new NxConvexShapeDesc;
	carShape->groupsMask.bits0 = 1;
	convexDesc->numVertices			= nbVerts;
	convexDesc->pointStrideBytes		= sizeof(NxVec3);
	convexDesc->points				= verts;
	convexDesc->flags |= NX_CF_COMPUTE_CONVEX;//|NX_CF_USE_LEGACY_COOKER;
	
	NxInitCooking();

	MemoryWriteBuffer buf;
	bool status = NxCookConvexMesh(*convexDesc, buf);
	if(status)
	{
		carShape->meshData = gPhysicsSDK->createConvexMesh(MemoryReadBuffer(buf.data));
		carShape->localPose.t.set(pos_x, pos_y, pos_z);
		VehDesc->carShapes.pushBack(carShape);
	}

	//delete[] verts;
}
BBDECL NxWheelDesc* BBCALL pxCarDescAddDriveWheel(NxVehicleDesc* VehDesc, float pos_x, float pos_y, float pos_z, float radius, float width)
{
	NxWheelDesc *wheelDesc = new NxWheelDesc;

		wheelDesc->wheelApproximation = 10;
		wheelDesc->wheelRadius = radius;
		wheelDesc->wheelWidth = width;
		wheelDesc->wheelSuspension = 0.2f;
		wheelDesc->springRestitution = 7000;
		wheelDesc->springDamping = 800;
		wheelDesc->springBias = 0.0f;
		wheelDesc->maxBrakeForce = 1.f;
		wheelDesc->frictionToSide = 5000;
		wheelDesc->frictionToFront = 500000;
		wheelDesc->wheelFlags |= NX_WF_USE_WHEELSHAPE;
		
		VehDesc->carWheels.pushBack(wheelDesc);
	
		wheelDesc->position.set( pos_x, pos_y, pos_z);
		NxU32 flags = NX_WF_BUILD_LOWER_HALF;
		bool frontWheelDrive = true;
		wheelDesc->wheelFlags |= (frontWheelDrive?NX_WF_ACCELERATED:0) | NX_WF_STEERABLE_INPUT | flags;

	return wheelDesc;
}
BBDECL NxWheelDesc* BBCALL pxCarDescAddWheel(NxVehicleDesc* VehDesc, float pos_x, float pos_y, float pos_z, float radius, float width)
{
	NxWheelDesc *wheelDesc = new NxWheelDesc;
	
		wheelDesc->wheelApproximation = 10;
		wheelDesc->wheelRadius = radius;
		wheelDesc->wheelWidth = width;
		wheelDesc->wheelSuspension = 0.2f;
		wheelDesc->springRestitution = 7000;
		wheelDesc->springDamping = 800;
		wheelDesc->springBias = 0.0f;
		wheelDesc->maxBrakeForce = 1.0f;
		wheelDesc->frictionToSide = 5000;
		wheelDesc->frictionToFront = 5000000;
		wheelDesc->wheelFlags |= NX_WF_USE_WHEELSHAPE;
			
		VehDesc->carWheels.pushBack(wheelDesc);
	
		wheelDesc->position.set( pos_x, pos_y, pos_z);
		NxU32 flags = NX_WF_BUILD_LOWER_HALF;
		bool backWheelDrive = false;
		wheelDesc->wheelFlags |= (backWheelDrive?NX_WF_ACCELERATED:0) | NX_WF_AFFECTED_BY_HANDBRAKE | flags;

	return wheelDesc;
}
BBDECL void BBCALL pxCarDescSetWheelPosition(NxWheelDesc* WhDesc, float pos_x, float pos_y, float pos_z)
{
	WhDesc->position.set(pos_x, pos_y, pos_z);
}
BBDECL void BBCALL pxCarDescSetWheelForm(NxWheelDesc* WhDesc, float radius, float width)
{
	WhDesc->wheelRadius = radius;
	WhDesc->wheelWidth = width;
}
BBDECL void BBCALL pxCarDescSetWheelParameter(NxWheelDesc* WhDesc, float restitution, float damping, float suspension)
{

	WhDesc->wheelSuspension = suspension;
	WhDesc->springRestitution = restitution;
	WhDesc->springDamping = damping;

}
BBDECL void BBCALL pxCarDescSetWheelParameter2(NxWheelDesc* WhDesc, float approximation, float bias, float brakeForce)
{
	WhDesc->wheelApproximation = NxU32(approximation);
	WhDesc->springBias = bias;
	WhDesc->maxBrakeForce = brakeForce;
	
}
BBDECL void BBCALL pxCarDescSetWheelEntity(NxWheelDesc* WhDesc, float* entity)
{
	WhDesc->entity = entity;
}
BBDECL void BBCALL pxCarSetFrictionToSide(NxVehicle* vehicle, int num, float friction)
{
	NxTireFunctionDesc latTFD;
	latTFD.extremumSlip = 1.0f;
	latTFD.extremumValue = 0.02f;
	latTFD.asymptoteSlip = 2.0f;
	latTFD.asymptoteValue = 0.01f;	
	latTFD.stiffnessFactor = friction;

	NxWheel* wheel = vehicle->getWheel(num);
	NxWheelShape* wheelShape = wheel->mainWheelShape;
	wheelShape->setLateralTireForceFunction(latTFD);
}
BBDECL void BBCALL pxCarSetFrictionToFront(NxVehicle* vehicle, int num, float friction)
{
	NxTireFunctionDesc latTFD;
	latTFD.extremumSlip = 1.0f;
	latTFD.extremumValue = 0.02f;
	latTFD.asymptoteSlip = 2.0f;
	latTFD.asymptoteValue = 0.01f;	
	latTFD.stiffnessFactor = friction;

	NxWheel* wheel = vehicle->getWheel(num);
	NxWheelShape* wheelShape = wheel->mainWheelShape;
	wheelShape->setLongitudalTireForceFunction(latTFD);
}
BBDECL NxVehicle* BBCALL pxCreateCar(NxVehicleDesc* VehDesc)
{
	static NxVehicle* vehicle = NxVehicle::createVehicle(gScene, VehDesc);
	
	NxQuat q;
	q.fromAngleAxis(180.0f, NxVec3(0.0f, 1.0f, 0.0f));
	NxActor* body = vehicle->getActor();
	BodyAddUserData(body);
	body->setGlobalOrientationQuat(q);
	mainSetMask(body);
		
	return vehicle;
}
BBDECL void BBCALL pxCarSetMotorForce(NxVehicle* vehicle, float force)
{
	vehicle->_motorForce = force;
}
BBDECL void BBCALL pxCarSetSteerPoint(NxVehicle* vehicle, float steerPoint_x, float steerPoint_y, float steerPoint_z)
{
	vehicle->_steeringSteerPoint = NxVec3(steerPoint_x, steerPoint_y, steerPoint_z);
}
BBDECL void BBCALL pxCarSetTurnPoint(NxVehicle* vehicle, float turnPoint_x, float turnPoint_y, float turnPoint_z)
{
	vehicle->_steeringTurnPoint = NxVec3(turnPoint_x, turnPoint_y, turnPoint_z);
}
BBDECL void BBCALL pxCarSetSteeringParameter(NxVehicle* vehicle, float steering_delta, float steering_maxAngle)
{
	vehicle->_digitalSteeringDelta = steering_delta;
	vehicle->_steeringMaxAngleRad = NxMath::degToRad(steering_maxAngle);
}
void WheelUpdateEntity(float *entity, NxWheelShape* wheel) 
{
	NxWheelContactData contactData;
	NxShape* whShape = wheel->getContact(contactData);
	//wheel->userData = 0;

			NxReal rollAngle;// =  
				memcpy(&rollAngle,&(wheel->userData),4);
			rollAngle += wheel->getAxleSpeed() * 1.0f/60.0f;
			while (rollAngle > NxTwoPi)	//normally just 1x
				rollAngle -= NxTwoPi;
			while (rollAngle < -NxTwoPi)
				rollAngle += NxTwoPi;

			//wheel->userData = *((void**)(&rollAngle));
			memcpy(&(wheel->userData),&rollAngle,4);

			NxMat34 pose;
			pose = wheel->getGlobalPose();

			NxReal r = wheel->getRadius();
			NxReal st = wheel->getSuspensionTravel();
			NxReal steerAngle = wheel->getSteerAngle();

			NxVec3 p0;
			NxVec3 dir;
			p0 = pose.t;  //cast from shape origin
			pose.M.getColumn(1, dir);
			dir = -dir;	//cast along -Y.
			NxReal castLength = r + st;	//cast ray this long

			NxMat33 rot, axisRot;
			rot.rotY(steerAngle);
			axisRot.rotY(0);

			NxMat33 rollRot;
			rollRot.rotX(rollAngle);

			pose.M = rot * pose.M * axisRot * rollRot;

			if (whShape)
			{
				pose.t = contactData.contactPoint;
				pose.t -= dir * r;	//go from contact pos to center pos.
			}
			else
			{
				pose.t = p0 + dir * st;
			}

	NxQuat q;
	pose.M.toQuat(q);

		entity[13] = q.x;
		entity[14] = q.y;
		entity[15] = q.z;
		entity[12] = -q.w;
		entity[16] = pose.t.x;
		entity[17] = pose.t.y;
		entity[18] = pose.t.z;
		int moved = 3;
		entity[11] = *((float*)(&moved));

}
BBDECL void BBCALL pxCarControl(NxVehicle* vehicle, float speed, float angle, bool brake)
{
		float steering = 0;
		if (angle) steering = angle;
		float acceleration = 0;
		if (speed) acceleration = speed;
		bool handbrake = 0;
		if (brake) handbrake = brake;
		
		if(NxAllVehicles::getActiveVehicle())
			vehicle->control(steering, false, acceleration, false, handbrake);

		int numWheel  = vehicle->getNbWheels();
		for (int i=0; i<numWheel; i++)
		{
			NxWheel* wheel = vehicle->getWheel(i);
			float* entity = wheel->entity;
			if (entity)
			{
				NxWheelShape* wheelShape = wheel->mainWheelShape;
				WheelUpdateEntity(entity, wheelShape);
			}
		}

			//NxAllVehicles::updateAllVehicles(1.0f/60.0f);
			vehicle->updateVehicle(0.02f);

}
//--------------------------------------------
BBDECL float BBCALL pxCarGetMotorForce(NxVehicle* vehicle)
{
	return vehicle->_motorForce;
}
BBDECL float BBCALL pxCarGetSteeringMaxAndle(NxVehicle* vehicle)
{
	return NxMath::radToDeg(vehicle->_steeringMaxAngleRad);
}
BBDECL float BBCALL pxCarGetSteeringDelta(NxVehicle* vehicle)
{
	return vehicle->_digitalSteeringDelta;
}
BBDECL float BBCALL pxCarGetLocalSpeedX(NxVehicle* vehicle)
{
	return vehicle->_localVelocity.x;
}
BBDECL float BBCALL pxCarGetLocalSpeedY(NxVehicle* vehicle)
{
	return vehicle->_localVelocity.y;
}
BBDECL float BBCALL pxCarGetLocalSpeedZ(NxVehicle* vehicle)
{
	return vehicle->_localVelocity.z;
}
BBDECL float BBCALL pxCarGetMaxSpeed(NxVehicle* vehicle)
{
	return vehicle->_maxVelocity;
}
BBDECL NxActor* BBCALL pxCarGetBody(NxVehicle* vehicle)
{
	NxActor* body = vehicle->getActor();
	return body;
}
BBDECL float* BBCALL pxCarGetWheelEntity(NxVehicle* vehicle, int num)
{
	NxWheel* wheel = vehicle->getWheel(num);
	return wheel->entity;
}
BBDECL float BBCALL pxCarGetWheelContactPointX(NxVehicle* vehicle, int num)
{
	NxWheel* wheel = vehicle->getWheel(num);
	NxWheelShape* wheelShape = wheel->mainWheelShape;
	NxWheelContactData contactData;
	wheelShape->getContact(contactData);

	return contactData.contactPoint.x;
}
BBDECL float BBCALL pxCarGetWheelContactPointY(NxVehicle* vehicle, int num)
{
	NxWheel* wheel = vehicle->getWheel(num);
	NxWheelShape* wheelShape = wheel->mainWheelShape;
	NxWheelContactData contactData;
	wheelShape->getContact(contactData);

	return contactData.contactPoint.y;
}
BBDECL float BBCALL pxCarGetWheelContactPointZ(NxVehicle* vehicle, int num)
{
	NxWheel* wheel = vehicle->getWheel(num);
	NxWheelShape* wheelShape = wheel->mainWheelShape;
	NxWheelContactData contactData;
	wheelShape->getContact(contactData);

	return contactData.contactPoint.z;
}
//----------------Wheel---------------------------------------
typedef struct pxWheelData
{
	float rollAngle;
	NxVec3 contactPos;
	NxVec3 oldPos;
	NxActor* actor;
	float travel, restitution, damping;
	int shape;
	NxMat34 posMat;
}WheelData;
NxWheelShape* AddWheelToActor(NxActor* actor, NxWheelDesc* wheelDesc)
{
	NxWheelShapeDesc wheelShapeDesc;

	NxMaterial* wsm;
//	if (!wsm)
//	{
		NxMaterialDesc m;
		m.flags |= NX_MF_DISABLE_FRICTION;
		m.restitution		= 0.09f;
		m.staticFriction	= 0.91f;
		m.dynamicFriction	= 0.91f;
		wsm = gScene->createMaterial(m);
//	}
	wheelShapeDesc.groupsMask.bits0 = 1;
	wheelShapeDesc.groupsMask.bits1 = 1;
	wheelShapeDesc.groupsMask.bits2 = 1;
	wheelShapeDesc.groupsMask.bits3 = 1;

	wheelShapeDesc.localPose.t = wheelDesc->position;
	NxQuat q;
	q.fromAngleAxis(0, NxVec3(0,1,0));
	wheelShapeDesc.localPose.M.fromQuat(q);

	NxReal heightModifier = (wheelDesc->wheelSuspension + wheelDesc->wheelRadius) / wheelDesc->wheelSuspension;

	wheelShapeDesc.suspension.spring = wheelDesc->springRestitution*heightModifier;
	wheelShapeDesc.suspension.damper = wheelDesc->springDamping*heightModifier;
	wheelShapeDesc.suspension.targetValue = wheelDesc->springBias*heightModifier;

	wheelShapeDesc.radius = wheelDesc->wheelRadius;
	wheelShapeDesc.suspensionTravel = wheelDesc->wheelSuspension; 
	wheelShapeDesc.inverseWheelMass = 0.1f;	//not given!? TODO

	NxTireFunctionDesc latTFD;
	latTFD.extremumSlip = 1.0f;
	latTFD.extremumValue = 0.02f;
	latTFD.asymptoteSlip = 2.0f;
	latTFD.asymptoteValue = 0.01f;	
	latTFD.stiffnessFactor = 1000000.0f;

	wheelShapeDesc.lateralTireForceFunction = latTFD;	//TODO
	// wheelShapeDesc.longitudalTireForceFunction = ;	//TODO

    NxWheelShape* wheelShape = NULL;
	wheelShape = static_cast<NxWheelShape *>(actor->createShape(wheelShapeDesc));

	WheelData* data = new WheelData;
	data->rollAngle = 0;
	data->contactPos.set(0,0,0);
	data->actor = actor;
	data->shape = Wheel_Shape;
	wheelShape->userData = data;
	gWheels.push_back(wheelShape);

	return wheelShape;
}
BBDECL NxWheelShape* BBCALL pxWheelAddToBody(NxActor* body, float pos_x, float pos_y, float pos_z)
{
	NxWheelDesc wheelDesc;
    wheelDesc.wheelApproximation = 10;
	wheelDesc.wheelRadius = 1;
	wheelDesc.wheelWidth = 0.1f;
	wheelDesc.wheelSuspension = 1;
	wheelDesc.springRestitution = 10;
	wheelDesc.springDamping = 0.5f;
	wheelDesc.springBias = 0;  
	wheelDesc.maxBrakeForce = 1;
	wheelDesc.frictionToFront = 0.91f;
	wheelDesc.frictionToSide = 0.99f;
	wheelDesc.position = NxVec3(pos_x,pos_y,pos_z);
	wheelDesc.wheelFlags = NX_WF_USE_WHEELSHAPE | NX_WF_BUILD_LOWER_HALF | NX_WF_ACCELERATED | 
		                   NX_WF_AFFECTED_BY_HANDBRAKE | NX_WF_STEERABLE_INPUT;

	NxWheelShape* wheel;
    wheel = AddWheelToActor(body, &wheelDesc);

	return wheel;
}
BBDECL void BBCALL pxWheelSetRadius(NxWheelShape* wheel, float radius)
{
	wheel->setRadius(radius);
}
BBDECL void BBCALL pxWheelSetRotation(NxWheelShape* wheel, float pitch, float yaw, float roll)
{
	NxQuat q = NxQuat( NxVec3(0,sinf(yaw*3.14159f/360),0 ), -cosf(yaw*3.14159f/360) ) * NxQuat(NxVec3( sinf(pitch*3.14159f/-360),0,0 ), -cosf(pitch*3.14159f/-360) ) * NxQuat( NxVec3( 0,0,sinf(roll*3.14159f/-360)), -cosf(roll*3.14159f/-360) );
	NxMat33 mat;
	mat.fromQuat(q);
	wheel->setGlobalOrientation(mat);
}
BBDECL void BBCALL pxWheelSetMotorTorque(NxWheelShape* wheel, float torque)
{
	NxActor &actor = wheel->getActor();
	actor.wakeUp(1);
	wheel->setMotorTorque( torque);
}
BBDECL void BBCALL pxWheelSetSteerAngle(NxWheelShape* wheel, float angle)
{
	wheel->setSteerAngle(NxMath::degToRad(angle));
}
BBDECL void BBCALL pxWheelSetBrakeTorque(NxWheelShape* wheel, float torque)
{
	wheel->setBrakeTorque(torque);
}
BBDECL void BBCALL pxWheelSetSuspension(NxWheelShape* wheel, float susp, float rest, float damping)
{
	NxReal heightModifier = (susp + wheel->getRadius())/susp;

	NxSpringDesc spring;
	spring.spring = rest*heightModifier;
	spring.damper = damping*heightModifier;
	spring.targetValue = 0; 

	wheel->setSuspension(spring);
	wheel->setSuspensionTravel(susp);

	WheelData* data = (WheelData*)wheel->userData;
	data->travel = susp;
	data->restitution = rest;
	data->damping = damping;
}
BBDECL void BBCALL pxWheelSetFrictionToSide(NxWheelShape* wheel, float friction)
{
	NxTireFunctionDesc latTFD;
	latTFD.extremumSlip = 1.0f;
	latTFD.extremumValue = 0.02f;
	latTFD.asymptoteSlip = 2.0f;
	latTFD.asymptoteValue = 0.01f;	
	latTFD.stiffnessFactor = friction;

	wheel->setLateralTireForceFunction(latTFD);
}
BBDECL void BBCALL pxWheelSetFrictionToFront(NxWheelShape* wheel, float friction)
{
	NxTireFunctionDesc latTFD;
	latTFD.extremumSlip = 1.0f;
	latTFD.extremumValue = 0.02f;
	latTFD.asymptoteSlip = 2.0f;
	latTFD.asymptoteValue = 0.01f;	
	latTFD.stiffnessFactor = friction;

	wheel->setLongitudalTireForceFunction(latTFD);
}
BBDECL void BBCALL pxWheelSetCollisionGroup(NxWheelShape* wheel, int group)
{
	wheel->setGroup(group);
}
BBDECL float BBCALL pxWheelGetContactX(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.contactPoint.x;
}
BBDECL float BBCALL pxWheelGetContactY(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.contactPoint.y;
}
BBDECL float BBCALL pxWheelGetContactZ(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.contactPoint.z;
}
BBDECL float BBCALL pxWheelGetContactNX(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.contactNormal.x;
}
BBDECL float BBCALL pxWheelGetContactNY(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.contactNormal.y;
}
BBDECL float BBCALL pxWheelGetContactNZ(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.contactNormal.z;
}
BBDECL float BBCALL pxWheelGetContactForce(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.contactForce;
}
BBDECL float BBCALL pxWheelGetContactLatDirectionX(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.lateralDirection.x;
}
BBDECL float BBCALL pxWheelGetContactLatDirectionY(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.lateralDirection.y;
}
BBDECL float BBCALL pxWheelGetContactLatDirectionZ(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.lateralDirection.z;
}
BBDECL float BBCALL pxWheelGetContactLatImpulse(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.lateralImpulse;
}
BBDECL float BBCALL pxWheelGetContactLonDirectionX(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.longitudalDirection.x;
}
BBDECL float BBCALL pxWheelGetContactLonDirectionY(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.longitudalDirection.y;
}
BBDECL float BBCALL pxWheelGetContactLonDirectionZ(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.longitudalDirection.z;
}
BBDECL float BBCALL pxWheelGetContactLonImpulse(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.longitudalImpulse;
}
BBDECL int BBCALL pxWheelGetContactMaterial(NxWheelShape* wheel)
{
	NxWheelContactData contactData;
	wheel->getContact(contactData);
	return contactData.otherShapeMaterialIndex;
}
BBDECL float BBCALL pxWheelGetSteerAngle(NxWheelShape* wheel)
{
	return NxMath::radToDeg (wheel->getSteerAngle());
}
BBDECL float BBCALL pxWheelGetAxleSpeed(NxWheelShape* wheel)
{
	return wheel->getAxleSpeed();
}
BBDECL float BBCALL pxWheelGetRadius(NxWheelShape* wheel)
{
	return wheel->getRadius();
}
BBDECL float BBCALL pxWheelGetSuspensionTravel(NxWheelShape* wheel)
{
	WheelData* data = (WheelData*)wheel->userData;
	return data->travel;
}
BBDECL float BBCALL pxWheelGetSuspensionRestitution(NxWheelShape* wheel)
{
	WheelData* data = (WheelData*)wheel->userData;
	return data->restitution;
}
BBDECL float BBCALL pxWheelGetSuspensionDamping(NxWheelShape* wheel)
{
	WheelData* data = (WheelData*)wheel->userData;
	return 	data->damping;
}
BBDECL float BBCALL pxWheelGetPositionX(NxWheelShape* wheel)
{
	return wheel->getGlobalPosition().x;
}
BBDECL float BBCALL pxWheelGetPositionY(NxWheelShape* wheel)
{
	return wheel->getGlobalPosition().y;
}
BBDECL float BBCALL pxWheelGetPositionZ(NxWheelShape* wheel)
{
	return wheel->getGlobalPosition().z;
}
BBDECL float BBCALL pxWheelGetFrictionToSide(NxWheelShape* wheel)
{
	NxTireFunctionDesc TFdesc = wheel->getLateralTireForceFunction();
	return TFdesc.stiffnessFactor;
}
BBDECL float BBCALL pxWheelGetFrictionToFront(NxWheelShape* wheel)
{
	NxTireFunctionDesc TFdesc = wheel->getLongitudalTireForceFunction();
	return TFdesc.stiffnessFactor;
}
//------------------------------------
BBDECL void BBCALL pxWheelSetMask(NxWheelShape* wheel, int mask)
{
	NxGroupsMask GrMask;//bits0, bits1, bits2, bits3;
	if (mask==0) 
	{
		GrMask.bits0 = 1;
		GrMask.bits1 = 0;
		GrMask.bits2 = 0;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==1) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 1;
		GrMask.bits2 = 0;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==2) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 0;
		GrMask.bits2 = 1;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==3) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 0;
		GrMask.bits2 = 0;
		GrMask.bits3 = 1;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}

	wheel->setGroupsMask(GrMask);

}

BBDECL void BBCALL pxWheelSetMaskCombine(NxWheelShape* wheel, int mask)
{
	NxGroupsMask GrMask;
	GrMask.bits0 = mask & 1;
	GrMask.bits1 = mask & 2;
	GrMask.bits2 = mask & 4;
	GrMask.bits3 = mask & 8;

	wheel->setGroupsMask(GrMask);
}

BBDECL void BBCALL pxWheelClearMask(NxWheelShape* wheel)
{
	NxGroupsMask GrMask;//bits0, bits1, bits2, bits3;

		GrMask.bits0 = 0xffffffff;
		GrMask.bits1 = 0xffffffff;
		GrMask.bits2 = 0xffffffff;
		GrMask.bits3 = 0xffffffff;

	wheel->setGroupsMask(GrMask);
}
BBDECL void BBCALL pxWheelDelete(NxWheelShape* wheel)
{
	WheelData* data = (WheelData*)wheel->userData;
	NxShape* shape = (NxShape*)wheel;
	data->actor->releaseShape(*shape);
}
BBDECL float BBCALL pxWheelSetEntity(float *entity, NxWheelShape* wheel, int step) 
{
	NxWheelContactData contactData;
	NxShape* whShape = wheel->getContact(contactData);
	WheelData* data = (WheelData*)wheel->userData;

			NxReal rollAngle = data->rollAngle;  
			rollAngle += wheel->getAxleSpeed()/gDeltaTime;
			while (rollAngle > NxTwoPi)	//normally just 1x
				rollAngle -= NxTwoPi;
			while (rollAngle < -NxTwoPi)
				rollAngle += NxTwoPi;

			data->rollAngle = rollAngle;
			//memcpy(&(wheel->userData),&rollAngle,4);

			NxMat34 pose;
			pose = wheel->getGlobalPose();

			NxReal r = wheel->getRadius();
			NxReal st = wheel->getSuspensionTravel();
			NxReal steerAngle = wheel->getSteerAngle();
			NxVec3 speed = data->actor->getLinearVelocity();

			NxVec3 p0;
			NxVec3 dir;
			p0 = pose.t;
			pose.M.getColumn(1, dir);
			dir = -dir;	//cast along -Y.

			NxMat33 rot, axisRot;
			rot.rotY(steerAngle);
			axisRot.rotY(0);

			NxMat33 rollRot;
			rollRot.rotX(rollAngle);

			pose.M = pose.M * axisRot*rot * rollRot;

			if (whShape)
			{
				if (step==1) pose.t = contactData.contactPoint + speed/60;
				else pose.t = contactData.contactPoint;
				pose.t -= dir * r;	//go from contact pos to center pos.
			}
			else
			{
				if (!data->actor->isSleeping()) pose.t = p0 + dir * st;
			}

	if (entity)
	{
		NxQuat q;
		pose.M.toQuat(q);

		entity[13] = q.x;
		entity[14] = q.y;
		entity[15] = q.z;
		entity[12] = -q.w;
		entity[16] = pose.t.x;
		entity[17] = pose.t.y;
		entity[18] = pose.t.z;
		int moved = 3;
		entity[11] = *((float*)(&moved));
	}else
	{
		data->posMat = pose;
	}
		return gDeltaTime;

}
BBDECL float BBCALL pxWheelGetPositionXSpec(NxWheelShape* wheel) 
{
	NxVec3 speed;
	NxWheelContactData contactData;
	NxShape* whShape = wheel->getContact(contactData);
	WheelData* data = (WheelData*)wheel->userData;
	
	NxMat34 pose = data->posMat;
	return pose.t.x;
}
BBDECL float BBCALL pxWheelGetPositionYSpec(NxWheelShape* wheel) 
{
	NxVec3 speed;
	NxWheelContactData contactData;
	NxShape* whShape = wheel->getContact(contactData);
	WheelData* data = (WheelData*)wheel->userData;

	NxMat34 pose = data->posMat;
	return pose.t.y;
}
BBDECL float BBCALL pxWheelGetPositionZSpec(NxWheelShape* wheel) 
{
	NxVec3 speed;
	NxWheelContactData contactData;
	NxShape* whShape = wheel->getContact(contactData);
	WheelData* data = (WheelData*)wheel->userData;

	NxMat34 pose = data->posMat;
	return pose.t.z;
}
BBDECL float BBCALL pxGetRotationPitch(NxQuat rquat)
{
	float xz = rquat.x * rquat.z; 
	float wy = -rquat.w * rquat.y;
	float yz = rquat.y * rquat.z; 
	float wx = -rquat.w * rquat.x; 
	float xx = rquat.x * rquat.x; 
	float yy = rquat.y * rquat.y; 
 
	float kx = 2*(xz - wy); 
	float ky = 2*(yz + wx); 
	float kz = 1 - 2 * (xx +yy); 
 
	float pitch = -NxMath::radToDeg(atan2f ( ky , sqrtf(kx*kx+kz*kz)));
	
	return pitch;

}
BBDECL float BBCALL pxGetRotationYaw(NxQuat rquat)
{
	NxQuat pq;

	float angle = pxGetRotationPitch( rquat);
	if (angle < 90.01 && angle >89.99)
	{
		NxQuat pq;
		pq.setXYZW(0.0001f,0,0,-1);
		rquat = rquat*pq;
		//return 333; 
	}else if (angle > -90.01 && angle < -89.99)
	{
		NxQuat pq;
		pq.setXYZW(0.0001f,0,0,-1);
		rquat = rquat*pq;
	}
	float xz = rquat.x * rquat.z; 
	float wy = -rquat.w * rquat.y;
	float yz = rquat.y * rquat.z; 
	float wx = -rquat.w * rquat.x; 
	float xx = rquat.x * rquat.x; 
	float yy = rquat.y * rquat.y; 
 
	float kx = 2*(xz - wy); 
	float ky = 2*(yz + wx); 
	float kz = 1 - 2 * (xx +yy); 
 
	float yaw = -NxMath::radToDeg(atan2f(kx,kz)); 
 	return yaw; 

}
BBDECL float BBCALL pxGetRotationRoll(NxQuat rquat)
{
	float angle = pxGetRotationPitch( rquat);
	if (angle < 90.01 && angle >89.99)
	{
		NxQuat pq;
		pq.setXYZW(0.0001f,0,0,-1);
		rquat = rquat*pq;
		//return 333; 
	}else if (angle > -90.01 && angle < -89.99)
	{
		NxQuat pq;
		pq.setXYZW(0.0001f,0,0,-1);
		rquat = rquat*pq;
	}
	float xx = rquat.x * rquat.x; 
	float zz = rquat.z * rquat.z; 
	float xy = rquat.x * rquat.y; 
	float wz = -rquat.w * rquat.z; 
	float roll = NxMath::radToDeg(atan2f(2*(xy-wz),1-2*(xx+zz)));//*180/3.14159;

 	return roll;
}
BBDECL void BBCALL pxWheelUpdateSpec(NxWheelShape* wheel, int mode) 
{
	NxVec3 speed;
	NxWheelContactData contactData;
	NxShape* whShape = wheel->getContact(contactData);
	WheelData* data = (WheelData*)wheel->userData;

			NxReal rollAngle = data->rollAngle;  
			rollAngle += wheel->getAxleSpeed()/gDeltaTime;
			while (rollAngle > NxTwoPi)	//normally just 1x
				rollAngle -= NxTwoPi;
			while (rollAngle < -NxTwoPi)
				rollAngle += NxTwoPi;

			data->rollAngle = rollAngle;
			//memcpy(&(wheel->userData),&rollAngle,4);

			NxMat34 pose;
			pose = wheel->getGlobalPose();

			NxReal r = wheel->getRadius();
			NxReal st = wheel->getSuspensionTravel();
			NxReal steerAngle = wheel->getSteerAngle();
			if (mode) speed = data->actor->getLinearVelocity();

			NxVec3 p0;
			NxVec3 dir;
			p0 = pose.t;
			pose.M.getColumn(1, dir);
			dir = -dir;	//cast along -Y.

			NxMat33 rot, axisRot;
			rot.rotY(steerAngle);
			axisRot.rotY(0);

			NxMat33 rollRot;
			rollRot.rotX(rollAngle);

			pose.M = pose.M * axisRot*rot * rollRot;

			if (whShape)
			{
				if (mode) pose.t = contactData.contactPoint + speed/60;
				else pose.t = contactData.contactPoint;
				pose.t -= dir * r;	//go from contact pos to center pos.
			}
			else
			{
				if (!data->actor->isSleeping()) pose.t = p0 + dir * st;
			}

		data->posMat = pose;
}
BBDECL float BBCALL pxWheelGetRotationPitchSpec(NxWheelShape* wheel) 
{
	NxWheelContactData contactData;
	NxShape* whShape = wheel->getContact(contactData);
	WheelData* data = (WheelData*)wheel->userData;

	NxMat34 pose = data->posMat;
	NxQuat rq;
	pose.M.toQuat(rq);

	return pxGetRotationPitch(rq);
}
BBDECL float BBCALL pxWheelGetRotationYawSpec(NxWheelShape* wheel) 
{
	NxWheelContactData contactData;
	NxShape* whShape = wheel->getContact(contactData);
	WheelData* data = (WheelData*)wheel->userData;

	NxMat34 pose = data->posMat;
	NxQuat rq;
	pose.M.toQuat(rq);

	return pxGetRotationYaw(rq);
}
BBDECL float BBCALL pxWheelGetRotationRollSpec(NxWheelShape* wheel) 
{
	NxWheelContactData contactData;
	NxShape* whShape = wheel->getContact(contactData);
	WheelData* data = (WheelData*)wheel->userData;

	NxMat34 pose = data->posMat;
	NxQuat rq;
	pose.M.toQuat(rq);

	return pxGetRotationRoll(rq);
}
//----------Joint Car---------------------
BBDECL NxD6Joint* BBCALL pxJointCreateSuspFront(NxActor* a0, NxActor* a1, float x, float y, float z)
{
	NxD6JointDesc d6Desc;
	d6Desc.actor[0] = a0;
	d6Desc.actor[1] = a1;
	d6Desc.setGlobalAnchor(NxVec3(x,y,z));
//	d6Desc.setGlobalAxis(NxVec3(0,0,-1));

	NxVec3 steer = NxVec3(0,0,1);
	NxQuat quat1 = a1->getGlobalOrientationQuat(); 
	quat1.w=-quat1.w;
	quat1.rotate(steer);
	NxVec3 turn = NxVec3(0,1,0);
	quat1.rotate(turn);

	NxVec3 steer0 = NxVec3(0,0,1);
	NxQuat quat0 = a0->getGlobalOrientationQuat(); 
	quat0.w=-quat0.w;
	quat0.rotate(steer0);
	NxVec3 turn0 = NxVec3(0,1,0);
	quat0.rotate(turn0);

	d6Desc.localAxis[0] = steer0;// steerx, steery, steerz      a0
	d6Desc.localAxis[1] = steer;
	d6Desc.localNormal[0] = turn0;// turnx, turny, turnz      a0
	d6Desc.localNormal[1] = turn;

	d6Desc.twistMotion = NX_D6JOINT_MOTION_FREE;
	d6Desc.swing1Motion = NX_D6JOINT_MOTION_LIMITED;
	d6Desc.swing2Motion = NX_D6JOINT_MOTION_LOCKED;

	d6Desc.xMotion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.yMotion = NX_D6JOINT_MOTION_LIMITED;
	d6Desc.zMotion = NX_D6JOINT_MOTION_LOCKED;

	d6Desc.linearLimit.value = 1;
	d6Desc.swing1Limit.value = NxPiF32 / 180.0f * 80.0f;

	//d6Desc.twistDrive.driveType.raiseFlagMask(NX_D6JOINT_DRIVE_VELOCITY);
	//d6Desc.twistDrive.driveType = NX_D6JOINT_DRIVE_VELOCITY;
	d6Desc.twistDrive.forceLimit = 1000.0f;

    d6Desc.swingDrive.driveType=NX_D6JOINT_DRIVE_POSITION;
	d6Desc.swingDrive.forceLimit=0.0f;
	d6Desc.swingDrive.spring=25000.0f;
	d6Desc.swingDrive.damping=0.01f;
	
//	d6Desc.projectionMode = NX_JPM_NONE;
	d6Desc.projectionMode = NX_JPM_POINT_MINDIST;

	NxD6Joint* d6Joint = (NxD6Joint*)gScene->createJoint(d6Desc);
	d6Joint->addLimitPlane(NxVec3(0,1,0), NxVec3(x,y,z));

	return (NxD6Joint*)d6Joint->is(NX_JOINT_D6);
}
BBDECL NxD6Joint* BBCALL pxJointCreateSuspBack(NxActor* a0, NxActor* a1, float x, float y, float z)
{
	NxD6JointDesc d6Desc;
	d6Desc.actor[0] = a0;
	d6Desc.actor[1] = a1;
	d6Desc.setGlobalAnchor(NxVec3(x,y,z));
//	d6Desc.setGlobalAxis(NxVec3(0,0,-1));
	NxVec3 steer = NxVec3(0,0,1);
	NxQuat quat1 = a1->getGlobalOrientationQuat(); 
	quat1.w=-quat1.w;
	quat1.rotate(steer);
	NxVec3 turn = NxVec3(0,1,0);
	quat1.rotate(turn);

	NxVec3 steer0 = NxVec3(0,0,1);
	NxQuat quat0 = a0->getGlobalOrientationQuat(); 
	quat0.w=-quat0.w;
	quat0.rotate(steer0);
	NxVec3 turn0 = NxVec3(0,1,0);
	quat0.rotate(turn0);

	d6Desc.localAxis[0] = steer0;// steerx, steery, steerz      a0
	d6Desc.localAxis[1] = steer;// steerx, steery, steerz      a1
//	d6Desc.localAxis[1].x = -steer.x;
//	d6Desc.localAxis[1].y = -steer.y;
	d6Desc.localNormal[0] = turn0;// turnx, turny, turnz      a0
	d6Desc.localNormal[1] = turn;// turnx, turny, turnz      a1

	d6Desc.twistMotion = NX_D6JOINT_MOTION_FREE;
	d6Desc.swing1Motion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.swing2Motion = NX_D6JOINT_MOTION_LOCKED;

	d6Desc.xMotion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.yMotion = NX_D6JOINT_MOTION_LIMITED;
	d6Desc.zMotion = NX_D6JOINT_MOTION_LOCKED;

	d6Desc.linearLimit.value = 1;
	d6Desc.swing1Limit.value = NxPiF32 / 180.0f * 80.0f;

	//d6Desc.twistDrive.driveType.raiseFlagMask(NX_D6JOINT_DRIVE_VELOCITY);
	//d6Desc.twistDrive.driveType = NX_D6JOINT_DRIVE_VELOCITY;
	d6Desc.twistDrive.forceLimit = 1000.0f;

    d6Desc.swingDrive.driveType=NX_D6JOINT_DRIVE_POSITION;
	d6Desc.swingDrive.forceLimit=0.0f;
	d6Desc.swingDrive.spring=25000.0f;
	d6Desc.swingDrive.damping=0.01f;
	
//	d6Desc.projectionMode = NX_JPM_NONE;
	d6Desc.projectionMode = NX_JPM_POINT_MINDIST;

	NxD6Joint* d6Joint = (NxD6Joint*)gScene->createJoint(d6Desc);
	d6Joint->addLimitPlane(NxVec3(0,1,0), NxVec3(x,y,z));
	//d6Joint->setDriveOrientation(NxQuat(0,NxVec3(0.0f,1.0f,0.0f))); 

	return (NxD6Joint*)d6Joint->is(NX_JOINT_D6);
}
BBDECL void BBCALL pxJointSuspSetSteerN(NxD6Joint* joint, float x, float y, float z)
{
    NxD6JointDesc d6Desc;
	NxActor* actor0;
	NxActor* actor1;
	joint->getActors(&actor0, &actor1);
	joint->saveToDesc(d6Desc);
	
	NxVec3 steer = NxVec3(x,y,z);
	NxQuat quat1 = actor1->getGlobalOrientationQuat(); 
	quat1.w=-quat1.w;
	quat1.rotate(steer);

	NxVec3 steer0 = NxVec3(x,y,z);
	NxQuat quat0 = actor0->getGlobalOrientationQuat(); 
	quat0.w=-quat0.w;
	quat0.rotate(steer0);
	
	steer0 = steer0/steer0.magnitude();
	steer = steer/steer.magnitude();
	d6Desc.localAxis[0] = steer0;
	d6Desc.localAxis[1] = steer;
	joint->loadFromDesc(d6Desc); 
}
BBDECL void BBCALL pxJointSuspSetTurnN(NxD6Joint* joint, float x, float y, float z)
{
    NxD6JointDesc d6Desc;
	NxActor* actor0;
	NxActor* actor1;
	joint->getActors(&actor0, &actor1);
	joint->saveToDesc(d6Desc);

	NxQuat quat1 = actor1->getGlobalOrientationQuat(); 
	NxVec3 turn = NxVec3(x,y,z);
	quat1.rotate(turn);

	NxQuat quat0 = actor0->getGlobalOrientationQuat(); 
	NxVec3 turn0 = NxVec3(x,y,z);
	quat0.rotate(turn0);

	turn0 = turn0/turn0.magnitude();
	turn = turn/turn.magnitude();
	d6Desc.localNormal[0] = turn0;
	d6Desc.localNormal[1] = turn;

	joint->loadFromDesc(d6Desc); 
}
BBDECL void BBCALL pxJointSuspSetLinLimit(NxD6Joint* joint, float lim)
{
    NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

	d6Desc.linearLimit.value = lim;

	joint->loadFromDesc(d6Desc); 
}
BBDECL void BBCALL pxJointSuspSetLinParameter(NxD6Joint* joint, float spring, float rest, float damp)
{
    NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

	d6Desc.linearLimit.restitution = rest;
	d6Desc.linearLimit.spring = spring;
	d6Desc.linearLimit.damping = damp;

	joint->loadFromDesc(d6Desc); 
}
BBDECL void BBCALL pxJointSuspSetAngLimit(NxD6Joint* joint, float lim)
{
    NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

	d6Desc.swing1Limit.value = NxPiF32 / 180.0f * lim;

	joint->loadFromDesc(d6Desc); 
}
BBDECL void BBCALL pxJointSuspSetAngParameter(NxD6Joint* joint, float spring, float rest, float damp)
{
    NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

	d6Desc.swing1Limit.restitution = rest;
	d6Desc.swing1Limit.spring = spring;
	d6Desc.swing1Limit.damping = damp;

	joint->loadFromDesc(d6Desc); 
}
BBDECL void BBCALL pxJointSuspSetAngle(NxD6Joint* joint, float angle)
{
	joint->setDriveOrientation(NxQuat(angle,NxVec3(0.0f,1.0f,0.0f))); 
}
BBDECL void BBCALL pxJointSuspSetSpeed(NxD6Joint* joint, float speed)
{
	NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

	NxActor* actor0;
	NxActor* actor1;
	joint->getActors(&actor0, &actor1);

	d6Desc.twistDrive.driveType.raiseFlagMask(NX_D6JOINT_DRIVE_VELOCITY);
	d6Desc.driveAngularVelocity.x = speed;
	NxVec3 moment = actor0->getAngularMomentum();  
	actor0->setAngularMomentum(moment);
	if (NxMath::abs(speed) < 0.1)d6Desc.twistDrive.driveType.lowerFlagMask(NX_D6JOINT_DRIVE_VELOCITY);

	joint->loadFromDesc(d6Desc); 
//	joint->setDriveAngularVelocity(NxVec3(speed,0,0));
}
BBDECL void BBCALL pxJointSuspSetBrake(NxD6Joint* joint, int mode)
{
	NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

	if (mode == 0)d6Desc.twistDrive.driveType.lowerFlagMask(NX_D6JOINT_DRIVE_VELOCITY);
	else 
	if (mode == 1)d6Desc.twistDrive.driveType.raiseFlagMask(NX_D6JOINT_DRIVE_VELOCITY);

	joint->loadFromDesc(d6Desc); 
}
BBDECL void BBCALL pxJointSuspSetMoment(NxD6Joint* joint, float speed)
{
	NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

	d6Desc.twistDrive.driveType.raiseFlagMask(NX_D6JOINT_DRIVE_VELOCITY);
	d6Desc.driveAngularVelocity.x = speed;
	if (NxMath::abs(speed) < 0.1)d6Desc.twistDrive.driveType.lowerFlagMask(NX_D6JOINT_DRIVE_VELOCITY);

	joint->loadFromDesc(d6Desc); 
	
//	joint->setDriveAngularVelocity(NxVec3(speed,0,0));
}
//---------Create Spring AndDamper Effector--------------------
BBDECL NxSpringAndDamperEffector* BBCALL pxCreateSpringAndDamperEffector(NxActor* body1, NxActor* body2)
{
	NxSpringAndDamperEffectorDesc sadeDesc;
	NxVec3 pos1 = body1->getCMassGlobalPosition();
	NxVec3 pos2 = body2->getCMassGlobalPosition();
	sadeDesc.body1 = body1;
	sadeDesc.body2 = body2;
	sadeDesc.pos1 = pos1;
	sadeDesc.pos2 = pos2;

	NxSpringAndDamperEffector *spring = gScene->createSpringAndDamperEffector(sadeDesc);

//	NxVec3 pos1 = body1->getCMassGlobalPosition();
//	NxVec3 pos2 = body2->getCMassGlobalPosition();
	NxVec3 springVec = pos2 - pos1;

	//spring->setBodies(body1, pos1, body2, pos2);

	NxReal distSpringRelaxed = springVec.magnitude();
	NxReal maxSpringCompressForce = 2000;
	NxReal distSpringCompressSaturate = 0.25f*distSpringRelaxed;
	NxReal maxSpringStretchForce = 2000;
	NxReal distSpringStretchSaturate = 2.0f*distSpringRelaxed;

	spring->setLinearSpring(distSpringCompressSaturate, distSpringRelaxed, distSpringStretchSaturate,
                        maxSpringCompressForce, maxSpringStretchForce);

	NxReal maxDamperCompressForce = 100;
	NxReal velDamperCompressSaturate = 200;
	NxReal maxDamperStretchForce = 100;
	NxReal velDamperStretchSaturate = 200;

	spring->setLinearDamper(velDamperCompressSaturate, velDamperStretchSaturate,
                        maxDamperCompressForce, maxDamperStretchForce);

	return spring;
}
BBDECL void BBCALL pxSetLinearSpring(NxSpringAndDamperEffector *spring, float SpringRelaxed,
									float CompressForce, float CompressSaturate,
									float StretchForce, float StretchSaturate)
{


	NxReal distSpringRelaxed = SpringRelaxed;
	NxReal maxSpringCompressForce = CompressForce;
	NxReal distSpringCompressSaturate = CompressSaturate*distSpringRelaxed;
	NxReal maxSpringStretchForce = StretchForce;
	NxReal distSpringStretchSaturate = StretchSaturate*distSpringRelaxed;

	spring->setLinearSpring(distSpringCompressSaturate, distSpringRelaxed, distSpringStretchSaturate,
                        maxSpringCompressForce, maxSpringStretchForce);


}
BBDECL void BBCALL pxSetLinearDamper(NxSpringAndDamperEffector *spring,
									float CompressForce, float CompressSaturate,
									float StretchForce, float StretchSaturate)
{


	NxReal maxDamperCompressForce = CompressForce;
	NxReal velDamperCompressSaturate = CompressSaturate;
	NxReal maxDamperStretchForce = StretchForce;
	NxReal velDamperStretchSaturate = StretchSaturate;

	spring->setLinearDamper(velDamperCompressSaturate, velDamperStretchSaturate,
                        maxDamperCompressForce, maxDamperStretchForce);


}
BBDECL void BBCALL pxDeleteEffector(NxEffector* effector)
{
	gScene->releaseEffector(*effector);
}
//---------Create CCD Sceleton--------------------
BBDECL void BBCALL pxCCDSkeletonEnable(int mode)
{
	gPhysicsSDK->setParameter(NX_CONTINUOUS_CD, NxReal(mode));
	gPhysicsSDK->setParameter(NX_CCD_EPSILON, 0.01f);
}
BBDECL void BBCALL pxCCDSkeletonSetEpsilon(float eps)
{
	gPhysicsSDK->setParameter(NX_CCD_EPSILON, eps);
}
static NxCCDSkeleton* CreateCCDSkeleton (float x, float y, float z)
{
	NxU32 triangles[3 * 12] = { 
		0,1,3,
		0,3,2,
		3,7,6,
		3,6,2,
		1,5,7,
		1,7,3,
		4,6,7,
		4,7,5,
		1,0,4,
		5,1,4,
		4,0,2,
		4,2,6
	};

	NxVec3 points[8];

	//static mesh
	points[0].set( x, -y, -z);
	points[1].set( x, -y,  z);
	points[2].set( x,  y, -z);
	points[3].set( x,  y,  z);

	points[4].set(-x, -y, -z);
	points[5].set(-x, -y,  z);
	points[6].set(-x,  y, -z);
	points[7].set(-x,  y,  z);

	NxSimpleTriangleMesh stm;
	stm.numVertices = 8;
	stm.numTriangles = 6*2;
	stm.pointStrideBytes = sizeof(NxVec3);
	stm.triangleStrideBytes = sizeof(NxU32)*3;

	stm.points = points;
	stm.triangles = triangles;
	stm.flags |= NX_MF_FLIPNORMALS;
	return gPhysicsSDK->createCCDSkeleton(stm);
}
BBDECL void BBCALL pxBodySetCCDSkeleton(NxActor* body, float x, float y, float z)
{
	NxShape* AllShapes = (NxShape*)body->getShapes()[0];
	AllShapes[0].setCCDSkeleton(CreateCCDSkeleton (x, y, z));
}
BBDECL void BBCALL pxBodySetFlagCCDSkeletonDynamic(NxActor* body)
{
	NxShape* AllShapes = (NxShape*)body->getShapes()[0];
	AllShapes[0].setFlag(NX_SF_DYNAMIC_DYNAMIC_CCD,true);
}
BBDECL void BBCALL pxBodySetCCDSkeletonThreshold(NxActor* body, float thres)
{
	body->setCCDMotionThreshold(thres);
}
BBDECL void BBCALL pxBodyGetCCDSkeleton(NxActor* body)
{
	NxShape*const* AllShapes = body->getShapes();
	int nbAllShapes = body->getNbShapes();
	for (int i=0; i<nbAllShapes; i++)
	{
		NxCCDSkeleton* sc = AllShapes[i]->getCCDSkeleton();
	}
}
BBDECL void BBCALL pxBodyDeleteCCDSkeleton(NxCCDSkeleton* ccd)
{
	gPhysicsSDK->releaseCCDSkeleton(*ccd);
}
//Create Ray
BBDECL NxRay* BBCALL pxCreateRay()
{
	NxRay* worldRay = new NxRay;
	return worldRay;
}
BBDECL void BBCALL pxRaySetDir(NxRay* ray, float nx, float ny, float nz)
{
	(ray)->dir.set(NxVec3(nx,ny,nz));
	(ray)->dir.normalize();
}
BBDECL void BBCALL pxRaySetPosition(NxRay* ray, float x, float y, float z)
{
	(ray)->orig = NxVec3(x, y, z);
}
BBDECL float BBCALL pxRayGetDistance(NxRay* ray, int mode)
{
	NxRaycastHit hit;
	NxShape* closestShape;
	if (mode == 0) closestShape = gScene->raycastClosestShape(*ray, NX_ALL_SHAPES, hit);
	else 
	if (mode == 1) closestShape = gScene->raycastClosestShape(*ray, NX_DYNAMIC_SHAPES, hit);
	else
	if (mode == 2) closestShape = gScene->raycastClosestShape(*ray, NX_STATIC_SHAPES, hit);
	
	if (closestShape)
	{
		return hit.distance;
	}
	else return 0;
}
BBDECL NxActor* BBCALL pxRayGetBody(NxRay* ray, int mode)
{
	NxRaycastHit hit;
	NxShape* closestShape;
	if (mode == 0) closestShape = gScene->raycastClosestShape(*ray, NX_ALL_SHAPES, hit);
	else 
	if (mode == 1) closestShape = gScene->raycastClosestShape(*ray, NX_DYNAMIC_SHAPES, hit);
	else
	if (mode == 2) closestShape = gScene->raycastClosestShape(*ray, NX_STATIC_SHAPES, hit);
	
	if (closestShape)
	{
		NxActor& body = hit.shape->getActor();
		return &body;
	}
	else return 0;
}
BBDECL int BBCALL pxRayGetMaterial(NxRay* ray, int mode)
{
	NxRaycastHit hit;
	NxShape* closestShape;
	if (mode == 0) closestShape = gScene->raycastClosestShape(*ray, NX_ALL_SHAPES, hit);
	else 
	if (mode == 1) closestShape = gScene->raycastClosestShape(*ray, NX_DYNAMIC_SHAPES, hit);
	else
	if (mode == 2) closestShape = gScene->raycastClosestShape(*ray, NX_STATIC_SHAPES, hit);
	
	if(closestShape)
	{
		NxMaterialIndex  material = hit.materialIndex;
		return material;
	}
	else return 0;
}
BBDECL float BBCALL pxRayGetPickX(NxRay* ray, int mode)
{
	NxRaycastHit hit;
	NxShape* closestShape; 
	if (mode == 0) closestShape = gScene->raycastClosestShape(*ray, NX_ALL_SHAPES, hit);
	else 
	if (mode == 1) closestShape = gScene->raycastClosestShape(*ray, NX_DYNAMIC_SHAPES, hit);
	else
	if (mode == 2) closestShape = gScene->raycastClosestShape(*ray, NX_STATIC_SHAPES, hit);
	
	if(closestShape)
	{
		return  hit.worldImpact.x;
	}
	else return 0;
}
BBDECL float BBCALL pxRayGetPickY(NxRay* ray, int mode)
{
	NxRaycastHit hit;
	NxShape* closestShape;
	if (mode == 0) closestShape = gScene->raycastClosestShape(*ray, NX_ALL_SHAPES, hit);
	else 
	if (mode == 1) closestShape = gScene->raycastClosestShape(*ray, NX_DYNAMIC_SHAPES, hit);
	else
	if (mode == 2) closestShape = gScene->raycastClosestShape(*ray, NX_STATIC_SHAPES, hit);
	
	if(closestShape)
	{
		return  hit.worldImpact.y;
	}
	else return 0;
}
BBDECL float BBCALL pxRayGetPickZ(NxRay* ray, int mode)
{
	NxRaycastHit hit;
	NxShape* closestShape;
	if (mode == 0) closestShape = gScene->raycastClosestShape(*ray, NX_ALL_SHAPES, hit);
	else 
	if (mode == 1) closestShape = gScene->raycastClosestShape(*ray, NX_DYNAMIC_SHAPES, hit);
	else
	if (mode == 2) closestShape = gScene->raycastClosestShape(*ray, NX_STATIC_SHAPES, hit);
	
	if(closestShape)
	{
		return  hit.worldImpact.z;
	}
	else return 0;
}
BBDECL float BBCALL pxRayGetPickNX(NxRay* ray, int mode)
{
	NxRaycastHit hit;
	NxShape* closestShape;
	if (mode == 0) closestShape = gScene->raycastClosestShape(*ray, NX_ALL_SHAPES, hit);
	else 
	if (mode == 1) closestShape = gScene->raycastClosestShape(*ray, NX_DYNAMIC_SHAPES, hit);
	else
	if (mode == 2) closestShape = gScene->raycastClosestShape(*ray, NX_STATIC_SHAPES, hit);
	
	if(closestShape)
	{
		return  hit.worldNormal.x;
	}
	else return 0;
}
BBDECL float BBCALL pxRayGetPickNY(NxRay* ray, int mode)
{
	NxRaycastHit hit;
	NxShape* closestShape;
	if (mode == 0) closestShape = gScene->raycastClosestShape(*ray, NX_ALL_SHAPES, hit);
	else 
	if (mode == 1) closestShape = gScene->raycastClosestShape(*ray, NX_DYNAMIC_SHAPES, hit);
	else
	if (mode == 2) closestShape = gScene->raycastClosestShape(*ray, NX_STATIC_SHAPES, hit);
	
	if(closestShape)
	{
		return  hit.worldNormal.y;
	}
	else return 0;
}
BBDECL float BBCALL pxRayGetPickNZ(NxRay* ray, int mode)
{
	NxRaycastHit hit;
	NxShape* closestShape;
	if (mode == 0) closestShape = gScene->raycastClosestShape(*ray, NX_ALL_SHAPES, hit);
	else 
	if (mode == 1) closestShape = gScene->raycastClosestShape(*ray, NX_DYNAMIC_SHAPES, hit);
	else
	if (mode == 2) closestShape = gScene->raycastClosestShape(*ray, NX_STATIC_SHAPES, hit);
	
	if(closestShape)
	{
		return  hit.worldNormal.z;
	}
	else return 0;  
}
//---------Regiser------------
BBDECL int BBCALL pxRegWriteDriverPath(char* str)
{
	HKEY hKey;
	DWORD dwDisp = 0;
	LPDWORD lpdwDisp = &dwDisp;
	char* l_strExampleKey = "SOFTWARE\\AGEIA Technologies";
	char* l_strDWordSample = "PhysXCore Path";

	LONG iSuccess = RegCreateKeyEx(HKEY_LOCAL_MACHINE, l_strExampleKey, 0L,NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey,lpdwDisp);

	if(iSuccess == ERROR_SUCCESS)
	{
		RegSetValueEx (hKey, l_strDWordSample, 0L, REG_SZ,(BYTE*) str, strlen(str));
		return 1;
	}
	return 0;
}
BBDECL int BBCALL pxRegWriteString(int RootKey, char* Path, char* Name, char* Data)
{
	HKEY hKey;
	HKEY key;
	if(RootKey == 0) key = HKEY_CLASSES_ROOT;
	if(RootKey == 1) key = HKEY_CURRENT_USER;
	if(RootKey == 2) key = HKEY_LOCAL_MACHINE;
	if(RootKey == 3) key = HKEY_USERS;
	if(RootKey == 4) key = HKEY_PERFORMANCE_DATA;
	DWORD dwDisp = 0;
	LPDWORD lpdwDisp = &dwDisp;

	LONG iSuccess = RegCreateKeyEx(key, Path, 0L,NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey,lpdwDisp);

	if(iSuccess == ERROR_SUCCESS)
	{
		RegSetValueEx (hKey, Name, 0L, REG_SZ,(BYTE*) Data, strlen(Data));
		return 1;
	}
	
	return 0;
}
BBDECL int BBCALL pxRegWriteInt(int RootKey, char* Path, char* Name, int Data)
{
	HKEY hKey;
	HKEY key;
	if(RootKey == 0) key = HKEY_CLASSES_ROOT;
	if(RootKey == 1) key = HKEY_CURRENT_USER;
	if(RootKey == 2) key = HKEY_LOCAL_MACHINE;
	if(RootKey == 3) key = HKEY_USERS;
	if(RootKey == 4) key = HKEY_PERFORMANCE_DATA;
	DWORD dwDisp = 0;
	LPDWORD lpdwDisp = &dwDisp;

	LONG iSuccess = RegCreateKeyEx(key, Path, 0L,NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey,lpdwDisp);

	if(iSuccess == ERROR_SUCCESS)
	{
		RegSetValueEx (hKey, Name, 0L, REG_DWORD,(BYTE*) &Data, sizeof(DWORD));
		return 1;
	}
	
	return 0;
}
BBDECL int BBCALL pxRegReadInt(int RootKey, char* Path, char* Name)
{
	HKEY hKey;
	HKEY key;
	if(RootKey == 0) key = HKEY_CLASSES_ROOT;
	if(RootKey == 1) key = HKEY_CURRENT_USER;
	if(RootKey == 2) key = HKEY_LOCAL_MACHINE;
	if(RootKey == 3) key = HKEY_USERS;
	if(RootKey == 4) key = HKEY_PERFORMANCE_DATA;
	DWORD dwDisp = 0;
	LPDWORD lpdwDisp = &dwDisp;

	DWORD dwVersion;
	LONG returnStatus;
	DWORD dwType=REG_DWORD;	
	DWORD dwSize=sizeof(DWORD);
    returnStatus = RegOpenKeyEx(key, Path, 0L, KEY_ALL_ACCESS, &hKey);

    if (returnStatus  == ERROR_SUCCESS)
    {
        returnStatus = RegQueryValueEx(hKey, Name, NULL, &dwType,(LPBYTE)&dwVersion, &dwSize);
        if (returnStatus == ERROR_SUCCESS)
        {
             return (int)dwVersion;
        }
     }

    RegCloseKey(hKey);
	return -1;
}

char g_lszValue[1023];
BBDECL char* BBCALL pxRegReadString(int RootKey, char* Path, char* Name)
{
	HKEY hKey;
	HKEY key;
	if(RootKey == 0) key = HKEY_CLASSES_ROOT;
	if(RootKey == 1) key = HKEY_CURRENT_USER;
	if(RootKey == 2) key = HKEY_LOCAL_MACHINE;
	if(RootKey == 3) key = HKEY_USERS;
	if(RootKey == 4) key = HKEY_PERFORMANCE_DATA;

     
     LONG returnStatus;
     DWORD dwType=REG_SZ;
     DWORD dwSize=1023;
     returnStatus = RegOpenKeyEx(key, Path, NULL,  KEY_ALL_ACCESS, &hKey);
     if (returnStatus == ERROR_SUCCESS)
     {
          returnStatus = RegQueryValueEx(hKey, Name, NULL, &dwType,(LPBYTE)&g_lszValue, &dwSize);
          if (returnStatus == ERROR_SUCCESS)
          {
			  RegCloseKey(hKey);
              return g_lszValue;
          }
     }
	 return (char*)"0";
}
BBDECL int BBCALL pxRegDeleteValue(int RootKey, char* Path, char* Name)
{
	HKEY hKey;
	HKEY key;
	if(RootKey == 0) key = HKEY_CLASSES_ROOT;
	if(RootKey == 1) key = HKEY_CURRENT_USER;
	if(RootKey == 2) key = HKEY_LOCAL_MACHINE;
	if(RootKey == 3) key = HKEY_USERS;
	if(RootKey == 4) key = HKEY_PERFORMANCE_DATA;
	DWORD dwDisp = 0;
	LPDWORD lpdwDisp = &dwDisp;

     LONG returnStatus;
     returnStatus = RegOpenKeyEx(key, Path, 0L,  KEY_ALL_ACCESS, &hKey);
     if (returnStatus == ERROR_SUCCESS)
     {
          returnStatus = RegDeleteValue(hKey, Name);
     }
	 return 0;
}
BBDECL int BBCALL pxRegDeleteKey(int RootKey, char* Path, char* Name)
{
	HKEY hKey;
	HKEY key;
	if(RootKey == 0) key = HKEY_CLASSES_ROOT;
	if(RootKey == 1) key = HKEY_CURRENT_USER;
	if(RootKey == 2) key = HKEY_LOCAL_MACHINE;
	if(RootKey == 3) key = HKEY_USERS;
	if(RootKey == 4) key = HKEY_PERFORMANCE_DATA;
	DWORD dwDisp = 0;
	LPDWORD lpdwDisp = &dwDisp;

     LONG returnStatus;
     returnStatus = RegOpenKeyEx(key, Path, 0L,  KEY_ALL_ACCESS, &hKey);
     if (returnStatus == ERROR_SUCCESS)
     {
          returnStatus = RegDeleteKey(hKey, Name);
		  return 1;
     }
	 return 0;
}
//Contact report
class MyContactReport : public NxUserContactReport
{
	public:
	void onContactNotify(NxContactPair& pair, NxU32 events)
	{
		//if (events & NX_NOTIFY_ON_START_TOUCH)
		

		NxContactStreamIterator i(pair.stream);
		
		NxActor* actor0 = pair.actors[0];
		NxActor* actor1 = pair.actors[1];

		
		if (pair.isDeletedActor[0] == 1 || pair.isDeletedActor[1] == 1) return;

		BodyData* data0 = (BodyData*)actor0->userData;
		BodyData* data1 = (BodyData*)actor1->userData;


		if (data0->contacttable == 1 && data1->contacttable == 1)
		{
			while(i.goNextPair())
			{
				while(i.goNextPatch())
				{
					const NxVec3& contactNormal = i.getPatchNormal();
					while(i.goNextPoint())
					{
						NxShape * tShape1;
						NxShape * tShape2;
						tShape1 = i.getShape(0);
						tShape2 = i.getShape(1);

						if (i.isDeletedShape(0)==0 && i.isDeletedShape(1)==0)//!tShape1->isWheel() && !tShape2->isWheel())
						{

							if (!tShape1->isWheel() && !tShape2->isWheel()) 
							{
						//	data0->nContacts = 0;
						//	data1->nContacts = 0;
							if(data0->nContacts < MAXCONTACTS)
							{
								data0->Contacts[data0->nContacts].pos = i.getPoint();
								data0->Contacts[data0->nContacts].normal = i.getPatchNormal();
	
								data0->Contacts[data0->nContacts].nforce = pair.sumNormalForce;
								data0->Contacts[data0->nContacts].tforce = pair.sumFrictionForce;
								data0->Contacts[data0->nContacts].body = actor1;
								data0->Contacts[data0->nContacts].shapes = tShape1;
								data0->contactEvent = events;
								data0->nContacts++;
							}
							if(data1->nContacts < MAXCONTACTS)
							{
								data1->Contacts[data1->nContacts].pos = i.getPoint();
								data1->Contacts[data1->nContacts].normal = i.getPatchNormal();
	
								data1->Contacts[data1->nContacts].nforce = pair.sumNormalForce;
								data1->Contacts[data1->nContacts].tforce = pair.sumFrictionForce;
								data1->Contacts[data1->nContacts].body = actor0;
								data1->Contacts[data1->nContacts].shapes = tShape2;
								data1->contactEvent = events;
								data1->nContacts++;
							}
							}
						}
					}
				}
			}
		}
	}
}gMyContactReport;
BBDECL NxActor* BBCALL pxContactGetBody(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].body;
}
BBDECL NxShape* BBCALL pxContactGetShape(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].shapes;
}
BBDECL int BBCALL pxGetContacts(NxActor* body)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->nContacts;
}
BBDECL float BBCALL pxContactGetPointX(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].pos[0];
}
BBDECL float BBCALL pxContactGetPointY(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].pos[1];
}
BBDECL float BBCALL pxContactGetPointZ(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].pos[2];
}
BBDECL float BBCALL pxContactGetPointNX(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].normal[0];
}
BBDECL float BBCALL pxContactGetPointNY(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].normal[1];
}
BBDECL float BBCALL pxContactGetPointNZ(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].normal[2];
}
BBDECL float BBCALL pxContactGetForceN(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
//	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].nforce.magnitude();
}
BBDECL float BBCALL pxContactGetForceNX(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].nforce.x;
}
BBDECL float BBCALL pxContactGetForceNY(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].nforce.y;
}
BBDECL float BBCALL pxContactGetForceNZ(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].nforce.z;
}
BBDECL float BBCALL pxContactGetForceT(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].tforce.magnitude();
}
BBDECL float BBCALL pxContactGetForceTX(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].tforce[0];
}
BBDECL float BBCALL pxContactGetForceTY(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].tforce[1];
}
BBDECL float BBCALL pxContactGetForceTZ(NxActor* body, int coll)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	return data->Contacts[coll].tforce[2];
}
BBDECL int BBCALL pxContactEventsOnStartTouch(NxActor* body)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	if (data->contactEvent & NX_NOTIFY_ON_START_TOUCH) return 1;
	return 0;
}
BBDECL int BBCALL pxContactEventsOnEndTouch(NxActor* body)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->nContacts < 1) return 0;
	if (data->contactEvent & NX_NOTIFY_ON_END_TOUCH) return 1;
	return 0;
}
//Delete Object
BBDECL void BBCALL pxDeleteBody(NxActor* body)
{
	if(body)
	{
		BodyData* data = (BodyData*)body->userData;
		data->dTriggerActors.clear();
		delete data;
		data = 0;
		gScene->releaseActor(*body);
		body->userData = 0;
		body = NULL;
	}
}

BBDECL void BBCALL pxDeleteJoint(NxJoint* joint)
{
		
		if(joint)
		{
			gScene->releaseJoint(*joint);
			joint = NULL;
		}
}
//Set Mask and Collision
BBDECL void BBCALL pxBodySetMagnetMask(NxActor* body, int mask)
{
	BodyData* data = (BodyData*)body->userData;
	data->magnetMask = mask;
}
BBDECL int BBCALL pxBodyGetMagnetMask(NxActor* body)
{
	BodyData* data = (BodyData*)body->userData;
	return data->magnetMask;
}
BBDECL void BBCALL pxMaskSet(NxActor* body, int mask)
{
	NxGroupsMask GrMask;//bits0, bits1, bits2, bits3;
//	GrMask.bits0 = mask & 1;
//	GrMask.bits1 = mask & 2;
//	GrMask.bits2 = mask & 4;
//	GrMask.bits3 = mask & 8;
	if (mask==0) 
	{
		GrMask.bits0 = 1;
		GrMask.bits1 = 0;
		GrMask.bits2 = 0;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==1) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 1;
		GrMask.bits2 = 0;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==2) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 0;
		GrMask.bits2 = 1;
		GrMask.bits3 = 0;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}
	else
	if (mask==3) 
	{
		GrMask.bits0 = 0;
		GrMask.bits1 = 0;
		GrMask.bits2 = 0;
		GrMask.bits3 = 1;
		//shapes[IDsh]->setGroupsMask(GrMask);
	}

	NxShape*const* AllShapes = body->getShapes();
	int nbAllShapes = body->getNbShapes();
	for (int i=0; i<nbAllShapes; i++)
	{
		AllShapes[i]->setGroupsMask(GrMask);
	}
}

BBDECL void BBCALL pxMaskCombineSet(NxActor* body, int mask)
{
	NxGroupsMask GrMask;
	GrMask.bits0 = mask & 1;
	GrMask.bits1 = mask & 2;
	GrMask.bits2 = mask & 4;
	GrMask.bits3 = mask & 8;

	NxShape*const* AllShapes = body->getShapes();
	int nbAllShapes = body->getNbShapes();
	for (int i=0; i<nbAllShapes; i++)
	{
		AllShapes[i]->setGroupsMask(GrMask);
	}
}

BBDECL void BBCALL pxMaskClear(NxActor* body)
{
	shapes = body->getShapes(); 
	int IDsh = body->getNbShapes()-1;

	NxGroupsMask GrMask;//bits0, bits1, bits2, bits3;

		GrMask.bits0 = 0xffffffff;
		GrMask.bits1 = 0xffffffff;
		GrMask.bits2 = 0xffffffff;
		GrMask.bits3 = 0xffffffff;

	shapes[IDsh]->setGroupsMask(GrMask);
}
BBDECL void BBCALL pxBodySetCollisionGroup(NxActor* body, int group)
{
	NxShape*const* AllShapes = body->getShapes();
	int nbAllShapes = body->getNbShapes();
	for (int i=0; i<nbAllShapes; i++)
	{
		AllShapes[i]->setGroup(group);
	}
}
BBDECL void BBCALL pxBodySetCollisionGroupPair(int group1, int group2)
{
	gScene->setActorGroupPairFlags(group1,group2,NX_NOTIFY_ON_START_TOUCH|NX_NOTIFY_ON_TOUCH|NX_NOTIFY_ON_END_TOUCH|NX_NOTIFY_FORCES);
}
BBDECL void BBCALL pxBodySetCollisionGroupFlag(int group1, int group2, int flag)
{
	gScene->setGroupCollisionFlag(group1, group2, flag > 0);
}
//Set Flags
BBDECL void BBCALL pxBodySetFrozenRotX(NxActor* body, int stat)
{
	if (stat == 1)	body->raiseBodyFlag(NX_BF_FROZEN_ROT_X);
	if (stat == 0)	body->clearBodyFlag(NX_BF_FROZEN_ROT_X);
}
BBDECL void BBCALL pxBodySetFrozenRotY(NxActor* body, int stat)
{
	if (stat == 1)	body->raiseBodyFlag(NX_BF_FROZEN_ROT_Y);
	if (stat == 0)	body->clearBodyFlag(NX_BF_FROZEN_ROT_Y);
}
BBDECL void BBCALL pxBodySetFrozenRotZ(NxActor* body, int stat)
{
	if (stat == 1)	body->raiseBodyFlag(NX_BF_FROZEN_ROT_Z);
	if (stat == 0)	body->clearBodyFlag(NX_BF_FROZEN_ROT_Z);
}
BBDECL void BBCALL pxBodySetFrozenRot(NxActor* body, int stat)
{
	if (stat == 1)	body->raiseBodyFlag(NX_BF_FROZEN_ROT);
	if (stat == 0)	body->clearBodyFlag(NX_BF_FROZEN_ROT);
}
BBDECL void BBCALL pxBodySetFrozenPosX(NxActor* body, int stat)
{
	if (stat == 1)	body->raiseBodyFlag(NX_BF_FROZEN_POS_X);
	if (stat == 0)	body->clearBodyFlag(NX_BF_FROZEN_POS_X);
}
BBDECL void BBCALL pxBodySetFrozenPosY(NxActor* body, int stat)
{
	if (stat == 1)	body->raiseBodyFlag(NX_BF_FROZEN_POS_Y);
	if (stat == 0)	body->clearBodyFlag(NX_BF_FROZEN_POS_Y);
}
BBDECL void BBCALL pxBodySetFrozenPosZ(NxActor* body, int stat)
{
	if (stat == 1)	body->raiseBodyFlag(NX_BF_FROZEN_POS_Z);
	if (stat == 0)	body->clearBodyFlag(NX_BF_FROZEN_POS_Z);
}
BBDECL void BBCALL pxBodySetFrozenPos(NxActor* body, int stat)
{
	if (stat == 1)	body->raiseBodyFlag(NX_BF_FROZEN_POS);
	if (stat == 0)	body->clearBodyFlag(NX_BF_FROZEN_POS);
}
BBDECL void BBCALL pxBodySetFrozen(NxActor* body, int stat)
{
	if (stat == 1)	body->raiseBodyFlag(NX_BF_FROZEN);
	if (stat == 0)	body->clearBodyFlag(NX_BF_FROZEN);
}

BBDECL void BBCALL pxBodySetFlagGravity(NxActor* body, int stat)
{
	if (stat == 0)	body->raiseBodyFlag(NX_BF_DISABLE_GRAVITY);
	if (stat == 1)	body->clearBodyFlag(NX_BF_DISABLE_GRAVITY);
}

BBDECL void BBCALL pxBodySetFlagCollision(NxActor* body, int stat)
{
	if (stat == 0)	body->raiseActorFlag(NX_AF_DISABLE_COLLISION);
	if (stat == 1)	body->clearActorFlag(NX_AF_DISABLE_COLLISION);
}

BBDECL void BBCALL pxBodySetFlagResponse(NxActor* body, int stat)
{
	if (stat == 0)	body->raiseActorFlag(NX_AF_DISABLE_RESPONSE);
	if (stat == 1)	body->clearActorFlag(NX_AF_DISABLE_RESPONSE);
}

BBDECL void BBCALL pxBodySetFlagContacttable(NxActor* body, int stat)
{
	BodyData* data = (BodyData*)body->userData;
	data->contacttable = stat;
}
BBDECL void BBCALL pxBodySetFlagRayCast(NxActor* body, int stat)
{
	NxShape*const* AllShapes = body->getShapes();
	int nbAllShapes = body->getNbShapes();
	for (int i=0; i<nbAllShapes; i++)
	{
		AllShapes[i]->setFlag(NX_SF_DISABLE_RAYCASTING, (1-stat > 0));
	}
}
//Create Joint
BBDECL NxJoint* BBCALL pxJointCreateHinge(NxActor* body1, NxActor* body2, float x, float y, float z, float nx, float ny, float nz)
{
	NxRevoluteJointDesc revoluteDesc;

	revoluteDesc.actor[0] = body1;
	revoluteDesc.actor[1] = body2;

	revoluteDesc.setGlobalAxis(NxVec3(nx,ny,nz)); //rotate around the x axis.
	revoluteDesc.setGlobalAnchor(NxVec3(x,y,z)); //anchor, point the axis passes through.
	//revoluteDesc.jointFlags |= NX_JF_COLLISION_ENABLED;

	NxJoint* hingeJoint = gScene->createJoint(revoluteDesc);
	return hingeJoint;
}

BBDECL NxSphericalJoint* BBCALL pxJointCreateSpherical(NxActor* body1, NxActor* body2, float x, float y, float z, float nx, float ny, float nz)
{
	NxSphericalJointDesc sphericalDesc;

	sphericalDesc.actor[0] = body1;
	sphericalDesc.actor[1] = body2;
	
	//sphericalDesc.flags |= NX_SJF_SWING_LIMIT_ENABLED;
	//sphericalDesc.swingAxis = NxVec3(0,1,0); 
	//sphericalDesc.swingLimit.value = 30*NxPi/180; 

	sphericalDesc.setGlobalAnchor(NxVec3(x,y,z)); 
	sphericalDesc.setGlobalAxis(NxVec3(nx, ny, nz));

	//sphericalDesc.jointFlags |= NX_JF_COLLISION_ENABLED;

	//(NxSphericalJoint*) sphereJoint = gScene->createJoint(sphericalDesc);
	return (NxSphericalJoint*)gScene->createJoint(sphericalDesc);//sphereJoint;
}

BBDECL NxDistanceJoint* BBCALL pxJointCreateDistance(NxActor* body1, NxActor* body2, float p1_x, float p1_y, float p1_z, float p2_x, float p2_y, float p2_z)
{
	NxDistanceJointDesc distDesc;    
	
	distDesc.actor[0] = body1;    
	distDesc.actor[1] = body2;    

	distDesc.localAnchor[0]=NxVec3(p1_x,p1_y,p1_z); //point which is constrained on actor0    
	distDesc.localAnchor[1]=NxVec3(p2_x,p2_y,p2_z); //point which is constrained in actor1
	//distDesc.jointFlags |= NX_JF_COLLISION_ENABLED;

	distDesc.maxDistance = 1;
	distDesc.minDistance = 0; 

	distDesc.flags = NX_DJF_MIN_DISTANCE_ENABLED | NX_DJF_MAX_DISTANCE_ENABLED; //Enforce a fixed distance.    

	NxDistanceJoint* distJoint=(NxDistanceJoint*)gScene->createJoint(distDesc); 
	return distJoint;
}

BBDECL NxJoint* BBCALL pxJointCreateFixed(NxActor* body1, NxActor* body2)
{
	NxFixedJointDesc fixedDesc;

	fixedDesc.actor[0] = body1;
	fixedDesc.actor[1] = body2;

	NxJoint *fixedJoint= gScene->createJoint(fixedDesc);

	return fixedJoint;
}

BBDECL NxJoint* BBCALL pxJointCreateCylindrical(NxActor* body1, NxActor* body2, float x, float y, float z, float nx, float ny, float nz, float min_limit, float max_limit)
{

	NxCylindricalJointDesc cylDesc;
	NxVec3 globalAnchor(x,y,z);
	NxVec3 globalAxis(nx,ny,nz);

	cylDesc.actor[0] = body1;
	cylDesc.actor[1] = body2;

	cylDesc.setGlobalAnchor(globalAnchor);
	cylDesc.setGlobalAxis(globalAxis);

	NxJoint* joint = gScene->createJoint(cylDesc);

	//joint->setLimitPoint(globalAnchor);

	// Add up-down limiting planes(to restrict how far the bodies can slide relative to eachother)
	joint->addLimitPlane(-globalAxis, globalAnchor + max_limit * globalAxis);
	joint->addLimitPlane(globalAxis, globalAnchor + min_limit * globalAxis);

	return joint;
}

BBDECL NxJoint* BBCALL pxJointCreatePrismatic(NxActor* body1, NxActor* body2, float x, float y, float z, float nx, float ny, float nz, float min_limit, float max_limit)
{

	NxPrismaticJointDesc prismaticDesc;
	NxVec3 globalAnchor(x,y,z);
	NxVec3 globalAxis(nx,ny,nz);

	prismaticDesc.actor[0] = body1;
	prismaticDesc.actor[1] = body2;

	prismaticDesc.setGlobalAnchor(globalAnchor); //The point constrained to the slide axis.
	prismaticDesc.setGlobalAxis(globalAxis); //The axis along which to slide.
	
	NxJoint* prismJoint = gScene->createJoint(prismaticDesc);


	//prismJoint->setLimitPoint(globalAnchor); //constrain sliding a a specific range
	prismJoint->addLimitPlane(-globalAxis, globalAnchor + max_limit*globalAxis);
	prismJoint->addLimitPlane(globalAxis, globalAnchor + min_limit*globalAxis);

	return prismJoint;
}
BBDECL NxPointOnLineJoint* BBCALL pxJointCreateOnLine(NxActor* body1, float x, float y, float z, float nx, float ny, float nz)
{

	NxPointOnLineJointDesc polDesc;

	NxVec3 globalAnchor(x,y,z);
	NxVec3 globalAxis(nx,ny,nz);

	polDesc.actor[0] = 0;
	polDesc.actor[1] = body1;

	polDesc.setGlobalAnchor(globalAnchor); // Point which is attached to the line.
	polDesc.setGlobalAxis(globalAxis); // Direction of line

	polDesc.jointFlags |= NX_JF_COLLISION_ENABLED;//enable collisions between actor0 and actor1

	NxPointOnLineJoint* polJoint = (NxPointOnLineJoint*)gScene->createJoint(polDesc);

	return polJoint;
}
BBDECL NxPointInPlaneJoint* BBCALL pxJointCreateInPlane(NxActor* body1, float x, float y, float z, float nx, float ny, float nz)
{
	NxPointInPlaneJointDesc pipDesc;

	NxVec3 globalAnchor(x,y,z);
	NxVec3 globalAxis(nx,ny,nz);


	pipDesc.actor[0] = 0;
	pipDesc.actor[1] = body1;

	pipDesc.setGlobalAnchor(globalAnchor); //point attached to plane
	pipDesc.setGlobalAxis(globalAxis); //point to attach

	pipDesc.jointFlags |= NX_JF_COLLISION_ENABLED; // enable collision detection between each actor.

	NxPointInPlaneJoint *pipJoint=(NxPointInPlaneJoint*)gScene->createJoint(pipDesc);
	return pipJoint;
}
BBDECL NxPulleyJoint* BBCALL pxJointCreatePulley(NxActor* body1, NxActor* body2, float distance, float stiff, float k_ratio)
{
	NxPulleyJointDesc pulleyDesc;

	NxVec3 pulley0(-5,40,0);
	NxVec3 pulley1(5,40,0);

	pulleyDesc.actor[0] = body1;
	pulleyDesc.actor[1] = body2;

	pulleyDesc.localAnchor[0] = NxVec3(0,1,0); //point on actor0 the rope is attached to.
	pulleyDesc.localAnchor[1] = NxVec3(0,1,0); //point on actor1 the rope is attached to.

	pulleyDesc.pulley[0] = pulley0;     // suspension points of two bodies in world space.
	pulleyDesc.pulley[1] = pulley1;     // suspension points of two bodies in world space.
	pulleyDesc.distance = distance;     // the rest length of the rope connecting the two objects.  
	pulleyDesc.stiffness = stiff;        // how stiff the constraint is, between 0 and 1 (stiffest)
	pulleyDesc.ratio = k_ratio;            // transmission ratio(eg if set to 2 would cause actor1 to move twice as far as actor0)
	pulleyDesc.flags = NX_PJF_IS_RIGID; // NX_PJF_IS_RIGID instructs the joint to maintain the minimum length/distance as well as the maximum. 
	pulleyDesc.jointFlags |= NX_JF_COLLISION_ENABLED;

	NxPulleyJoint* pulleyJoint = (NxPulleyJoint *)gScene->createJoint(pulleyDesc);
	return pulleyJoint;
}
//Create D6Joint
BBDECL NxD6Joint* BBCALL pxJointCreateD6Joint(NxActor* body1, NxActor* body2, float x, float y, float z, float nx, float ny, float nz)
{
	NxD6JointDesc d6Desc;
	d6Desc.actor[0] = body1;
	d6Desc.actor[1] = body2;
	d6Desc.setGlobalAnchor(NxVec3(x ,y ,z));
	d6Desc.setGlobalAxis(NxVec3(nx ,ny ,nz));

	d6Desc.twistMotion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.swing1Motion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.swing2Motion = NX_D6JOINT_MOTION_LOCKED;

	d6Desc.xMotion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.yMotion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.zMotion = NX_D6JOINT_MOTION_LOCKED;

	NxReal gLinearLimit = 5.0f;
	NxReal gSwing1Limit = NxPiF32 / 180.0f * 30.0f;
	NxReal gSwing2Limit = NxPiF32 / 180.0f * 70.0f;
	NxReal gTwistLowLimit = NxPiF32 / 180.0f * -90.0f;
	NxReal gTwistHighLimit = NxPiF32 / 180.0f * 45.0f;

	d6Desc.linearLimit.value = gLinearLimit;
	d6Desc.swing1Limit.value = gSwing1Limit;
	d6Desc.swing2Limit.value = gSwing2Limit;
	d6Desc.twistLimit.low.value =  gTwistLowLimit;
	d6Desc.twistLimit.high.value = gTwistHighLimit;

//	d6Desc.projectionMode = NX_JPM_NONE;
	d6Desc.projectionMode = NX_JPM_POINT_MINDIST;

//	d6Desc.jointFlags |= NX_JF_COLLISION_ENABLED;

	NxJoint* d6Joint = gScene->createJoint(d6Desc);

	return (NxD6Joint*)d6Joint->is(NX_JOINT_D6);
}
BBDECL void BBCALL pxD6JointSetPosMotion(NxD6Joint* joint, int xMot, int yMot, int zMot)
{
    NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

	// set the DOF
	if (!xMot) d6Desc.xMotion = NX_D6JOINT_MOTION_LOCKED;
	else
	if (xMot == 1) d6Desc.xMotion = NX_D6JOINT_MOTION_LIMITED;
	else
	if (xMot == 2) d6Desc.xMotion = NX_D6JOINT_MOTION_FREE;

	if (yMot == 0) d6Desc.yMotion = NX_D6JOINT_MOTION_LOCKED;
	else
	if (yMot == 1) d6Desc.yMotion = NX_D6JOINT_MOTION_LIMITED;
	else
	if (yMot == 2) d6Desc.yMotion = NX_D6JOINT_MOTION_FREE;

	if (zMot == 0) d6Desc.zMotion = NX_D6JOINT_MOTION_LOCKED;
	else
	if (zMot == 1) d6Desc.zMotion = NX_D6JOINT_MOTION_LIMITED;
	else
	if (zMot == 2) d6Desc.zMotion = NX_D6JOINT_MOTION_FREE;


	joint->loadFromDesc(d6Desc);
}

BBDECL void BBCALL pxD6JointSetAngleMotion(NxD6Joint* joint, int twistMot, int swing1Mot, int swing2Mot)
{
    NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

	// set the DOF
	if (twistMot == 0) d6Desc.twistMotion = NX_D6JOINT_MOTION_LOCKED;
	else
	if (twistMot == 1) d6Desc.twistMotion = NX_D6JOINT_MOTION_LIMITED;
	else
	if (twistMot == 2) d6Desc.twistMotion = NX_D6JOINT_MOTION_FREE;

	if (swing1Mot == 0) d6Desc.swing1Motion = NX_D6JOINT_MOTION_LOCKED;
	else
	if (swing1Mot == 1) d6Desc.swing1Motion = NX_D6JOINT_MOTION_LIMITED;
	else
	if (swing1Mot == 2) d6Desc.swing1Motion = NX_D6JOINT_MOTION_FREE;

	if (swing2Mot == 0) d6Desc.swing2Motion = NX_D6JOINT_MOTION_LOCKED;
	else
	if (swing2Mot == 1) d6Desc.swing2Motion = NX_D6JOINT_MOTION_LIMITED;
	else
	if (swing2Mot == 2) d6Desc.swing2Motion = NX_D6JOINT_MOTION_FREE;

	joint->loadFromDesc(d6Desc);
}
BBDECL void BBCALL pxD6JointSetLinearLimit(NxD6Joint* joint, float lim)
{
    NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

		d6Desc.linearLimit.value = lim;

	joint->loadFromDesc(d6Desc);
}
BBDECL void BBCALL pxD6JointSetSwingLimit(NxD6Joint* joint, float swing1Lim, float swing2Lim)
{
    NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

		d6Desc.swing1Limit.value = swing1Lim*3.14f/180;
		d6Desc.swing2Limit.value = swing2Lim*3.14f/180;

	joint->loadFromDesc(d6Desc);
}
BBDECL void BBCALL pxD6JointSetTwistLimit(NxD6Joint* joint, float lowLim, float heighLim)
{
    NxD6JointDesc d6Desc;
	joint->saveToDesc(d6Desc);

		d6Desc.twistLimit.low.value = lowLim*3.14f/180;
		d6Desc.twistLimit.high.value = heighLim*3.14f/180;

	joint->loadFromDesc(d6Desc);
}
BBDECL NxActor* BBCALL pxJCreateCarFrontJoint(NxActor* body1, NxActor* body2, float x, float y, float z, float nx, float ny, float nz)
{
	NxBodyDesc BodyDesc;
	NxActorDesc *ActorDesc = new NxActorDesc;
	ActorDesc->density		= 1;
	
	NxBoxShapeDesc BoxDesc;
	BoxDesc.dimensions = NxVec3(0, 0, 0);
	BoxDesc.groupsMask.bits0 = 1;

	ActorDesc->shapes.pushBack(&BoxDesc);
	ActorDesc->body = &BodyDesc;

	NxActor* body = gScene->createActor(*ActorDesc);
			body->setMass(1);
			body->setGlobalPosition(NxVec3(x ,y ,z));

	NxD6JointDesc d6Desc;
	d6Desc.actor[0] = body1;
	d6Desc.actor[1] = body;
	d6Desc.setGlobalAnchor(NxVec3(x ,y ,z));
	d6Desc.setGlobalAxis(NxVec3(nx ,ny ,nz));

	d6Desc.twistMotion = NX_D6JOINT_MOTION_FREE;
	d6Desc.swing1Motion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.swing2Motion = NX_D6JOINT_MOTION_LOCKED;

	d6Desc.xMotion = NX_D6JOINT_MOTION_FREE;
	d6Desc.yMotion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.zMotion = NX_D6JOINT_MOTION_LOCKED;

	NxReal gLinearLimit = 5.0f;
	NxReal gSwing1Limit = NxPiF32 / 180.0f * 30.0f;
	NxReal gSwing2Limit = NxPiF32 / 180.0f * 70.0f;
	NxReal gTwistLowLimit = NxPiF32 / 180.0f * -90.0f;
	NxReal gTwistHighLimit = NxPiF32 / 180.0f * 45.0f;

	d6Desc.linearLimit.value = gLinearLimit;
	d6Desc.swing1Limit.value = gSwing1Limit;
	d6Desc.swing2Limit.value = gSwing2Limit;
	d6Desc.twistLimit.low.value =  gTwistLowLimit;
	d6Desc.twistLimit.high.value = gTwistHighLimit;

//	d6Desc.projectionMode = NX_JPM_POINT_MINDIST;

//	d6Desc.jointFlags |= NX_JF_COLLISION_ENABLED;

	NxRevoluteJointDesc revoluteDesc;

	revoluteDesc.actor[0] = body;
	revoluteDesc.actor[1] = body2;

	revoluteDesc.setGlobalAxis(NxVec3(0,0,1)); //rotate around the x axis.
	revoluteDesc.setGlobalAnchor(NxVec3(x,y,z)); //anchor, point the axis passes through.

	NxJoint* hingeJoint = gScene->createJoint(revoluteDesc);

	NxJoint* d6Joint = gScene->createJoint(d6Desc);

	return body;//(NxD6Joint*)d6Joint->is(NX_JOINT_D6);
}
BBDECL NxActor* BBCALL pxJCreateCarBackJoint(NxActor* body1, NxActor* body2, float x, float y, float z, float nx, float ny, float nz)
{
	NxBodyDesc BodyDesc;
	NxActorDesc *ActorDesc = new NxActorDesc;
	ActorDesc->density		= 1;
	
	NxBoxShapeDesc BoxDesc;
	BoxDesc.dimensions = NxVec3(0, 0, 0);
	BoxDesc.groupsMask.bits0 = 1;

	ActorDesc->shapes.pushBack(&BoxDesc);
	ActorDesc->body = &BodyDesc;

	NxActor* body = gScene->createActor(*ActorDesc);
			body->setMass(1);
			body->setGlobalPosition(NxVec3(x ,y ,z));

	NxD6JointDesc d6Desc;
	d6Desc.actor[0] = body1;
	d6Desc.actor[1] = body;
	d6Desc.setGlobalAnchor(NxVec3(x ,y ,z));
	d6Desc.setGlobalAxis(NxVec3(nx ,ny ,nz));

	d6Desc.twistMotion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.swing1Motion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.swing2Motion = NX_D6JOINT_MOTION_LOCKED;

	d6Desc.xMotion = NX_D6JOINT_MOTION_FREE;
	d6Desc.yMotion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.zMotion = NX_D6JOINT_MOTION_LOCKED;

	NxReal gLinearLimit = 5.0f;
	NxReal gSwing1Limit = NxPiF32 / 180.0f * 30.0f;
	NxReal gSwing2Limit = NxPiF32 / 180.0f * 70.0f;
	NxReal gTwistLowLimit = NxPiF32 / 180.0f * -90.0f;
	NxReal gTwistHighLimit = NxPiF32 / 180.0f * 45.0f;

	d6Desc.linearLimit.value = gLinearLimit;
	d6Desc.swing1Limit.value = gSwing1Limit;
	d6Desc.swing2Limit.value = gSwing2Limit;
	d6Desc.twistLimit.low.value =  gTwistLowLimit;
	d6Desc.twistLimit.high.value = gTwistHighLimit;

	d6Desc.projectionMode = NX_JPM_POINT_MINDIST;

//	d6Desc.jointFlags |= NX_JF_COLLISION_ENABLED;

	NxRevoluteJointDesc revoluteDesc;

	revoluteDesc.actor[0] = body;
	revoluteDesc.actor[1] = body2;

	revoluteDesc.setGlobalAxis(NxVec3(0,0,1)); //rotate around the x axis.
	revoluteDesc.setGlobalAnchor(NxVec3(x,y,z)); //anchor, point the axis passes through.

	NxJoint* hingeJoint = gScene->createJoint(revoluteDesc);

	NxJoint* d6Joint = gScene->createJoint(d6Desc);

	return body;//(NxD6Joint*)d6Joint->is(NX_JOINT_D6);
}
BBDECL NxD6Joint* BBCALL pxJCreateJoint(NxActor* body1, NxActor* body2, float x, float y, float z, float nx, float ny, float nz)
{
	NxD6JointDesc d6Desc;
	d6Desc.actor[0] = body1;
	d6Desc.actor[1] = body2;
	d6Desc.setGlobalAnchor(NxVec3(x ,y ,z));
	d6Desc.setGlobalAxis(NxVec3(nx ,ny ,nz));

	d6Desc.twistMotion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.swing1Motion = NX_D6JOINT_MOTION_FREE;
	d6Desc.swing2Motion = NX_D6JOINT_MOTION_FREE;

	d6Desc.xMotion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.yMotion = NX_D6JOINT_MOTION_LOCKED;
	d6Desc.zMotion = NX_D6JOINT_MOTION_LOCKED;



//	d6Desc.projectionMode = NX_JPM_NONE;
	d6Desc.projectionMode = NX_JPM_POINT_MINDIST;

//	d6Desc.jointFlags |= NX_JF_COLLISION_ENABLED;

	NxJoint* d6Joint = gScene->createJoint(d6Desc);

	return (NxD6Joint*)d6Joint->is(NX_JOINT_D6);
}
//Set Joint Parameter
BBDECL void BBCALL pxJointHingeSetCollision(NxRevoluteJoint* joint)
{
	NxRevoluteJointDesc desc;
	joint->saveToDesc(desc); 

	desc.jointFlags |= NX_JF_COLLISION_ENABLED;

	joint->loadFromDesc(desc);
}

BBDECL void BBCALL pxJointHingeSetLimit(NxRevoluteJoint* joint, float min, float max)
{
	NxJointLimitPairDesc limitDesc;
	limitDesc.high.value = max*NxPi/180;
	limitDesc.low.value = min*NxPi/180;
	joint->setLimits(limitDesc);

}
BBDECL void BBCALL pxJointHingeSetSpring(NxRevoluteJoint* joint, float spr, float targetVal)
{
	NxRevoluteJointDesc desc;
	joint->saveToDesc(desc); 

	desc.flags |= NX_RJF_SPRING_ENABLED;
	NxSpringDesc ss;
	if (spr != 0) ss.spring = spr;
	ss.targetValue = targetVal;
	desc.spring = ss;

	joint->loadFromDesc(desc);
}

BBDECL void BBCALL pxJointSphericalSetCollision(NxSphericalJoint* joint)
{
	NxSphericalJointDesc desc;
	joint->saveToDesc(desc); 

	desc.jointFlags |= NX_JF_COLLISION_ENABLED;

	joint->loadFromDesc(desc);
}

BBDECL void BBCALL pxJointSphericalSetLimitAngle(NxSphericalJoint* joint, float angle, float hardn, float restit)
{
	NxSphericalJointDesc desc;
	joint->saveToDesc(desc) ; 

	desc.flags |= NX_SJF_SWING_LIMIT_ENABLED;
	desc.swingLimit.value = (NxReal)angle*NxPi/180;
	if (hardn != 0) desc.swingLimit.hardness = hardn;
	if (restit != 0) desc.swingLimit.restitution = restit;

	/*desc.flags |= NX_SJF_SWING_LIMIT_ENABLED;
	desc.swingLimit.value = (NxReal)angle*NxPi/180;
	desc.swingLimit.hardness = hardn;
	desc.swingLimit.restitution = restit;*/

	joint->loadFromDesc(desc);
}

BBDECL void BBCALL pxJointSphericalSetLimitTwist(NxSphericalJoint* joint, float mintwist, float maxtwist, float spr, float damp, float targetVal)
{
	NxSphericalJointDesc desc;
	joint->saveToDesc(desc) ; 
	
	desc.flags |= NX_SJF_TWIST_LIMIT_ENABLED;
	desc.twistLimit.low.value = (NxReal)mintwist*NxPi/180;
	desc.twistLimit.low.hardness = 0.5;
	desc.twistLimit.low.restitution = 0.5;
	desc.twistLimit.high.value = (NxReal)maxtwist*NxPi/180;
	desc.twistLimit.high.hardness = 0.5;
	desc.twistLimit.high.restitution = 0.5;

	desc.flags |= NX_SJF_TWIST_SPRING_ENABLED;
	NxSpringDesc ss;
	if (spr != 0) desc.twistSpring.spring = spr;
	if (damp != 0) desc.twistSpring.damper = damp;
	desc.twistSpring.targetValue = targetVal;
	desc.twistSpring = ss;

	/*desc.flags |= NX_SJF_TWIST_LIMIT_ENABLED;
	desc.twistLimit.low.value = mintwist*NxPi/180;
	desc.twistLimit.high.value = maxtwist*NxPi/180;

	desc.flags |= NX_SJF_TWIST_SPRING_ENABLED;
	desc.twistSpring.spring = spr;
	desc.twistSpring.damper = damp;*/

	joint->loadFromDesc(desc);
}

BBDECL void BBCALL pxJointSphericalSetLimitSpring(NxSphericalJoint* joint, float spr, float damp, float targetVal)
{
	NxSphericalJointDesc desc;
	joint->saveToDesc(desc); 

	desc.flags |= NX_SJF_SWING_SPRING_ENABLED;
	NxSpringDesc ss;
	if (spr != 0) ss.spring = spr;
	if (damp != 0) ss.damper = damp;
	ss.targetValue = targetVal;
	desc.swingSpring = ss;

	joint->loadFromDesc(desc);
}

BBDECL void BBCALL pxJointDistanceSetPoint(NxDistanceJoint* joint, float mindist, float maxdist)
{
	NxDistanceJointDesc desc;
	joint->saveToDesc(desc) ; 

	desc.minDistance = mindist;  
	desc.maxDistance = maxdist;
//	desc.flags=NX_DJF_MIN_DISTANCE_ENABLED | NX_DJF_MAX_DISTANCE_ENABLED;

	joint->loadFromDesc(desc);
}
BBDECL void BBCALL pxJointDistanceSetSpring(NxDistanceJoint* joint, float spr, float damp)
{
	NxDistanceJointDesc desc;
	joint->saveToDesc(desc) ; 

	desc.spring.damper = damp;
	desc.spring.spring = spr;
	desc.flags |= NX_DJF_SPRING_ENABLED;

	joint->loadFromDesc(desc);
}
BBDECL void BBCALL pxJointDistanceSetCollision(NxDistanceJoint* joint)
{
	NxDistanceJointDesc desc;
	joint->saveToDesc(desc); 

	desc.jointFlags |= NX_JF_COLLISION_ENABLED;

	joint->loadFromDesc(desc);
}
BBDECL void BBCALL pxJointCylindricalSetCollision(NxCylindricalJoint* joint)
{
	NxCylindricalJointDesc desc;
	joint->saveToDesc(desc); 

	desc.jointFlags |= NX_JF_COLLISION_ENABLED;

	joint->loadFromDesc(desc);
}
BBDECL void BBCALL pxJointAddLimitPlane(NxJoint* joint, float x, float y, float z, float nx, float ny, float nz)
{

	joint->addLimitPlane(NxVec3(nx,ny,nz),NxVec3(x,y,z));

}

BBDECL void BBCALL pxJointPulleySetAnchor(NxPulleyJoint* joint, float x1,float y1, float z1, float x2,float y2, float z2)
{
	NxPulleyJointDesc desc;
	NxVec3 pulley0(x1,y1,z1);
	NxVec3 pulley1(x2,y2,z2);

	joint->saveToDesc(desc) ; 

		desc.pulley[0] = pulley0;     
		desc.pulley[1] = pulley1;     

	joint->loadFromDesc(desc);
}
BBDECL void BBCALL pxJointPulleySetLocalAttachBody(NxPulleyJoint* joint, float x1,float y1, float z1, float x2,float y2, float z2)
{
	NxPulleyJointDesc desc;

	joint->saveToDesc(desc) ; 

	desc.localAnchor[0] = NxVec3(x1,y1,z1); 
	desc.localAnchor[1] = NxVec3(x2,y2,z2);   

	joint->loadFromDesc(desc);
}
BBDECL void BBCALL pxJointHingeSetMotor(NxRevoluteJoint* joint, float force, float velTarget)
{
	NxRevoluteJointDesc desc;
	joint->saveToDesc(desc) ; 

	desc.motor.maxForce = force;
	desc.motor.velTarget = velTarget; 
	desc.flags |= NX_RJF_MOTOR_ENABLED;

	joint->loadFromDesc(desc);

}

BBDECL void BBCALL pxJointSetBreakable(NxJoint* joint, float force, float torque)
{
	joint->setBreakable(force, torque);
}
BBDECL int BBCALL pxJointIsBroken(NxJoint* joint)
{
	NxJointState state = joint->getState();  
	if (state & NX_JS_BROKEN) return 1;
	else return 0;
}
//Set Objekt Parameter
BBDECL void BBCALL pxBodySetMass(NxActor* body, float mass)
{
	body->setMass(mass);
	body->updateMassFromShapes(0, mass);
}
BBDECL void BBCALL pxBodySetCMassGlobalPosition(NxActor* body, float x, float y, float z)
{
		body->setCMassOffsetGlobalPosition(NxVec3(x,y,z));
}

BBDECL void BBCALL pxBodySetCMassLocalPosition(NxActor* body, float x, float y, float z)
{
		body->setCMassOffsetLocalPosition(NxVec3(x,y,z));
}

BBDECL void BBCALL pxBodySetMassSpaceInertiaTensor(NxActor* body, float x, float y, float z)
{
		body->setMassSpaceInertiaTensor(NxVec3(x,y,z)); 
}

BBDECL void BBCALL pxBodySetMyForce(NxActor* body, float lx, float ly, float lz)
{
	NxVec3 speed = NxVec3(lx,ly,lz);
	NxQuat quat = body->getGlobalOrientationQuat(); 
	quat=-quat;
	quat.rotate(speed);
	speed.y = ly;
	body->addForce(speed,NX_IMPULSE );
}
BBDECL void BBCALL pxBodyAddForce(NxActor* body, float vx, float vy, float vz, int mode)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->StaticBody == 0)
	switch(mode)
	{
	    case 0: body->addForce(NxVec3(vx,vy,vz),NX_FORCE );
		break;
		case 1: body->addForce(NxVec3(vx,vy,vz),NX_IMPULSE );
		break;
		case 2: body->addForce(NxVec3(vx,vy,vz),NX_VELOCITY_CHANGE );
		break; 
		case 3: body->addForce(NxVec3(vx,vy,vz),NX_SMOOTH_IMPULSE );
		break; 
		case 4: body->addForce(NxVec3(vx,vy,vz),NX_SMOOTH_VELOCITY_CHANGE );
		break; 
		case 5: body->addForce(NxVec3(vx,vy,vz),NX_ACCELERATION );
		break;
	}
}
BBDECL void BBCALL pxBodyAddForceAtPos(NxActor* body, float vx, float vy, float vz, float px, float py, float pz, int mode)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->StaticBody == 0)
	switch(mode)
	{
	    case 0: body->addForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_FORCE );
		break;
		case 1: body->addForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_IMPULSE );
		break;
		case 2: body->addForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_VELOCITY_CHANGE );
		break; 
		case 3: body->addForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_SMOOTH_IMPULSE );
		break; 
		case 4: body->addForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_SMOOTH_VELOCITY_CHANGE );
		break; 
		case 5: body->addForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_ACCELERATION );
		break;
	}
}
BBDECL void BBCALL pxBodyAddForceAtLocalPos(NxActor* body, float vx, float vy, float vz, float px, float py, float pz, int mode)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->StaticBody == 0)
	switch(mode)
	{
	    case 0: body->addForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_FORCE );
		break;
		case 1: body->addForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_IMPULSE );
		break;
		case 2: body->addForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_VELOCITY_CHANGE );
		break; 
		case 3: body->addForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_SMOOTH_IMPULSE );
		break; 
		case 4: body->addForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_SMOOTH_VELOCITY_CHANGE );
		break; 
		case 5: body->addForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_ACCELERATION );
		break;
	}
}
BBDECL void BBCALL pxBodyAddLocalForce(NxActor* body, float vx, float vy, float vz, int mode)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->StaticBody == 0)
	switch(mode)
	{
	    case 0: body->addLocalForce(NxVec3(vx,vy,vz),NX_FORCE );
		break;
		case 1: body->addLocalForce(NxVec3(vx,vy,vz),NX_IMPULSE );
		break;
		case 2: body->addLocalForce(NxVec3(vx,vy,vz),NX_VELOCITY_CHANGE );
		break; 
		case 3: body->addLocalForce(NxVec3(vx,vy,vz),NX_SMOOTH_IMPULSE );
		break; 
		case 4: body->addLocalForce(NxVec3(vx,vy,vz),NX_SMOOTH_VELOCITY_CHANGE );
		break; 
		case 5: body->addLocalForce(NxVec3(vx,vy,vz),NX_ACCELERATION );
		break;
	}
}
BBDECL void BBCALL pxBodyAddLocalForceAtPos(NxActor* body, float vx, float vy, float vz, float px, float py, float pz, int mode)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->StaticBody == 0)
	switch(mode)
	{
	    case 0: body->addLocalForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_FORCE );
		break;
		case 1: body->addLocalForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_IMPULSE );
		break;
		case 2: body->addLocalForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_VELOCITY_CHANGE );
		break; 
		case 3: body->addLocalForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_SMOOTH_IMPULSE );
		break; 
		case 4: body->addLocalForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_SMOOTH_VELOCITY_CHANGE );
		break; 
		case 5: body->addLocalForceAtPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_ACCELERATION );
		break;
	}
}
BBDECL void BBCALL pxBodyAddLocalForceAtLocalPos(NxActor* body, float vx, float vy, float vz, float px, float py, float pz, int mode)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->StaticBody == 0)
	switch(mode)
	{
	    case 0: body->addLocalForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_FORCE );
		break;
		case 1: body->addLocalForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_IMPULSE );
		break;
		case 2: body->addLocalForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_VELOCITY_CHANGE );
		break; 
		case 3: body->addLocalForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_SMOOTH_IMPULSE );
		break; 
		case 4: body->addLocalForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_SMOOTH_VELOCITY_CHANGE );
		break; 
		case 5: body->addLocalForceAtLocalPos(NxVec3(vx,vy,vz), NxVec3(px,py,pz),NX_ACCELERATION );
		break;
	}
}
BBDECL void BBCALL pxBodyAddTorque(NxActor* body, float vx, float vy, float vz, int mode)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->StaticBody == 0)
	switch(mode)
	{
	    case 0: body->addTorque(NxVec3(vx,vy,vz),NX_FORCE );
		break;
		case 1: body->addTorque(NxVec3(vx,vy,vz),NX_IMPULSE );
		break;
		case 2: body->addTorque(NxVec3(vx,vy,vz),NX_VELOCITY_CHANGE );
		break; 
		case 3: body->addTorque(NxVec3(vx,vy,vz),NX_SMOOTH_IMPULSE );
		break; 
		case 4: body->addTorque(NxVec3(vx,vy,vz),NX_SMOOTH_VELOCITY_CHANGE );
		break; 
		case 5: body->addTorque(NxVec3(vx,vy,vz),NX_ACCELERATION );
		break;
	}
}
BBDECL void BBCALL pxBodyAddLocalTorque(NxActor* body, float vx, float vy, float vz, int mode)
{
	BodyData* data = (BodyData*)body->userData;
	if (data->StaticBody == 0)
	switch(mode)
	{
	    case 0: body->addLocalTorque(NxVec3(vx,vy,vz),NX_FORCE );
		break;
		case 1: body->addLocalTorque(NxVec3(vx,vy,vz),NX_IMPULSE );
		break;
		case 2: body->addLocalTorque(NxVec3(vx,vy,vz),NX_VELOCITY_CHANGE );
		break; 
		case 3: body->addLocalTorque(NxVec3(vx,vy,vz),NX_SMOOTH_IMPULSE );
		break; 
		case 4: body->addLocalTorque(NxVec3(vx,vy,vz),NX_SMOOTH_VELOCITY_CHANGE );
		break; 
		case 5: body->addLocalTorque(NxVec3(vx,vy,vz),NX_ACCELERATION );
		break;
	}
}
BBDECL void BBCALL pxBodySetAngularSpeed(NxActor* body, float ax, float ay, float az)
{
	body->setAngularVelocity(NxVec3(ax,ay,az));
}
BBDECL void BBCALL pxBodySetLinearSpeed(NxActor* body, float lx, float ly, float lz)
{
	body->setLinearVelocity(NxVec3(lx,ly,lz));
}
BBDECL void BBCALL pxBodySetLocalLinearSpeed(NxActor* body, float lx, float ly, float lz)
{
	NxVec3 speed = NxVec3(lx,ly,lz);
	NxQuat quat = body->getGlobalOrientationQuat(); 
	quat=-quat;
	quat.rotate(speed);
	body->setLinearVelocity(speed);
}
BBDECL void BBCALL pxBodySetLocalAngularSpeed(NxActor* body, float ax, float ay, float az)
{
	NxVec3 speed = NxVec3(ax,ay,az);
	NxQuat quat = body->getGlobalOrientationQuat(); 
	quat=-quat;
	quat.rotate(speed);
	body->setAngularVelocity(speed);
}
BBDECL void BBCALL pxBodySetAngularDamping(NxActor* body, float angDamp)
{
	body->setAngularDamping(angDamp);
}
BBDECL void BBCALL pxBodySetLinearDamping(NxActor* body, float linDamp)
{
	body->setLinearDamping(linDamp);
}
BBDECL void BBCALL pxBodySetAngularMomentum(NxActor* body, float ax, float ay, float az)
{
	body->setAngularMomentum(NxVec3(ax,ay,az));
}
BBDECL void BBCALL pxBodySetLinearMomentum(NxActor* body, float lx, float ly, float lz)
{
	body->setLinearMomentum(NxVec3(lx,ly,lz));
}
BBDECL void BBCALL pxBodySetMaxAngularSpeed(NxActor* body, float speed)
{
	body->setMaxAngularVelocity(speed);
}
BBDECL void BBCALL pxBodySetSleepAngularVelocity(NxActor* body, float tres)
{
//	gPhysicsSDK->setParameter(NX_DEFAULT_SLEEP_ANG_VEL_SQUARED,0.14*0.14);
	body->setSleepAngularVelocity(tres);
}
BBDECL void BBCALL pxBodySetSleepEnergyThreshold(NxActor* body, float tres)
{
	//gPhysicsSDK->setParameter(NX_DEFAULT_SLEEP_ENERGY,0.005);
	body->setSleepEnergyThreshold(tres);
}
BBDECL void BBCALL pxBodySetSleepLinearVelocity(NxActor* body, float tres)
{
//	gPhysicsSDK->setParameter(NX_DEFAULT_SLEEP_LIN_VEL_SQUARED,0.15*0.15);
	body->setSleepLinearVelocity(tres);
}
BBDECL void BBCALL pxBodySetSleepWakeUp(NxActor* body, float wakeCounterValue)
{
	body->wakeUp(wakeCounterValue);
}
BBDECL void BBCALL pxBodySetSleepPut(NxActor* body)
{
	body->putToSleep();
}
BBDECL void BBCALL pxBodySetSolverIterationCount(NxActor* body, int itercount)
{
	body->setSolverIterationCount(itercount);
}
BBDECL void BBCALL pxBodySetBodyName(NxActor* body, const char* name)
{
	body->setName(name);
}
BBDECL void BBCALL pxBodySetBodyEntity(NxActor* body, float* entity)
{
	BodyData* data = (BodyData*)body->userData;
	data->Entity = entity;
}
BBDECL void BBCALL pxBodySetBodyUserData(NxActor* body, int userdata)
{
	BodyData* data = (BodyData*)body->userData;
	data->BlitzData = userdata;
}
BBDECL void BBCALL pxMoveBodyToPoint(NxActor* body, float maxspeed, float x, float y, float z)
{
	NxVec3 pos = body->getGlobalPosition();
	float distance = sqrtf((x-pos.x)*(x-pos.x) + (y-pos.y)*(y-pos.y) + (z-pos.z)*(z-pos.z));
	float konst = maxspeed/distance;
	body->setLinearVelocity(NxVec3((x-pos.x)*konst, (y-pos.y)*konst, (z-pos.z)*konst));
}
NxBoxShapeDesc* DrawBox(NxShape* box)
{
	NxMat34 pose = box->getGlobalPose();

	NxBoxShapeDesc * BoxDesc = new NxBoxShapeDesc;
	BoxDesc->dimensions = NxVec3(2, 2, 2);
	BoxDesc->localPose.t.set(0, -5, 0);
	//box->isBox()->saveToDesc(*BoxDesc);
	return BoxDesc;
	//body1->createShape(*BoxDesc);

	//NxVec3 boxDim = box->isBox()->getDimensions();
}
BBDECL void BBCALL pxAddBodytoBody(NxActor* body1, NxActor* body2)
{
	NxShape*const* shapes = body2->getShapes();
	NxU32 nShapes = body2->getNbShapes();
	while (nShapes--)
	{
		switch(shapes[nShapes]->getType())
		{
		    case NX_SHAPE_BOX:
			{	
				NxBoxShapeDesc *BoxDesc = new NxBoxShapeDesc;
				NxMat34 pose_sh = shapes[nShapes]->getGlobalPose();
				shapes[nShapes]->isBox()->saveToDesc(*BoxDesc);
				NxMat34 pose_bd = body1->getGlobalPose();
				BoxDesc->localPose.t.set(pose_sh.t - pose_bd.t);
				BoxDesc->localPose.M = pose_sh.M;
				body1->createShape(*BoxDesc);
				float mass = body1->getMass() + body2->getMass();
				body1->updateMassFromShapes(0,mass); 
			}
			break;
		    case NX_SHAPE_SPHERE:
			{	
				NxSphereShapeDesc *BoxDesc = new NxSphereShapeDesc;
				NxMat34 pose_sh = shapes[nShapes]->getGlobalPose();
				shapes[nShapes]->isSphere()->saveToDesc(*BoxDesc);
				NxMat34 pose_bd = body1->getGlobalPose();
				BoxDesc->localPose.t.set(pose_sh.t - pose_bd.t);
				BoxDesc->localPose.M = pose_sh.M;
				body1->createShape(*BoxDesc);
				float mass = body1->getMass() + body2->getMass();
				body1->updateMassFromShapes(0,mass); 
			}
			break;
		    case NX_SHAPE_CAPSULE:
			{	
				NxCapsuleShapeDesc *BoxDesc = new NxCapsuleShapeDesc;
				NxMat34 pose_sh = shapes[nShapes]->getGlobalPose();
				shapes[nShapes]->isCapsule()->saveToDesc(*BoxDesc);
				NxMat34 pose_bd = body1->getGlobalPose();
				BoxDesc->localPose.t.set(pose_sh.t - pose_bd.t);
				BoxDesc->localPose.M = pose_sh.M;
				body1->createShape(*BoxDesc);
				float mass = body1->getMass() + body2->getMass();
				body1->updateMassFromShapes(0,mass); 
			}
			break;
		    case NX_SHAPE_CONVEX:
			{	
				NxConvexShapeDesc *BoxDesc = new NxConvexShapeDesc;
				NxMat34 pose_sh = shapes[nShapes]->getGlobalPose();
				shapes[nShapes]->isConvexMesh()->saveToDesc(*BoxDesc);
				NxMat34 pose_bd = body1->getGlobalPose();
				BoxDesc->localPose.t.set(pose_sh.t - pose_bd.t);
				BoxDesc->localPose.M = pose_sh.M;
				body1->createShape(*BoxDesc);
				float mass = body1->getMass() + body2->getMass();
				body1->updateMassFromShapes(0,mass); 
			}
			break;		
		    case NX_SHAPE_MESH:
			{	
				NxTriangleMeshShapeDesc *BoxDesc = new NxTriangleMeshShapeDesc;
				NxMat34 pose_sh = shapes[nShapes]->getGlobalPose();
				shapes[nShapes]->isTriangleMesh()->saveToDesc(*BoxDesc);
				NxMat34 pose_bd = body1->getGlobalPose();
				BoxDesc->localPose.t.set(pose_sh.t - pose_bd.t);
				BoxDesc->localPose.M = pose_sh.M;
				body1->createShape(*BoxDesc);
				float mass = body1->getMass() + body2->getMass();
				body1->updateMassFromShapes(0,mass); 
			}
			break;	
		}
	}	
	if(body2)
	{
		gScene->releaseActor(*body2);
		body2 = NULL;
	}
	//body1
}
BBDECL NxActor* BBCALL pxCopyBody(NxActor* body)
{
	NxBoxShapeDesc BoxDesc1;
	//BoxDesc1.dimensions = NxVec3(0, 0, 0);

	NxActorDesc ActorDesc;
	ActorDesc.shapes.pushBack(&BoxDesc1);
		
	if (body->getMass())
	{
		NxBodyDesc BodyDesc;
		BodyDesc.mass = body->getMass();
		ActorDesc.body			= &BodyDesc;
	}

	NxActor* body1 = gScene->createActor(ActorDesc);

	NxShape*const* shapes = body->getShapes();
	NxU32 nShapes = body->getNbShapes();
	while (nShapes--)
	{
		switch(shapes[nShapes]->getType())
		{
		    case NX_SHAPE_BOX:
			{	
				NxBoxShapeDesc *BoxDesc = new NxBoxShapeDesc;
				shapes[nShapes]->isBox()->saveToDesc(*BoxDesc);
				body1->createShape(*BoxDesc);
			}
			break;
		    case NX_SHAPE_SPHERE:
			{	
				NxSphereShapeDesc *BoxDesc = new NxSphereShapeDesc;
				shapes[nShapes]->isSphere()->saveToDesc(*BoxDesc);
				body1->createShape(*BoxDesc);
			}
			break;
		    case NX_SHAPE_CAPSULE:
			{	
				NxCapsuleShapeDesc *BoxDesc = new NxCapsuleShapeDesc;
				shapes[nShapes]->isCapsule()->saveToDesc(*BoxDesc);
				body1->createShape(*BoxDesc);
			}
			break;
		    case NX_SHAPE_CONVEX:
			{	
				NxConvexShapeDesc *BoxDesc = new NxConvexShapeDesc;
				shapes[nShapes]->isConvexMesh()->saveToDesc(*BoxDesc);
				body1->createShape(*BoxDesc);
			}
			break;		
		    case NX_SHAPE_MESH:
			{	
				NxTriangleMeshShapeDesc *BoxDesc = new NxTriangleMeshShapeDesc;
				shapes[nShapes]->isTriangleMesh()->saveToDesc(*BoxDesc);
				body1->createShape(*BoxDesc);
			}
			break;	
		}
	}	
	NxShape* shape = (NxShape*)body1->getShapes()[0];
	body1->releaseShape(*shape);
	mainSetMask(body1);
	BodyAddUserData(body1);
	//body1->updateMassFromShapes(0,body->getMass()); 

	return body1;
}
//Get Objekt Parameter
BBDECL float BBCALL pxBodyGetAngularDamping(NxActor* body)
{
	return body->getAngularDamping();
}
BBDECL float BBCALL pxBodyGetLinearDamping(NxActor* body)
{
	return body->getLinearDamping();
}
BBDECL float BBCALL pxBodyGetAngularMomentum(NxActor* body)
{
	NxVec3 moment = body->getAngularMomentum();
	return moment.magnitude();
}
BBDECL float BBCALL pxBodyGetLinearMomentum(NxActor* body)
{
	NxVec3 moment = body->getLinearMomentum();
	return sqrtf(moment.x*moment.x+moment.y*moment.y+moment.z*moment.z);
}
BBDECL float BBCALL pxBodyGetCMassGlobalPositionX(NxActor* body)
{
	return body->getCMassGlobalPosition().x;
	
}
BBDECL float BBCALL pxBodyGetCMassGlobalPositionY(NxActor* body)
{
	return body->getCMassGlobalPosition().y;
	
}
BBDECL float BBCALL pxBodyGetCMassGlobalPositionZ(NxActor* body)
{
	return body->getCMassGlobalPosition().z;
	
}
BBDECL float BBCALL pxBodyGetCMassLocalPositionX(NxActor* body)
{
	return body->getCMassLocalPosition().x;
}
BBDECL float BBCALL pxBodyGetCMassLocalPositionY(NxActor* body)
{
	return body->getCMassLocalPosition().y;
}
BBDECL float BBCALL pxBodyGetCMassLocalPositionZ(NxActor* body)
{
	return body->getCMassLocalPosition().z;
}
BBDECL float BBCALL pxBodyGetMass(NxActor* body)
{
	return body->getMass();
}
BBDECL float BBCALL pxBodyGetMaxAngularVelocity(NxActor* body)
{
	return body->getMaxAngularVelocity();
}
BBDECL int BBCALL pxBodyGetNbShapes(NxActor* body)
{
	return body->getNbShapes();
}
BBDECL float BBCALL pxBodyGetSleepAngularVelocity(NxActor* body)
{
	return body->getSleepAngularVelocity();
}
BBDECL float BBCALL pxBodyGetSleepLinearVelocity(NxActor* body)
{
	return body->getSleepLinearVelocity();
}
BBDECL int BBCALL pxBodyIsDynamic(NxActor* body)
{
	return body->isDynamic();
}
BBDECL int BBCALL pxBodyIsSleeping(NxActor* body)
{
	return body->isSleeping();
}
BBDECL const char* BBCALL pxGetBodyName(NxActor* body)
{
	return body->getName();
}
BBDECL float* BBCALL pxGetBodyEntity(NxActor* body)
{
	BodyData* data = (BodyData*)body->userData;
	return data->Entity;
}
BBDECL int BBCALL pxGetBodyUserData(NxActor* body)
{	
	if (FAILED(body->userData)) return 0 ;
	BodyData* data = (BodyData*)body->userData;
	return data->BlitzData;
}
// deltaTime
float UpdateTime()
{
	static unsigned int previousTime = timeGetTime();
	unsigned int currentTime = timeGetTime();
	unsigned int elapsedTime = currentTime - previousTime;
	previousTime = currentTime;
	return (float)(elapsedTime)*0.001f;
}
__inline unsigned __int64 Hash(unsigned char* str, unsigned int len)
{
   unsigned __int64 hash = 0x6bdef34aa12fff01;
   unsigned int i    = 0;

   for(i = 0; i < len; str++, i++)
   {
      hash ^= ((i & 1) == 0) ? (  (hash <<  7) ^ (*str) ^ (hash >> 3)) :
                               (~((hash << 11) ^ (*str) ^ (hash >> 5)));
   }

   return (hash & 0x7e7e7e7e7e7e7e7e);
}

int mstrcmp(char* s1, char* s2)
{
	 for(int i = 0; s1[i] && s2[i]; i++)
	 {
		 if (s1[i] < s2[i]) return -1;
		 if (s1[i] > s2[i]) return 1;
	 }
//	if (s1[i] < s2[i]) return -1;
//	if (s1[i] > s2[i]) return 1;
	return 0;
}
int scanLine (char* str)
{
// ******************************************************
// *													*
// *				XorsTeam comments					*
// *													*
// *  Originally this function checked the license key  *
// *													*
// ******************************************************
	return 0;
}

	typedef int (__cdecl *ffd)(int,UserAllocator*,ErrorStream*,const NxPhysicsSDKDesc &  desc, int* ); 
	ffd df;
// Create World
void ReleaseNx()
{
	if (gScene) gPhysicsSDK->releaseScene(*gScene);
	NxCloseCooking();
	gScene = 0;
	if (gPhysicsSDK)  gPhysicsSDK->release();
}
BBDECL NxScene* BBCALL pxCreateWorldWithoutDrivers(int plane, char* password)
{
	//ReleaseNx();
	bah = scanLine(password);
	gMyAllocator = new UserAllocator;
	gErrorStream = new ErrorStream;
	int con = NX_PHYSICS_SDK_VERSION;
	NxPhysicsSDKDesc desc;
	NxSDKCreateError errorCode = NXCE_NO_ERROR;

/*	#if defined(PX_DRIVER_SET)
		HMODULE sd=LoadLibrary((LPCTSTR)"PhysXCore.dll");
		df=(ffd)GetProcAddress(sd,"NpCreatePhysicsSDK");
 
		NxGetUtilLib=(fUtilLib)GetProcAddress(sd,"NpGetUtilLib");
		NxGetPhysicsSDKAllocator=(fAllocator)GetProcAddress(sd,"NpGetPhysicsSDKAllocator");
		gPhysicsSDK=(NxPhysicsSDK*)(df(NX_PHYSICS_SDK_VERSION, (UserAllocator*)gMyAllocator, (ErrorStream*)gErrorStream, NxPhysicsSDKDesc(), &con));
	#else*/
		gPhysicsSDK = NxCreatePhysicsSDK(NX_PHYSICS_SDK_VERSION, NULL, gErrorStream,desc,&errorCode);
//	#endif
	if(!gPhysicsSDK) ShowError("Drivers not found!!!!", "pxCreateWorld", __FILE__ , __LINE__);
	if(scanLine(password)) cubici = 600;
	if(scanLine(password)) scubici = 600;
	#ifndef NX_DISABLE_HARDWARE
	NxHWVersion hwCheck = gPhysicsSDK->getHWVersion();
	if (hwCheck == NX_HW_VERSION_NONE) 
	{
		gHardwareSimulation = false;
	}
	#endif
	gPhysicsSDK->setParameter(NX_CONTINUOUS_CD, 1);
	gPhysicsSDK->setParameter(NX_CCD_EPSILON, 0.01f);
//	gPhysicsSDK->setParameter(NX_SKIN_WIDTH, 0.005f);

	// Create a scene
	bool status = InitCooking();
	if (!status) {
		ShowError("InitCooking false", "pxCreateWorld", __FILE__ , __LINE__);
	}
	NxInitCooking();
	NxSceneDesc sceneDesc;
	sceneDesc.gravity				= gDefaultGravity;
	sceneDesc.userContactReport		= &gMyContactReport;
	if (gHardwareSimulation)
		sceneDesc.simType = NX_SIMULATION_HW;
	gScene = gPhysicsSDK->createScene(sceneDesc);

	gScene->setActorGroupPairFlags(0,0,NX_NOTIFY_ON_START_TOUCH|NX_NOTIFY_ON_TOUCH|NX_NOTIFY_ON_END_TOUCH|NX_NOTIFY_FORCES);
	if (!gScene) ShowError("Scene don't creat!!!!", "pxCreateWorld", __FILE__ , __LINE__);
	NxMaterial * defaultMaterial = gScene->getMaterialFromIndex(0); 
	defaultMaterial->setRestitution(0.0f);
	defaultMaterial->setStaticFriction(0.5f);
	defaultMaterial->setDynamicFriction(0.5f);
	gScene->setFilterOps(NX_FILTEROP_OR, NX_FILTEROP_OR, NX_FILTEROP_AND);
	gScene->setFilterBool(true);
	NxGroupsMask zeroMask;
	zeroMask.bits0 = zeroMask.bits1 = zeroMask.bits2 = zeroMask.bits3 = 0;
	gScene->setFilterConstant0(zeroMask);
	gScene->setFilterConstant1(zeroMask);
	if (!gPhysicsSDK) ShowError("Drivers not found!!!!", "pxCreateWorld", __FILE__ , __LINE__);

	// Create ground plane
	if (plane ==1)
	{
		NxPlaneShapeDesc PlaneDesc;
		PlaneDesc.groupsMask.bits0 = 1;//0xffffffff;
		PlaneDesc.groupsMask.bits1 = 1;//0xffffffff;
		PlaneDesc.groupsMask.bits2 = 1;//0xffffffff;
		PlaneDesc.groupsMask.bits3 = 1;//0xffffffff;
		NxActorDesc ActorDesc;
		//PlaneDesc.shapeFlags |= NX_SF_POINT_CONTACT_FORCE;
		ActorDesc.shapes.pushBack(&PlaneDesc);
		global_plane = gScene->createActor(ActorDesc);
		BodyData* data = new BodyData;
		data->contacttable = 1;
		global_plane->userData = data;
	//	return global_plane;
		if (!global_plane) ShowError("global Plane don't creat!!!!", "pxCreateWorld", __FILE__ , __LINE__);

	}
	return gScene;
}
BBDECL NxScene* BBCALL pxCreateWorld(int plane, char* password)
{
	//ReleaseNx();
	bah = scanLine(password);
	std::cout << "1" << std::endl;
	gMyAllocator = new UserAllocator;
	gErrorStream = new ErrorStream;
	int con = NX_PHYSICS_SDK_VERSION;
	std::cout << "1" << std::endl;
	NxPhysicsSDKDesc desc;
	NxSDKCreateError errorCode = NXCE_NO_ERROR;


		HMODULE sd=LoadLibrary((LPCTSTR)"PhysXCore.dll");
		df=(ffd)GetProcAddress(sd,"NpCreatePhysicsSDK");
 
		NxGetUtilLib=(fUtilLib)GetProcAddress(sd,"NpGetUtilLib");
		NxGetPhysicsSDKAllocator=(fAllocator)GetProcAddress(sd,"NpGetPhysicsSDKAllocator");
		gPhysicsSDK=(NxPhysicsSDK*)(df(NX_PHYSICS_SDK_VERSION, (UserAllocator*)gMyAllocator, (ErrorStream*)gErrorStream, NxPhysicsSDKDesc(), &con));
	std::cout << "1" << std::endl;
	//	gPhysicsSDK = NxCreatePhysicsSDK(NX_PHYSICS_SDK_VERSION, NULL, gErrorStream,desc,&errorCode);

	if(!gPhysicsSDK) ShowError("Drivers not found!!!!", "pxCreateWorld", __FILE__ , __LINE__);
	if(scanLine(password)) cubici = 600;
	if(scanLine(password)) scubici = 600;
	
	#ifndef NX_DISABLE_HARDWARE
	NxHWVersion hwCheck = gPhysicsSDK->getHWVersion();
	if (hwCheck == NX_HW_VERSION_NONE) 
	{
		gHardwareSimulation = false;
	}
	#endif
	std::cout << "1" << std::endl;
	gPhysicsSDK->setParameter(NX_CONTINUOUS_CD, 1);
	gPhysicsSDK->setParameter(NX_CCD_EPSILON, 0.01f);
	gPhysicsSDK->getFoundationSDK().getRemoteDebugger()->connect ("localhost", 5425);
	std::cout << "1" << std::endl;
//	gPhysicsSDK->setParameter(NX_SKIN_WIDTH, 0.005f);

	// Create a scene
	bool status = InitCooking();
	std::cout << "1" << std::endl;
	if (!status) {
		ShowError("InitCooking false", "pxCreateWorld", __FILE__ , __LINE__);
	}
	NxInitCooking();
	std::cout << "1" << std::endl;
	NxSceneDesc sceneDesc;
	sceneDesc.gravity				= gDefaultGravity;
	sceneDesc.userContactReport		= &gMyContactReport;
	if (gHardwareSimulation)
		sceneDesc.simType = NX_SIMULATION_HW;
	gScene = gPhysicsSDK->createScene(sceneDesc);
	std::cout << "1" << std::endl;

	gScene->setActorGroupPairFlags(0,0,NX_NOTIFY_ON_START_TOUCH|NX_NOTIFY_ON_TOUCH|NX_NOTIFY_ON_END_TOUCH|NX_NOTIFY_FORCES);
	if (!gScene) ShowError("Scene don't creat!!!!", "pxCreateWorld", __FILE__ , __LINE__);
	std::cout << "1" << std::endl;
	NxMaterial * defaultMaterial = gScene->getMaterialFromIndex(0); 
	defaultMaterial->setRestitution(0.0f);
	defaultMaterial->setStaticFriction(0.5f);
	defaultMaterial->setDynamicFriction(0.5f);
	std::cout << "1" << std::endl;
	gScene->setFilterOps(NX_FILTEROP_OR, NX_FILTEROP_OR, NX_FILTEROP_AND);
	gScene->setFilterBool(true);
	NxGroupsMask zeroMask;
	zeroMask.bits0 = zeroMask.bits1 = zeroMask.bits2 = zeroMask.bits3 = 0;
	gScene->setFilterConstant0(zeroMask);
	gScene->setFilterConstant1(zeroMask);
	std::cout << "1" << std::endl;
	if (!gPhysicsSDK) ShowError("Drivers not found!!!!", "pxCreateWorld", __FILE__ , __LINE__);

	// Create ground plane
	if (plane ==1)
	{
		NxPlaneShapeDesc PlaneDesc;
		PlaneDesc.groupsMask.bits0 = 1;//0xffffffff;
		PlaneDesc.groupsMask.bits1 = 1;//0xffffffff;
		PlaneDesc.groupsMask.bits2 = 1;//0xffffffff;
		PlaneDesc.groupsMask.bits3 = 1;//0xffffffff;
		NxActorDesc ActorDesc;
		//PlaneDesc.shapeFlags |= NX_SF_POINT_CONTACT_FORCE;
		ActorDesc.shapes.pushBack(&PlaneDesc);
		global_plane = gScene->createActor(ActorDesc);
		BodyData* data = new BodyData;
		data->contacttable = 1;
		global_plane->userData = data;
	//	return global_plane;
		if (!global_plane) ShowError("global Plane don't creat!!!!", "pxCreateWorld", __FILE__ , __LINE__);

	}
	std::cout << "1" << std::endl;
	return gScene;
}
BBDECL void BBCALL pxRenderPhysic(float time, int sinc)
{
	// Physics code
	if(gScene && !gPause && bah!=1)
	{
		NxActor** allactors = gScene->getActors();
		int nbAllActors = gScene->getNbActors();
		for (int i=0; i<nbAllActors; i++)
		{
			BodyData* data = (BodyData*)(allactors[i]->userData);
			if (data)
			{
				data->Trigger = NULL;
				data->nContacts = 0; 
			//	data->numTriggerActors = 0;
			}
		}

		if(!sinc)
		{
			gDeltaTime = time; 
			gScene->simulate(1.0f/gDeltaTime);
		}
		else 
		{
			gDeltaTime = time*1.0f/UpdateTime();
			if (gDeltaTime<20) gDeltaTime = 20;
			gScene->simulate(1.0f/gDeltaTime);
		}
		gScene->flushStream();
		gScene->fetchResults(NX_RIGID_BODY_FINISHED, true);
		bah = bah-1;

		/*for (NxWheelShape **mWheel = gWheels.begin(); mWheel != gWheels.end(); mWheel++)
		{
			NxWheelContactData contactData;
			(*mWheel)->getContact(contactData);
			WheelData* data = (WheelData*)(*mWheel)->userData;
			data->contactPos = contactData.contactPoint;
		}*/
	}
}
BBDECL void BBCALL pxSetPause(int pause)
{
	gPause = pause;

}
BBDECL void BBCALL pxSetGravity(float gx, float gy, float gz)
{
	gScene->setGravity(NxVec3(gx, gy, gz));

}
BBDECL void BBCALL pxSetTiming (float maxTimeStep, int maxIter, int StepMethod)
{
	if (StepMethod==0) gScene->setTiming(float(maxTimeStep), maxIter, NX_TIMESTEP_FIXED);
	if (StepMethod==1) gScene->setTiming(float(maxTimeStep), maxIter, NX_TIMESTEP_VARIABLE);
	if (StepMethod==2) gScene->setTiming(float(maxTimeStep), maxIter, NX_TIMESTEP_INHERIT);
}
BBDECL void BBCALL pxSDKSetParameter(float param)
{
	gPhysicsSDK->setParameter(NX_SKIN_WIDTH, param);
}
BBDECL int BBCALL pxGetNumberAllBody()
{
	int nbAllActors = gScene->getNbActors();
	return nbAllActors;
}
BBDECL void BBCALL pxDestroyWorld()
{
	ReleaseNx();
}
BBDECL NxScene* BBCALL pxCreateScene()
{
	NxSceneDesc sceneDesc;
	sceneDesc.gravity				= gDefaultGravity;
	sceneDesc.userContactReport		= &gMyContactReport;
	NxScene* NewScene = gPhysicsSDK->createScene(sceneDesc);

	return NewScene;
}
BBDECL void BBCALL pxSceneChange(NxScene* scene)
{
	gScene = scene;
}
BBDECL int BBCALL pxChekPPU()
{
	#ifndef NX_DISABLE_HARDWARE
	NxHWVersion hwCheck = gPhysicsSDK->getHWVersion();
	if (hwCheck == NX_HW_VERSION_NONE) 
	{
		return 0;
	}
	#endif
	return 1;
}
BBDECL int BBCALL pxChekPPUMode()
{
	return gHardwareSimulation;
}
BBDECL void BBCALL pxSetHardwareSimulation(int mode)
{
	gHardwareSimulation = mode > 0;
}
//SetPosition
BBDECL void BBCALL pxBodySetPosition(NxActor* body, float pos_x, float pos_y, float pos_z)
{
	if (body->isDynamic())
	{
		body->setGlobalPosition(NxVec3(float(pos_x), float(pos_y), float(pos_z)));
	}
	else
	{
		NxVec3 anhorPos = NxVec3(0,0,0);
		NxShape*const* AllShapes = body->getShapes();
		int nbAllShapes = body->getNbShapes();
		for (int i=0; i<nbAllShapes; i++)
		{
			anhorPos = ((ShapeData*)AllShapes[i]->userData)->bpos;
			AllShapes[i]->setGlobalPosition(NxVec3(pos_x, pos_y, pos_z)+anhorPos);
		}
	}
}
BBDECL void BBCALL pxBodySetRotation(NxActor* body, float pitch, float yaw, float roll)
{
	NxQuat qo = NxQuat( NxVec3( 0,NxMath::sin(NxMath::degToRad(yaw*0.5f)),0 ), -NxMath::cos(NxMath::degToRad(yaw*0.5f))) * 
		NxQuat(NxVec3( NxMath::sin(NxMath::degToRad(-pitch*0.5f)),0,0 ), -NxMath::cos(NxMath::degToRad(-pitch*0.5f)) ) * 
		NxQuat( NxVec3( 0,0,NxMath::sin(NxMath::degToRad(-roll*0.5f))), -NxMath::cos(NxMath::degToRad(-roll*0.5f)) );

	if (body->isDynamic())
	{
	body->setGlobalOrientationQuat(qo);
	}
	else
	{
		NxMat33 anhorRot;
		anhorRot.fromQuat(qo);
		NxShape*const* AllShapes = body->getShapes();
		int nbAllShapes = body->getNbShapes();
		for (int i=0; i<nbAllShapes; i++)
		{
			AllShapes[i]->setLocalOrientation(anhorRot);
		}
	}
}
//GetPosition
NxQuat AngleToQuat(float pitch, float yaw, float roll)
{
	NxQuat qo = NxQuat( NxVec3( 0,NxMath::sin(NxMath::degToRad(yaw*0.5f)),0 ), -NxMath::cos(NxMath::degToRad(yaw*0.5f))) * 
		NxQuat(NxVec3( NxMath::sin(NxMath::degToRad(-pitch*0.5f)),0,0 ), -NxMath::cos(NxMath::degToRad(-pitch*0.5f)) ) * 
		NxQuat( NxVec3( 0,0,NxMath::sin(NxMath::degToRad(-roll*0.5f))), -NxMath::cos(NxMath::degToRad(-roll*0.5f)) );

	return qo;
}
BBDECL float BBCALL pxBodyGetPositionX(NxActor* body)
{
	NxVec3 pos;
	if (body->isDynamic())
	{
		pos = body->getGlobalPosition();
	} else 
	{
		NxVec3 anhorPos = NxVec3(0,0,0);
		NxShape*const* AllShapes = body->getShapes();
		anhorPos = ((ShapeData*)AllShapes[0]->userData)->bpos;
		pos = AllShapes[0]->getGlobalPosition() - anhorPos;
	}
	return pos.x;
}
BBDECL float BBCALL pxBodyGetPositionY(NxActor* body)
{
	NxVec3 pos;
	if (body->isDynamic())
	{
		pos = body->getGlobalPosition();
	} else 
	{
		NxVec3 anhorPos = NxVec3(0,0,0);
		NxShape*const* AllShapes = body->getShapes();
		anhorPos = ((ShapeData*)AllShapes[0]->userData)->bpos;
		pos = AllShapes[0]->getGlobalPosition() - anhorPos;
	}
	return pos.y;
}
BBDECL float BBCALL pxBodyGetPositionZ(NxActor* body)
{
	NxVec3 pos;
	if (body->isDynamic())
	{
		pos = body->getGlobalPosition();
	} else 
	{
		NxVec3 anhorPos = NxVec3(0,0,0);
		NxShape*const* AllShapes = body->getShapes();
		anhorPos = ((ShapeData*)AllShapes[0]->userData)->bpos;
		pos = AllShapes[0]->getGlobalPosition() - anhorPos;
	}
	return pos.z;
}
BBDECL float BBCALL pxBodyGetRotationQuatX(NxActor* body)
{
	if (body->isDynamic())
	{
		q = body->getGlobalOrientationQuat();
	} else 
	{
		NxShape*const* AllShapes = body->getShapes();
		AllShapes[0]->getGlobalOrientation().toQuat(q);
	}
	return q.x;
}
BBDECL float BBCALL pxBodyGetRotationQuatY(NxActor* body)
{
	if (body->isDynamic())
	{
		q = body->getGlobalOrientationQuat();
	} else 
	{
		NxShape*const* AllShapes = body->getShapes();
		AllShapes[0]->getGlobalOrientation().toQuat(q);
	}
	return q.y;
}
BBDECL float BBCALL pxBodyGetRotationQuatZ(NxActor* body)
{
	if (body->isDynamic())
	{
		q = body->getGlobalOrientationQuat();
	} else 
	{
		NxShape*const* AllShapes = body->getShapes();
		AllShapes[0]->getGlobalOrientation().toQuat(q);
	}
	return q.z;
}
BBDECL float BBCALL pxBodyGetRotationQuatW(NxActor* body)
{
	if (body->isDynamic())
	{
		q = body->getGlobalOrientationQuat();
	} else 
	{
		NxShape*const* AllShapes = body->getShapes();
		AllShapes[0]->getGlobalOrientation().toQuat(q);
	}
	return q.w;
}
BBDECL float BBCALL pxBodyGetRotationPitch(NxActor* body)
{
	if (body->isDynamic())
	{
		q = body->getGlobalOrientationQuat();
	} else 
	{
		NxShape*const* AllShapes = body->getShapes();
		AllShapes[0]->getGlobalOrientation().toQuat(q);
	}
	q.normalize();
	float xz = q.x * q.z; 
	float wy = -q.w * q.y;
	float yz = q.y * q.z; 
	float wx = -q.w * q.x; 
	float xx = q.x * q.x; 
	float yy = q.y * q.y; 
 
	float kx = 2*(xz - wy); 
	float ky = 2*(yz + wx); 
	float kz = 1 - 2 * (xx +yy); 
 
	float pitch = -NxMath::radToDeg(atan2f ( ky , sqrtf(kx*kx+kz*kz)));//*180/3.14159; 
	
	return pitch;
/*	float sqw = q.w*q.w; 
	float sqx = q.x*q.x; 
	float sqy = q.y*q.y; 
	float sqz = q.z*q.z; 
	return (float) NxMath::radToDeg(atan2f(2.0 * (q.y*q.z + q.x*q.w),(-sqx - sqy + sqz + sqw)));*/
}
BBDECL float BBCALL pxBodyGetRotationYaw(NxActor* body)
{
	NxQuat pq;
	if (body->isDynamic())
	{
		q = body->getGlobalOrientationQuat();
	} else 
	{
		NxShape*const* AllShapes = body->getShapes();
		AllShapes[0]->getGlobalOrientation().toQuat(q);
	}
	float angle = pxBodyGetRotationPitch( body);
	if (angle < 90.01 && angle >89.99)
	{
		NxQuat pq;
		pq.setXYZW(0.0001f,0,0,-1);
		q = q*pq;
		//return 333; 
	}else if (angle > -90.01 && angle < -89.99)
	{
		NxQuat pq;
		pq.setXYZW(0.0001f,0,0,-1);
		q = q*pq;
	}
	float xz = q.x * q.z; 
	float wy = -q.w * q.y;
	float yz = q.y * q.z; 
	float wx = -q.w * q.x; 
	float xx = q.x * q.x; 
	float yy = q.y * q.y; 
 
	float kx = 2*(xz - wy); 
	float ky = 2*(yz + wx); 
	float kz = 1 - 2 * (xx +yy); 
 
	float yaw = -NxMath::radToDeg(atan2f(kx,kz)); 
 	return yaw; 

	/*float sqw = q.w*q.w; 
	float sqx = q.x*q.x; 
	float sqy = q.y*q.y; 
	float sqz = q.z*q.z; 
	return (float) -NxMath::radToDeg(asin(-2.0 * (q.x*q.z - q.y*q.w)));*/

}
BBDECL float BBCALL pxBodyGetRotationRoll(NxActor* body)
{
	if (body->isDynamic())
	{
		q = body->getGlobalOrientationQuat();
	} else 
	{
		NxShape*const* AllShapes = body->getShapes();
		AllShapes[0]->getGlobalOrientation().toQuat(q);
	}
	q.normalize();
	float angle = pxBodyGetRotationPitch( body);
	if (angle < 90.01 && angle >89.99)
	{
		NxQuat pq;
		pq.setXYZW(0.0001f,0,0,-1);
		q = q*pq;
		//return 333; 
	}else if (angle > -90.01 && angle < -89.99)
	{
		NxQuat pq;
		pq.setXYZW(0.0001f,0,0,-1);
		q = q*pq;
	}
	float xx = q.x * q.x; 
	float zz = q.z * q.z; 
	float xy = q.x * q.y; 
	float wz = -q.w * q.z; 
	float roll = NxMath::radToDeg(atan2f(2*(xy-wz),1-2*(xx+zz)));//*180/3.14159;

 	return roll;
	/*float sqw = q.w*q.w; 
	float sqx = q.x*q.x; 
	float sqy = q.y*q.y; 
	float sqz = q.z*q.z; 
	return (float) NxMath::radToDeg(atan2f(2.0 * (q.x*q.y + q.z*q.w),(sqx - sqy - sqz + sqw)));*/

}
BBDECL float BBCALL pxBodyGetRotationYaw2(NxActor* body)
{
	NxMat33 rmat;
	NxVec3 row0, row1, row2;
	rmat = body->getGlobalOrientation();
	row0 = rmat.getColumn (0);
	row1 = rmat.getColumn (1);
	row2 = rmat.getColumn (2);

	float yaw = NxMath::radToDeg(-atan2f(row1.x,row0.x));
	return yaw+360;
}
BBDECL void BBCALL pxBodyGetRotMatrix(NxActor* body, float* rdim)
{
	NxMat33 rmat;
	NxVec3 row0, row1, row2;
	rmat = body->getGlobalOrientation();
	q = body->getGlobalOrientationQuat();
	NxQuat pq = AngleToQuat(1, 0, 0);
	row0 = rmat.getColumn (0);
	row1 = rmat.getColumn (1);
	row2 = rmat.getColumn (2);

	rdim[0] = pq.x;
	rdim[1] = pq.y;
	rdim[2] = pq.z;
	rdim[3] = pq.w;

/*	rdim[0] = row0.x;
	rdim[1] = row0.y;
	rdim[2] = row0.z;
	rdim[3] = row1.x;
	rdim[4] = row1.y;
	rdim[5] = row1.z;
	rdim[6] = row2.x;
	rdim[7] = row2.y;
	rdim[8] = row2.z;*/
}
BBDECL float BBCALL pxBodyGetLinearSpeed(NxActor* body)
{
	NxVec3 speed = body->getLinearVelocity();
	return sqrtf(speed.x*speed.x+speed.y*speed.y+speed.z*speed.z);
}
BBDECL float BBCALL pxBodyGetLinearSpeedX(NxActor* body)
{
	NxVec3 speed = body->getLinearVelocity();
	return speed.x;
}
BBDECL float BBCALL pxBodyGetLinearSpeedY(NxActor* body)
{
	NxVec3 speed = body->getLinearVelocity();
	return speed.y;
}
BBDECL float BBCALL pxBodyGetLinearSpeedZ(NxActor* body)
{
	NxVec3 speed = body->getLinearVelocity();
	return speed.z;
}
BBDECL float BBCALL pxBodyGetLinearVecSpeedX(NxActor* body)
{
	NxVec3 speed = body->getLinearVelocity();
	speed.normalize();
	return speed.x;
}
BBDECL float BBCALL pxBodyGetLinearVecSpeedY(NxActor* body)
{
	NxVec3 speed = body->getLinearVelocity();
	speed.normalize();
	return speed.y;
}
BBDECL float BBCALL pxBodyGetLinearVecSpeedZ(NxActor* body)
{
	NxVec3 speed = body->getLinearVelocity();
	speed.normalize();
	return speed.z;
}
BBDECL float BBCALL pxBodyGetLocalLinearSpeedX(NxActor* body)
{
	NxVec3 speed = body->getLinearVelocity();
	NxQuat quat = body->getGlobalOrientationQuat(); 
	quat.w=-quat.w;
	quat.rotate(speed);
	return speed.x;
}
BBDECL float BBCALL pxBodyGetLocalLinearSpeedY(NxActor* body)
{
	NxVec3 speed = body->getLinearVelocity();
	NxQuat quat = body->getGlobalOrientationQuat();
	quat.w=-quat.w;
	quat.rotate(speed);
	return speed.y;
}
BBDECL float BBCALL pxBodyGetLocalLinearSpeedZ(NxActor* body)
{
	NxVec3 speed = body->getLinearVelocity();
	NxQuat quat = body->getGlobalOrientationQuat(); 
	quat.w=-quat.w;
	quat.rotate(speed);
	return speed.z;
}
BBDECL float BBCALL pxBodyGetLocalPointSpeed(NxActor* body, float x, float y, float z)
{
	return body->getLocalPointVelocity(NxVec3(x,y,z)).magnitude();
}
BBDECL float BBCALL pxBodyGetLocalPointSpeedX(NxActor* body, float x, float y, float z)
{
	return body->getLocalPointVelocity(NxVec3(x,y,z)).x; 
}
BBDECL float BBCALL pxBodyGetLocalPointSpeedY(NxActor* body, float x, float y, float z)
{
	return body->getLocalPointVelocity(NxVec3(x,y,z)).y;
}
BBDECL float BBCALL pxBodyGetLocalPointSpeedZ(NxActor* body, float x, float y, float z)
{
	return body->getLocalPointVelocity(NxVec3(x,y,z)).z;
}
BBDECL float BBCALL pxBodyGetAngularSpeed(NxActor* body)
{
	NxVec3 speed = body->getAngularVelocity();
	return speed.magnitude();
}
BBDECL float BBCALL pxBodyGetAngularSpeedX(NxActor* body)
{
	NxVec3 speed = body->getAngularVelocity();
	return speed.x;
}
BBDECL float BBCALL pxBodyGetAngularSpeedY(NxActor* body)
{
	NxVec3 speed = body->getAngularVelocity();
	return speed.y;
}
BBDECL float BBCALL pxBodyGetAngularSpeedZ(NxActor* body)
{
	NxVec3 speed = body->getAngularVelocity();
	return speed.z;
}
BBDECL float BBCALL pxBodyGetLocalAngularSpeedX(NxActor* body)
{
	NxVec3 speed = body->getAngularVelocity();
	NxQuat quat = body->getGlobalOrientationQuat(); 
	quat.w=-quat.w;
	quat.rotate(speed);
	return speed.x;
}
BBDECL float BBCALL pxBodyGetLocalAngularSpeedY(NxActor* body)
{
	NxVec3 speed = body->getAngularVelocity();
	NxQuat quat = body->getGlobalOrientationQuat();
	quat.w=-quat.w;
	quat.rotate(speed);
	return speed.y;
}
BBDECL float BBCALL pxBodyGetLocalAngularSpeedZ(NxActor* body)
{
	NxVec3 speed = body->getAngularVelocity();
	NxQuat quat = body->getGlobalOrientationQuat(); 
	quat.w=-quat.w;
	quat.rotate(speed);
	return speed.z;
}

/*typedef struct mMATRIX
{
public:
    mMATRIX() {};
    mMATRIX( CONST FLOAT * );
    mMATRIX( CONST mMATRIX& );
    mMATRIX( FLOAT _11, FLOAT _12, FLOAT _13, FLOAT _14,
                FLOAT _21, FLOAT _22, FLOAT _23, FLOAT _24,
                FLOAT _31, FLOAT _32, FLOAT _33, FLOAT _34,
                FLOAT _41, FLOAT _42, FLOAT _43, FLOAT _44 );


    // access grants
    FLOAT& operator () ( UINT Row, UINT Col );
    FLOAT  operator () ( UINT Row, UINT Col ) const;

    // casting operators
    operator FLOAT* ();
    operator CONST FLOAT* () const;


    friend mMATRIX operator * ( FLOAT, CONST mMATRIX& );

    BOOL operator == ( CONST mMATRIX& ) const;
    BOOL operator != ( CONST mMATRIX& ) const;

} mMATRIX, *LPmMATRIX;*/
typedef struct _mMATRIX {
    union {
        struct {
            float        _11, _12, _13, _14;
            float        _21, _22, _23, _24;
            float        _31, _32, _33, _34;
            float        _41, _42, _43, _44;

        };
        float m[4][4];
    };
} mMATRIX;

BBDECL mMATRIX* BBCALL pxBodyGetMatrix(NxActor* body)
{
	mMATRIX *mat = new mMATRIX;
	body->getGlobalPose().getColumnMajor44((float*)mat);

	return mat;
}
BBDECL void BBCALL pxBodyDeleteMatrix(mMATRIX* mat)
{
	delete[] mat;
}
BBDECL float BBCALL pxBodyGetMatrixElement(mMATRIX* mat, int r, int c)
{
	return mat->m[r][c];
}
//Material
BBDECL int BBCALL pxCreateMaterial()
{
	NxMaterialDesc	material;
	material.restitution		= 0.0f;
//	material.staticFriction		= 0.5f;
	material.dynamicFriction	= 0.5f;
//	material.flags				= NX_MF_DISABLE_STRONG_FRICTION;
	NxMaterialIndex MaterialIndex = gScene->createMaterial(material)->getMaterialIndex();
	return MaterialIndex;
}
BBDECL void BBCALL pxMaterialSetToBody(NxActor* body, int mat)
{
	NxShape*const* AllShapes = body->getShapes();
	int nbAllShapes = body->getNbShapes();
	for (int i=0; i<nbAllShapes; i++)
	{
		AllShapes[i]->setMaterial(mat);
	}
}
BBDECL void BBCALL pxMaterialSetRestitution(int mat, float rest)
{
	NxMaterial* materials = gScene -> getMaterialFromIndex  ( mat ) ;
	materials->setRestitution(rest);
}
BBDECL void BBCALL pxMaterialSetRestitutionCombineMode(int mat, NxCombineMode mode)
{
	NxMaterial* materials = gScene -> getMaterialFromIndex  ( mat ) ;
	materials->setRestitutionCombineMode(mode);
}
BBDECL void BBCALL pxMaterialSetStFriction(int mat, float fric)
{
	
	NxMaterial* materials = gScene -> getMaterialFromIndex  ( mat ) ;
	materials->setStaticFriction(fric);

}

BBDECL void BBCALL pxMaterialSetDyFriction(int mat, float fric)
{
	
	NxMaterial* materials = gScene -> getMaterialFromIndex  ( mat ) ;
	materials->setDynamicFriction(fric);

}

BBDECL void BBCALL pxMaterialSetFrictionV(int mat, float sfric, float dfric)
{
	
	NxMaterial* materials = gScene -> getMaterialFromIndex  ( mat ) ;
	materials->setStaticFrictionV(sfric);
	materials->setDynamicFrictionV(dfric);

}

BBDECL void BBCALL pxMaterialSetFrictionCombineMode(int mat, NxCombineMode mode)
{
	NxMaterial* materials = gScene -> getMaterialFromIndex  ( mat ) ;
	materials->setFrictionCombineMode(mode);
}
BBDECL int BBCALL pxCreateAnisotripicMaterial(float nx, float ny, float nz)
{
	
	//anisotropic friction material
	NxMaterialDesc	material;
	material.restitution		= 0.0f;
	material.staticFriction		= 0.1f;
	material.dynamicFriction	= 0.1f;	
	material.dynamicFrictionV	= 0.8f;
	material.staticFrictionV	= 1.0f;
	material.dirOfAnisotropy.set(nx,ny,nz);	//this will be used on one of the ramps so that boxes slide off at an angle
	material.dirOfAnisotropy.normalize();
	material.flags				= NX_MF_ANISOTROPIC;
	NxMaterialIndex anisoMaterialIndex = gScene->createMaterial(material)->getMaterialIndex();

	return anisoMaterialIndex;
}
BBDECL void BBCALL pxMaterialSetFlagStrongFriction(int mat)
{
	NxMaterial* materials = gScene -> getMaterialFromIndex  ( mat ) ;
	materials->setFlags(NX_MF_DISABLE_STRONG_FRICTION);
}
BBDECL int BBCALL pxGetMaterial(NxActor* body)
{
	shapes = body-> getShapes(); 
	int IDsh = body->getNbShapes()-1;

	NxMaterialIndex material = shapes[IDsh]->getMaterial();
	return material;
}
//-----------Sahpes--------------------
BBDECL int BBCALL pxShapesGetNumber(NxActor* body)
{
	return body->getNbShapes();
}
BBDECL NxShape* BBCALL pxShapesGetShape(NxActor* body, int num)
{
	return body->getShapes()[num];
}
BBDECL float BBCALL pxShapesGetPositionX(NxShape* shape)
{
	return shape->getGlobalPosition().x;
}
BBDECL float BBCALL pxShapesGetPositionY(NxShape* shape)
{
	return shape->getGlobalPosition().y;
}
BBDECL float BBCALL pxShapesGetPositionZ(NxShape* shape)
{
	return shape->getGlobalPosition().z;
}
BBDECL float BBCALL pxShapesGetRotationPitch(NxShape* shape)
{
	shape->getGlobalOrientation().toQuat(q);

	q.normalize();
	float xz = q.x * q.z; 
	float wy = -q.w * q.y;
	float yz = q.y * q.z; 
	float wx = -q.w * q.x; 
	float xx = q.x * q.x; 
	float yy = q.y * q.y; 
 
	float kx = 2*(xz - wy); 
	float ky = 2*(yz + wx); 
	float kz = 1 - 2 * (xx +yy); 
 
	float pitch = -NxMath::radToDeg(atan2f ( ky , sqrtf(kx*kx+kz*kz)));
	
	return pitch;
}
BBDECL float BBCALL pxShapesGetRotationYaw(NxShape* shape)
{
	shape->getGlobalOrientation().toQuat(q);

	q.normalize();
	float xz = q.x * q.z; 
	float wy = -q.w * q.y;
	float yz = q.y * q.z; 
	float wx = -q.w * q.x; 
	float xx = q.x * q.x; 
	float yy = q.y * q.y; 
 
	float kx = 2*(xz - wy); 
	float ky = 2*(yz + wx); 
	float kz = 1 - 2 * (xx +yy); 
 
	float yaw = -NxMath::radToDeg(atan2f(kx,kz)); 
 	return yaw; 
}
BBDECL float BBCALL pxShapesGetRotationRoll(NxShape* shape)
{
	shape->getGlobalOrientation().toQuat(q);

	q.normalize();
	float xx = q.x * q.x; 
	float zz = q.z * q.z; 
	float xy = q.x * q.y; 
	float wz = -q.w * q.z; 
	float roll = NxMath::radToDeg(atan2f(2*(xy-wz),1-2*(xx+zz)));

 	return roll;
}
BBDECL void BBCALL pxShapesSetPosition(NxShape* shape, float pos_x, float pos_y, float pos_z)
{
	shape->setGlobalPosition(NxVec3(pos_x, pos_y, pos_z));
}
BBDECL void BBCALL pxShapesSetRotation(NxShape* shape, float pitch, float yaw, float roll)
{
	NxQuat qo = NxQuat( NxVec3( 0,NxMath::sin(NxMath::degToRad(yaw*0.5f)),0 ), -NxMath::cos(NxMath::degToRad(yaw*0.5f))) * 
		NxQuat(NxVec3( NxMath::sin(NxMath::degToRad(-pitch*0.5f)),0,0 ), -NxMath::cos(NxMath::degToRad(-pitch*0.5f)) ) * 
		NxQuat( NxVec3( 0,0,NxMath::sin(NxMath::degToRad(-roll*0.5f))), -NxMath::cos(NxMath::degToRad(-roll*0.5f)) );

	NxMat33 anhorRot;
	anhorRot.fromQuat(qo);
	shape->setLocalOrientation(anhorRot);

}
//End
BBDECL float BBCALL pxGetAngleBetweenVec(float v1x, float v1y, float v1z, float v2x, float v2y, float v2z)
{
	return NxAngle(NxVec3(v1x, v1y, v1z), NxVec3(v2x, v2y, v2z))*57.29578f;
}
//End
BBDECL int BBCALL GetNumber()
{
		number=gScene->getNbActors()-1;
		return number;
}
BBDECL void BBCALL test1(float* ent1, float* ent2, float* ent3, float* ent4,
						 float* ent5, float* ent6, float* ent7, float* ent8)
{
	NxCloth **cloth = gScene->getCloths();
	NxVec3* vert_buffer = new NxVec3[8];
	cloth[0]->getPositions(vert_buffer, sizeof(NxVec3));

	ent1[16] = vert_buffer[0].x;
	ent1[17] = vert_buffer[0].y;
	ent1[18] = vert_buffer[0].z;
	int moved = 3;
	ent1[11] = *((float*)(&moved));

	ent2[16] = vert_buffer[1].x;
	ent2[17] = vert_buffer[1].y;
	ent2[18] = vert_buffer[1].z;
	ent2[11] = *((float*)(&moved));

	ent3[16] = vert_buffer[2].x;
	ent3[17] = vert_buffer[2].y;
	ent3[18] = vert_buffer[2].z;
	ent3[11] = *((float*)(&moved));

	ent4[16] = vert_buffer[3].x;
	ent4[17] = vert_buffer[3].y;
	ent4[18] = vert_buffer[3].z;
	ent4[11] = *((float*)(&moved));

	ent5[16] = vert_buffer[4].x;
	ent5[17] = vert_buffer[4].y;
	ent5[18] = vert_buffer[4].z;
	ent5[11] = *((float*)(&moved));

	ent6[16] = vert_buffer[5].x;
	ent6[17] = vert_buffer[5].y;
	ent6[18] = vert_buffer[5].z;
	ent6[11] = *((float*)(&moved));

	ent7[16] = vert_buffer[6].x;
	ent7[17] = vert_buffer[6].y;
	ent7[18] = vert_buffer[6].z;
	ent7[11] = *((float*)(&moved));

	ent8[16] = vert_buffer[7].x;
	ent8[17] = vert_buffer[7].y;
	ent8[18] = vert_buffer[7].z;
	ent8[11] = *((float*)(&moved));

}
BBDECL float BBCALL test_tri(unsigned long *surf)
{

	LPBBTRIANGE ThisTriangle;
	ThisTriangle = (struct _BBTRIANGE *)*(surf+11);

	unsigned short int *IndexBuf, *IndexBufEnd, IndexCount;
	IndexBuf =          (unsigned short int *)*(surf+11);
	IndexBufEnd =     (unsigned short int *)*(surf+12);
	IndexCount = (IndexBufEnd-IndexBuf)/3;
	
	//*(surf+17)=0;

	return (float) IndexCount;

}
void pxSetEntityMoved(float* entity)
{
    int moved = 3;
	entity[11] = *((float*)(&moved));
	//int child_adress = entity[4]
	//int* SurfaceInfo  = ((int**)entity)[161];
    float* child = ((float**)entity)[4];
    if (child) pxSetEntityMoved(child);
 //   float* sosed = ((float**)entity)[1];
 //   if (sosed) pxSetEntityMoved(sosed);
}
BBDECL void BBCALL pxBodySetEntity(float *entity, NxActor* body) 
{
	NxVec3 pos;
	if (body->isDynamic())
	{
		pos = body->getGlobalPosition();
		q = body->getGlobalOrientationQuat();
	} else 
	{
		NxVec3 anhorPos = NxVec3(0,0,0);
		NxShape*const* AllShapes = body->getShapes();
		anhorPos = ((ShapeData*)AllShapes[0]->userData)->bpos;
		pos = AllShapes[0]->getGlobalPosition() - anhorPos;
		AllShapes[0]->getGlobalOrientation().toQuat(q);
	}

    entity[13] = q.x;
	entity[14] = q.y;
	entity[15] = q.z;
	entity[12] = -q.w;
	entity[16] = pos.x;
	entity[17] = pos.y;
	entity[18] = pos.z;
	pxSetEntityMoved(entity);
//	int moved = 3;
//	entity[11] = *((float*)(&moved));
		//return entity;
}
BBDECL void BBCALL pxDX9BodySetEntity(float *entity, NxActor* body) 
{
	NxVec3 pos;
	if (body->isDynamic())
	{
		pos = body->getGlobalPosition();
		q = body->getGlobalOrientationQuat();
	} else 
	{
		NxVec3 anhorPos = NxVec3(0,0,0);
		NxShape*const* AllShapes = body->getShapes();
		anhorPos = ((ShapeData*)AllShapes[0]->userData)->bpos;
		pos = AllShapes[0]->getGlobalPosition() - anhorPos;
		AllShapes[0]->getGlobalOrientation().toQuat(q);
	}

	entity[53] = pos.x;
	entity[54] = pos.y;
	entity[55] = pos.z;

	entity[56] = q.x;
	entity[57] = q.y;
	entity[58] = q.z;
	entity[59] = q.w;
	entity[63] = 1;
}
BBDECL float BBCALL pxReadMemoryEntity(float *entity, int num) 
{
	return entity[num];
}
BBDECL float BBCALL pxReadMemoryBody(NxActor *body, int num) 
{
	NxVec3 pos;
	if (body->isDynamic())
	{
		pos = body->getGlobalPosition();
		q = body->getGlobalOrientationQuat();
	} else 
	{
		NxVec3 anhorPos = NxVec3(0,0,0);
		NxShape*const* AllShapes = body->getShapes();
		anhorPos = ((ShapeData*)AllShapes[0]->userData)->bpos;
		pos = AllShapes[0]->getGlobalPosition() - anhorPos;
		AllShapes[0]->getGlobalOrientation().toQuat(q);
	}
	if (num==0) return q.x;
	if (num==1) return q.y;
	if (num==2) return q.z;
	if (num==3) return q.w;
	return 0;
}
BBDECL void BBCALL pxBodyAddEntity(float *entity, NxActor* body) 
{

//	number_add_actor = number_add_actor+1;
//	actors[number_add_actor-1] =  body;
//	mesh_actors[number_add_actor-1] = entity;

}

BBDECL int BBCALL peeksint(int addr)
{
     return *((int*)(addr));
}
BBDECL float* BBCALL pxBufferCreate(int size)
{
	float* buff = new float[size];
	return buff;
}
BBDECL int* BBCALL pxIntBufferCreate(int size)
{
	int* buff = new int[size];
	return buff;
}
BBDECL void BBCALL pxBufferSetElement(float* buff, int nelem, float elem)
{
	buff[nelem] = elem;
}
BBDECL float BBCALL pxBufferGetElement(float* buff, int nelem)
{
	return buff[nelem];
}
BBDECL void BBCALL pxBufferDelete(float* buff)
{
	delete[] buff;
}
int main()
{
	return 0;
}

